@@ -1,4 +1,4 @@
-/* $Id: version.h.in 1742 2008-04-16 06:30:00Z ensonic $
+/* $Id: version.h.in 3611 2011-08-27 07:53:12Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -22,29 +22,11 @@
 #ifndef BT_VERSION_H
 #define BT_VERSION_H
 
-/** 
- * BT_MAJOR_VERSION:
- *
- * buzztard version stamp, major part; determined from configure.ac
+/*
+ * buzztard version stamp; determined from configure.ac
  */
 #define BT_MAJOR_VERSION 0
-/**
- * BT_MINOR_VERSION:
- *
- * buzztard version stamp, minor part; determined from configure.ac
- */
-#define BT_MINOR_VERSION 5
-/**
- * BT_MICRO_VERSION:
- *
- * buzztard version stamp, micro part; determined from configure.ac
- */
+#define BT_MINOR_VERSION 6
 #define BT_MICRO_VERSION 0
 
-#ifndef BT_CORE_C
-extern const unsigned int bt_major_version;
-extern const unsigned int bt_minor_version;
-extern const unsigned int bt_micro_version;
-#endif
-
 #endif // BT_VERSION
