@@ -1,4 +1,4 @@
-/* $Id: song-info.c 2455 2009-06-02 20:44:57Z ensonic $
+/* $Id: song-info.c 3611 2011-08-27 07:53:12Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -27,17 +27,17 @@
  * recording a song to a file (rendering) in the form of meta-tags.
  *
  * Further there are fields that determine rythm and song-speed. The speed is
- * determined by BtSongInfo::bpm. The rythm is determined by BtSongInfo::bars
- * and BtSongInfo::tpb. If 'bars' is 16, than on can have 1/16 notes.
+ * determined by #BtSongInfo:bpm. The rythm is determined by #BtSongInfo:bars
+ * and #BtSongInfo:tpb. If 'bars' is 16, than on can have 1/16 notes.
  * And if 'ticks per beat' is 4 one will have 4 beats - a classic 4/4 meassure.
  * For a 3/4 meassure, 'bars' would be 12. Thus bars = beats * tpb.
  */
 /* @todo: add more metadata
  *  - copyright: GST_TAG_COPYRIGHT
+ *    string: "(C) YYYY <SONG_INFO_AUTHOR>"
  *  - license: GST_TAG_LICENSE (http://creativecommons.org/licenses/)
- *
- * @todo: add sample-rate and channels properties
- *  - they will be set on sink-bin
+ *    - would be nice to use liblicense (not widely packaged :/)
+ *    - we'll have liblicense support in gst-0.10.36
  */
 
 #define BT_CORE
@@ -92,14 +92,20 @@ struct _BtSongInfoPrivate {
   gchar *create_dts,*change_dts;
 };
 
-static GObjectClass *parent_class=NULL;
-
 // date time stamp format YYYY-MM-DDThh:mm:ssZ
 #define DTS_LEN 20
 
 /* default name for new songs */
 #define DEFAULT_SONG_NAME _("untitled song")
 
+//-- the class
+
+static void bt_song_info_persistence_interface_init(gpointer const g_iface, gpointer const iface_data);
+
+G_DEFINE_TYPE_WITH_CODE (BtSongInfo, bt_song_info, G_TYPE_OBJECT,
+  G_IMPLEMENT_INTERFACE (BT_TYPE_PERSISTENCE,
+    bt_song_info_persistence_interface_init));
+
 //-- helper methods
 
 /*
@@ -138,7 +144,7 @@ BtSongInfo *bt_song_info_new(const BtSong * const song) {
 
 //-- io interface
 
-static xmlNodePtr bt_song_info_persistence_save(const BtPersistence * const persistence, xmlNodePtr const parent_node, const BtPersistenceSelection * const selection) {
+static xmlNodePtr bt_song_info_persistence_save(const BtPersistence * const persistence, xmlNodePtr const parent_node) {
   const BtSongInfo * const self = BT_SONG_INFO(persistence);
   xmlNodePtr node=NULL;
 
@@ -150,13 +156,13 @@ static xmlNodePtr bt_song_info_persistence_save(const BtPersistence * const pers
       gchar *file_path,*file_name,*ext;
       
       g_object_get(self->priv->song,"song-io",&song_io,NULL);
-      g_object_get(G_OBJECT(song_io),"file-name",&file_path,NULL);
+      g_object_get(song_io,"file-name",&file_path,NULL);
       file_name=g_path_get_basename(file_path);
       if((ext=strrchr(file_name,'.'))) {
         *ext='\0';
       }
       GST_INFO("using '%s' instead of default title",file_name);  
-      g_object_set(G_OBJECT(self),"name",file_name,NULL);
+      g_object_set((gpointer)self,"name",file_name,NULL);
       g_free(file_name);
       g_free(file_path);
       g_object_unref(song_io);
@@ -187,7 +193,7 @@ static xmlNodePtr bt_song_info_persistence_save(const BtPersistence * const pers
   return(node);
 }
 
-static BtPersistence *bt_song_info_persistence_load(const GType type, const BtPersistence * const persistence, xmlNodePtr node, const BtPersistenceLocation * const location, GError **err, va_list var_args) {
+static BtPersistence *bt_song_info_persistence_load(const GType type, const BtPersistence * const persistence, xmlNodePtr node, GError **err, va_list var_args) {
   const BtSongInfo * const self = BT_SONG_INFO(persistence);
 
   GST_DEBUG("PERSISTENCE::song-info");
@@ -209,12 +215,12 @@ static BtPersistence *bt_song_info_persistence_load(const GType type, const BtPe
             !strncmp(property_name,"create-dts",10) ||
             !strncmp(property_name,"change-dts",10)
           ) {
-            g_object_set(G_OBJECT(self),property_name,elem,NULL);
+            g_object_set((gpointer)self,property_name,elem,NULL);
           }
           else if(!strncmp(property_name,"bpm",3) ||
             !strncmp(property_name,"tpb",3) ||
             !strncmp(property_name,"bars",4)) {
-            g_object_set(G_OBJECT(self),property_name,atol((char *)elem),NULL);
+            g_object_set((gpointer)self,property_name,atol((char *)elem),NULL);
           }
           xmlFree(elem);
         }
@@ -235,17 +241,6 @@ static void bt_song_info_persistence_interface_init(gpointer const g_iface, gpoi
 
 //-- g_object overrides
 
-#if 0
-static void bt_setup_constructed(GObject *object) {
-  BtSongInfo *self=BT_SONG_INFO(object);
-  
-  if(G_OBJECT_CLASS(parent_class)->constructed)
-    G_OBJECT_CLASS(parent_class)->constructed(object);
-
-  g_return_val_if_fail(BT_IS_SONG(self->priv->song));
-}
-#endif
-
 /* returns a property for the given property_id for this object */
 static void bt_song_info_get_property(GObject * const object, const guint property_id, GValue * const value, GParamSpec * const pspec) {
   const BtSongInfo * const self = BT_SONG_INFO(object);
@@ -320,7 +315,6 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
           self->priv->info=NULL;
           gst_tag_list_remove_tag(self->priv->taglist,GST_TAG_DESCRIPTION);
         }
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the info for song_info: %s",self->priv->info);
       }
     } break;
@@ -336,7 +330,6 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
           self->priv->name=NULL;
           gst_tag_list_remove_tag(self->priv->taglist,GST_TAG_TITLE);
         }
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the name for song_info: %s",self->priv->name);
       }
     } break;
@@ -352,7 +345,6 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
           self->priv->genre=NULL;
           gst_tag_list_remove_tag(self->priv->taglist,GST_TAG_GENRE);
         }
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the genre for song_info: %s",self->priv->genre);
       }
     } break;
@@ -368,7 +360,6 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
           self->priv->author=NULL;
           gst_tag_list_remove_tag(self->priv->taglist,GST_TAG_ARTIST);
         }
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the author for song_info: %s",self->priv->author);
       }
     } break;
@@ -376,11 +367,10 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
       gulong val=g_value_get_ulong(value);
       if(self->priv->beats_per_minute!=val) {
         self->priv->beats_per_minute = g_value_get_ulong(value);
-#if (GST_VERSION_MAJOR>=0 && GST_VERSION_MINOR>=10 && GST_VERSION_MICRO>=12) || (GST_VERSION_MAJOR>=0 && GST_VERSION_MINOR>=10 && GST_VERSION_MICRO>=11 && GST_VERSION_NANO>0 )
+#if GST_CHECK_VERSION(0,10,12)
         gst_tag_list_add(self->priv->taglist, GST_TAG_MERGE_REPLACE,GST_TAG_BEATS_PER_MINUTE, (gdouble)self->priv->beats_per_minute,NULL);
 #endif
         bt_song_info_tempo_changed(self);
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the bpm for song_info: %lu",self->priv->beats_per_minute);
       }
     } break;
@@ -389,7 +379,6 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
       if(self->priv->ticks_per_beat!=val) {
         self->priv->ticks_per_beat = g_value_get_ulong(value);
         bt_song_info_tempo_changed(self);
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the tpb for song_info: %lu",self->priv->ticks_per_beat);
       }
     } break;
@@ -398,7 +387,6 @@ static void bt_song_info_set_property(GObject * const object, const guint proper
       if(self->priv->bars!=val) {
         self->priv->bars = g_value_get_ulong(value);
         bt_song_info_tempo_changed(self);
-        bt_song_set_unsaved(self->priv->song,TRUE);
         GST_DEBUG("set the bars for song_info: %lu",self->priv->bars);
       }
     } break;
@@ -462,7 +450,7 @@ static void bt_song_info_dispose(GObject * const object) {
   GST_DEBUG("!!!! self=%p",self);
   g_object_try_weak_unref(self->priv->song);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_song_info_parent_class)->dispose(object);
 }
 
 static void bt_song_info_finalize(GObject * const object) {
@@ -481,13 +469,12 @@ static void bt_song_info_finalize(GObject * const object) {
   g_free(self->priv->create_dts);
   g_free(self->priv->change_dts);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(bt_song_info_parent_class)->finalize(object);
 }
 
 //-- class internals
 
-static void bt_song_info_init(GTypeInstance * const instance, gconstpointer g_class) {
-  BtSongInfo * const self = BT_SONG_INFO(instance);
+static void bt_song_info_init(BtSongInfo * self) {
   time_t now=time(NULL);
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_SONG_INFO, BtSongInfoPrivate);
@@ -515,14 +502,17 @@ static void bt_song_info_init(GTypeInstance * const instance, gconstpointer g_cl
 #endif
   gst_tag_list_add(self->priv->taglist, GST_TAG_MERGE_REPLACE,
     GST_TAG_TITLE, self->priv->name,
+    GST_TAG_ARTIST, self->priv->author,
     GST_TAG_DATE, self->priv->tag_date,
+#if GST_CHECK_VERSION(0,10,12)
+    GST_TAG_BEATS_PER_MINUTE, (gdouble)self->priv->beats_per_minute,
+#endif
     NULL);
 }
 
 static void bt_song_info_class_init(BtSongInfoClass * const klass) {
   GObjectClass * const gobject_class = G_OBJECT_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtSongInfoPrivate));
 
   gobject_class->set_property = bt_song_info_set_property;
@@ -590,7 +580,7 @@ static void bt_song_info_class_init(BtSongInfoClass * const klass) {
   g_object_class_install_property(gobject_class,SONG_INFO_TPB,
                                   g_param_spec_ulong("tpb",
                                      "tpb prop",
-                                     "how many event fire in one fraction of a beat",
+                                     "event granularity in one beat",
                                      1,
                                      128,
                                      4,
@@ -620,28 +610,3 @@ static void bt_song_info_class_init(BtSongInfoClass * const klass) {
                                      G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 }
 
-GType bt_song_info_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtSongInfoClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_song_info_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtSongInfo),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_song_info_init, // instance_init
-      NULL // value_table
-    };
-    const GInterfaceInfo persistence_interface_info = {
-      (GInterfaceInitFunc) bt_song_info_persistence_interface_init,  // interface_init
-      NULL, // interface_finalize
-      NULL  // interface_data
-    };
-    type = g_type_register_static(G_TYPE_OBJECT,"BtSongInfo",&info,0);
-    g_type_add_interface_static(type, BT_TYPE_PERSISTENCE, &persistence_interface_info);
-  }
-  return type;
-}
