@@ -1,4 +1,4 @@
-/* $Id: song.c 2549 2009-09-03 07:13:16Z ensonic $
+/* $Id: song.c 3767 2011-12-08 19:55:28Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -27,10 +27,14 @@
  *
  * To load or save a song, use a #BtSongIO object. These implement loading and
  * saving for different file-formats.
+ *
+ * One can seek in a song by setting the #BtSong:play-pos property. Likewise one
+ * can watch the property to display the playback position.
+ *
+ * The #BtSong:play-rate property can be used to change the playback speed and
+ * direction.
  */
-/* @todo: idle looping (needed for playing machines live)
- * - add an "is-idle" property
- * - make bt_song_idle_{start,stop} static
+/* idle looping (needed for playing machines live)
  * - states:
  *   is_playing is_idle
  *   FALSE      FALSE    no playback
@@ -59,6 +63,10 @@
  *       bt_song_idle_stop();
  *   }
  */
+/* fast seeking
+ * - see bt_song_change_play_rate()
+ * - there is a problem with wrapping around in loop mode, when playing backwards
+ */
 #define BT_CORE
 #define BT_SONG_C
 
@@ -94,8 +102,8 @@ enum {
   SONG_SEQUENCE,
   SONG_SETUP,
   SONG_WAVETABLE,
-  SONG_UNSAVED,
   SONG_PLAY_POS,
+  SONG_PLAY_RATE,
   SONG_IS_PLAYING,
   SONG_IS_IDLE,
   SONG_IO
@@ -111,11 +119,10 @@ struct _BtSongPrivate {
   BtSetup*     setup;
   BtWavetable* wavetable;
 
-  /* whenever the song is changed, unsave should be set TRUE */
-  gboolean unsaved;
-
   /* the playback position of the song */
   gulong play_pos,play_end;
+  gdouble play_rate;
+
   /* flag to signal playing and idle states */
   gboolean is_playing,is_preparing;
   gboolean is_idle,is_idle_active;
@@ -144,12 +151,64 @@ struct _BtSongPrivate {
   BtSongIO *song_io;
 };
 
-static GObjectClass *parent_class=NULL;
+//-- the class
+
+static void bt_song_persistence_interface_init(gpointer const g_iface, gpointer const iface_data);
+
+G_DEFINE_TYPE_WITH_CODE (BtSong, bt_song, G_TYPE_OBJECT,
+  G_IMPLEMENT_INTERFACE (BT_TYPE_PERSISTENCE,
+    bt_song_persistence_interface_init));
 
-//static guint signals[LAST_SIGNAL]={0,};
 
 //-- helper
 
+static void bt_song_update_play_seek_event(const BtSong * const self) {
+  gboolean loop;
+  glong loop_start,loop_end,length;
+  gulong play_pos=self->priv->play_pos;
+
+  g_object_get(self->priv->sequence,"loop",&loop,"loop-start",&loop_start,"loop-end",&loop_end,"length",&length,NULL);
+  const GstClockTime bar_time=bt_sequence_get_bar_time(self->priv->sequence);
+
+  //GST_INFO("rate %lf, loop %d? %ld ... %ld, length %ld, pos %lu",self->priv->play_rate,loop,loop_start,loop_end,length,play_pos);
+
+  if(loop_start==-1) loop_start=0;
+  if(loop_end==-1) loop_end=length+1;
+  if(play_pos>=loop_end) play_pos=loop_start;
+  
+  // remember end for eos
+  self->priv->play_end=loop_end;
+  
+  GST_INFO("loop %d? %ld ... %ld, length %ld, pos %lu, bar_time %"GST_TIME_FORMAT,
+    loop,loop_start,loop_end,length,play_pos,GST_TIME_ARGS(bar_time));
+
+  if(self->priv->play_seek_event) gst_event_unref(self->priv->play_seek_event);
+  if(self->priv->loop_seek_event) gst_event_unref(self->priv->loop_seek_event);
+    
+  /* we need to use FLUSH for play (due to prerolling), otherwise:
+     0:00:00.866899000 15884 0x81cee70 DEBUG             basesink gstbasesink.c:1644:gst_base_sink_do_sync:<player> prerolling object 0x818ce90
+     0:00:00.866948000 15884 0x81cee70 DEBUG             basesink gstbasesink.c:1493:gst_base_sink_wait_preroll:<player> waiting in preroll for flush or PLAYING
+     but not for loop
+   */
+  if (loop) {
+    self->priv->play_seek_event = gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+        GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT,
+        GST_SEEK_TYPE_SET, play_pos*bar_time,
+        GST_SEEK_TYPE_SET, loop_end*bar_time);
+    self->priv->loop_seek_event = gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+        GST_SEEK_FLAG_SEGMENT,
+        GST_SEEK_TYPE_SET, loop_start*bar_time,
+        GST_SEEK_TYPE_SET, loop_end*bar_time);
+  }
+  else {
+    self->priv->play_seek_event = gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+        GST_SEEK_FLAG_FLUSH,
+        GST_SEEK_TYPE_SET, play_pos*bar_time,
+        GST_SEEK_TYPE_SET, loop_end*bar_time);
+    self->priv->loop_seek_event = NULL;
+  }
+}
+
 /*
  * bt_song_seek_to_play_pos:
  * @self: #BtSong to seek
@@ -162,14 +221,15 @@ static void bt_song_seek_to_play_pos(const BtSong * const self) {
   gboolean loop;
   glong loop_end,length;
 
-  if(!self->priv->is_playing) return;
+  if(!self->priv->is_playing)
+    return;
 
   g_object_get(self->priv->sequence,"loop",&loop,"loop-end",&loop_end,"length",&length,NULL);
   const GstClockTime bar_time=bt_sequence_get_bar_time(self->priv->sequence);
 
   GST_INFO("loop %d? %ld, length %ld, bar_time %"GST_TIME_FORMAT,loop,loop_end,length,GST_TIME_ARGS(bar_time));
 
-  // we need to flush, otheriwse mixing goes out of sync */
+  // we need to flush, otherwise mixing goes out of sync
   if (loop) {
     event = gst_event_new_seek(1.0, GST_FORMAT_TIME,
         GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT,
@@ -183,62 +243,77 @@ static void bt_song_seek_to_play_pos(const BtSong * const self) {
         GST_SEEK_TYPE_SET, (length+1)*bar_time);
   }
   if(!(gst_element_send_event(GST_ELEMENT(self->priv->master_bin),event))) {
-      GST_WARNING("element failed to seek to play_pos event");
+    GST_WARNING("element failed to seek to play_pos event");
   }
 }
 
+static void bt_song_change_play_rate(const BtSong * const self) {
+  GstEvent *event;
+  gboolean loop;
+  glong loop_start,loop_end,length;
+
+  if(!self->priv->is_playing) return;
+
+  g_object_get(self->priv->sequence,"loop",&loop,"loop-start",&loop_start,"loop-end",&loop_end,"length",&length,NULL);
+  const GstClockTime bar_time=bt_sequence_get_bar_time(self->priv->sequence);
+
+  GST_INFO("rate %lf, loop %d?",self->priv->play_rate,loop);
+  bt_song_update_play_seek_event(self);
+
+  /* in 0.10.24 we do have step events (gst_event_new_step), they require an
+   * initial seek if the direction has changed */
+
+  // changing the playback rate should mostly affect sinks
+  // still we need to flush to avoid adder locking up (fixed in 0.10.35.1 - no?)
+  // and we need to give the position to workaround basesrc starting from 0
+  if (loop) {
+    if(self->priv->play_rate>0.0) {
+      event=gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+          GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT,
+          //GST_SEEK_FLAG_SEGMENT,
+          GST_SEEK_TYPE_SET, self->priv->play_pos*bar_time,
+          GST_SEEK_TYPE_SET, loop_end*bar_time);
+    }
+    else {
+      event=gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+          GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT,
+          //GST_SEEK_FLAG_SEGMENT,
+          GST_SEEK_TYPE_SET, loop_start*bar_time,
+          GST_SEEK_TYPE_SET, self->priv->play_pos*bar_time);
+    }
+  }
+  else {
+    if(self->priv->play_rate>0.0) {
+      event=gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+          GST_SEEK_FLAG_FLUSH,
+          //GST_SEEK_FLAG_NONE,
+          GST_SEEK_TYPE_SET, self->priv->play_pos*bar_time,
+          GST_SEEK_TYPE_SET, (length+1)*bar_time);
+    }
+    else {
+      event=gst_event_new_seek(self->priv->play_rate, GST_FORMAT_TIME,
+          GST_SEEK_FLAG_FLUSH,
+          //GST_SEEK_FLAG_NONE,
+          GST_SEEK_TYPE_SET, G_GINT64_CONSTANT(0),
+          GST_SEEK_TYPE_SET, self->priv->play_pos*bar_time);
+    }
+  }
+  if(!(gst_element_send_event(GST_ELEMENT(self->priv->master_bin),event))) {
+    GST_WARNING("element failed to change playback rate");
+  }
+  GST_INFO("rate updated");
+}
+
 /*
- * bt_song_update_play_seek_event:
+ * bt_song_update_play_seek_event_and_play_pos:
  * @self: #BtSong to seek
  *
  * Prepares a new playback segment, that goes from the new start position (loop
  * or song start) to the new end position (loop or song end).
  * Also calls bt_song_seek_to_play_pos() to update the current playback segment.
  */
-static void bt_song_update_play_seek_event(const BtSong * const self) {
-  gboolean loop;
-  glong loop_start,loop_end,length;
-  gulong play_pos=self->priv->play_pos;
-
-  g_object_get(self->priv->sequence,"loop",&loop,"loop-start",&loop_start,"loop-end",&loop_end,"length",&length,NULL);
-  const GstClockTime bar_time=bt_sequence_get_bar_time(self->priv->sequence);
-
-  //GST_INFO("loop %d? %ld ... %ld, length %ld, pos %lu", loop,loop_start,loop_end,length,play_pos);
-
-  if(loop_start==-1) loop_start=0;
-  if(loop_end==-1) loop_end=length;
-  if(play_pos>=loop_end) play_pos=loop_start;
-  
-  // remember end for eos
-  self->priv->play_end=loop_end;
-  
-  GST_INFO("loop %d? %ld ... %ld, length %ld, pos %lu, bar_time %"GST_TIME_FORMAT,
-    loop,loop_start,loop_end,length,play_pos,GST_TIME_ARGS(bar_time));
-
-  if(self->priv->play_seek_event) gst_event_unref(self->priv->play_seek_event);
-  if(self->priv->loop_seek_event) gst_event_unref(self->priv->loop_seek_event);
-  /* we need to use FLUSH for play (due to prerolling), otherwise:
-     0:00:00.866899000 15884 0x81cee70 DEBUG             basesink gstbasesink.c:1644:gst_base_sink_do_sync:<player> prerolling object 0x818ce90
-     0:00:00.866948000 15884 0x81cee70 DEBUG             basesink gstbasesink.c:1493:gst_base_sink_wait_preroll:<player> waiting in preroll for flush or PLAYING
-     but not for loop
-   */
-  if (loop) {
-    self->priv->play_seek_event = gst_event_new_seek(1.0, GST_FORMAT_TIME,
-        GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT,
-        GST_SEEK_TYPE_SET, play_pos*bar_time,
-        GST_SEEK_TYPE_SET, loop_end*bar_time);
-    self->priv->loop_seek_event = gst_event_new_seek(1.0, GST_FORMAT_TIME,
-        GST_SEEK_FLAG_SEGMENT,
-        GST_SEEK_TYPE_SET, loop_start*bar_time,
-        GST_SEEK_TYPE_SET, loop_end*bar_time);
-  }
-  else {
-    self->priv->play_seek_event = gst_event_new_seek(1.0, GST_FORMAT_TIME,
-        GST_SEEK_FLAG_FLUSH,
-        GST_SEEK_TYPE_SET, play_pos*bar_time,
-        GST_SEEK_TYPE_SET, loop_end*bar_time);
-    self->priv->loop_seek_event = NULL;
-  }
+static void bt_song_update_play_seek_event_and_play_pos(const BtSong * const self) {
+  bt_song_update_play_seek_event(self);
   /* the update needs to take the current play-position into account */
   bt_song_seek_to_play_pos(self);
 }
@@ -307,10 +382,10 @@ static void bt_song_send_tags(const BtSong * const self) {
         break;
     }
   }
-  gst_iterator_free (it);
+  gst_iterator_free(it);
   gst_event_unref(tag_event);
 #endif
-  gst_tag_list_free(taglist);
+  gst_element_found_tags(GST_ELEMENT(self->priv->bin),taglist);
 }
 
 
@@ -328,9 +403,17 @@ static void on_song_segment_done(const GstBus * const bus, const GstMessage * co
   
   gst_message_parse_segment_done((GstMessage *)message,&format,&position);
 #endif
+#if 0
+  /* time it takes from emitting to handling the event, depends on a change in
+   * gstreamer core to timestamp messages */
+  {
+    GstClockTime delta=GST_CLOCK_DIFF(GST_MESSAGE_TIMESTAMP (message),gst_util_get_timestamp());
+    GST_WARNING("received SEGMENT_DONE after=%"GST_TIME_FORMAT,GST_TIME_ARGS(delta));
+  }
+#endif
 
 #if GST_CHECK_VERSION(0,10,22)
-  GST_WARNING("received SEGMENT_DONE (%u) bus message: %p, from %s, with fmt=%s, ts=%"GST_TIME_FORMAT,
+  GST_INFO("received SEGMENT_DONE (%u) bus message: %p, from %s, with fmt=%s, ts=%"GST_TIME_FORMAT,
     seek_seqnum,message,GST_OBJECT_NAME(GST_MESSAGE_SRC(message)),
     gst_format_get_name(format),GST_TIME_ARGS(position));
 #else
@@ -376,7 +459,7 @@ static void on_song_segment_done(const GstBus * const bus, const GstMessage * co
       GST_WARNING("element failed to handle continuing play seek event");
     }
     else {
-      GST_WARNING("-> loop");
+      GST_INFO("-> loop");
       /*
       gst_pipeline_set_new_stream_time (GST_PIPELINE (self->priv->bin), 0);
       gst_element_get_state (GST_ELEMENT (self->priv->bin), NULL, NULL, 40 * GST_MSECOND);
@@ -391,7 +474,7 @@ static void on_song_segment_done(const GstBus * const bus, const GstMessage * co
 static void on_song_eos(const GstBus * const bus, const GstMessage * const message, gconstpointer user_data) {
   const BtSong * const self = BT_SONG(user_data);
 
-  GST_INFO("received EOS bus message from: %s", 
+  GST_INFO("received EOS bus message from: %s",
     GST_OBJECT_NAME (GST_MESSAGE_SRC (message)));
 
   if(GST_MESSAGE_SRC(message) == GST_OBJECT(self->priv->bin)) {
@@ -416,18 +499,17 @@ static gboolean on_song_paused_timeout(gpointer user_data) {
 static void on_song_state_changed(const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
   const BtSong * const self = BT_SONG(user_data);
 
-  g_assert(user_data);
   //GST_INFO("user_data=%p,<%s>, bin=%p, msg->src=%p,<%s>",
   //  user_data, G_OBJECT_TYPE_NAME(G_OBJECT(user_data)),
   //  self->priv->bin,GST_MESSAGE_SRC(message),G_OBJECT_TYPE_NAME(GST_MESSAGE_SRC(message)));
   
   if(self->priv->is_idle_active) return;
-  
+
   if(GST_MESSAGE_SRC(message) == GST_OBJECT(self->priv->bin)) {
     GstState oldstate,newstate,pending;
 
     gst_message_parse_state_changed(message,&oldstate,&newstate,&pending);
-    GST_INFO("state change on the bin: %s -> %s", 
+    GST_INFO("state change on the bin: %s -> %s",
       gst_element_state_get_name(oldstate),gst_element_state_get_name(newstate));
     switch(GST_STATE_TRANSITION(oldstate,newstate)) {
 #ifndef USE_READY_FOR_STOPPED
@@ -445,9 +527,9 @@ static void on_song_state_changed(const GstBus * const bus, GstMessage *message,
         // this should be sequence->play_start
         self->priv->play_pos=0;
         // seek to start time
-        GST_DEBUG("seek event");
+        GST_DEBUG_OBJECT(self->priv->master_bin,"seek event: %"GST_PTR_FORMAT,self->priv->play_seek_event);
         if(!(gst_element_send_event(GST_ELEMENT(self->priv->master_bin),gst_event_ref(self->priv->play_seek_event)))) {
-          GST_WARNING("bin failed to handle seek event");
+          GST_WARNING_OBJECT(self->priv->master_bin, "bin failed to handle seek event");
         }
         break;
 #endif
@@ -473,19 +555,19 @@ static void on_song_state_changed(const GstBus * const bus, GstMessage *message,
 
 static void on_song_async_done(const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
   const BtSong * const self = BT_SONG(user_data);
-
-  g_assert(user_data);
   
   if(GST_MESSAGE_SRC(message) == GST_OBJECT(self->priv->bin)) {
     GST_INFO("async state-change done");
+    if(self->priv->paused_timeout_id) {
+      g_source_remove(self->priv->paused_timeout_id);
+      self->priv->paused_timeout_id=0;
+    }
     bt_song_update_playback_position(self);
   }
 }
 
-static void on_song_clock_lost (const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
+static void on_song_clock_lost(const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
   const BtSong * const self = BT_SONG(user_data);
-
-  g_assert(user_data);
   
   if(GST_MESSAGE_SRC(message) == GST_OBJECT(self->priv->bin)) {
     GST_INFO("clock·lost!·PAUSE·and·PLAY·to·select·a·new·clock");
@@ -496,23 +578,23 @@ static void on_song_clock_lost (const GstBus * const bus, GstMessage *message, g
 }
 
 static void bt_song_on_loop_changed(BtSequence * const sequence, GParamSpec * const arg, gconstpointer user_data) {
-  bt_song_update_play_seek_event(BT_SONG(user_data));
+  bt_song_update_play_seek_event_and_play_pos(BT_SONG(user_data));
 }
 
 static void bt_song_on_loop_start_changed(BtSequence * const sequence, GParamSpec * const arg, gconstpointer user_data) {
-  bt_song_update_play_seek_event(BT_SONG(user_data));
+  bt_song_update_play_seek_event_and_play_pos(BT_SONG(user_data));
 }
 
 static void bt_song_on_loop_end_changed(BtSequence * const sequence, GParamSpec * const arg, gconstpointer user_data) {
-  bt_song_update_play_seek_event(BT_SONG(user_data));
+  bt_song_update_play_seek_event_and_play_pos(BT_SONG(user_data));
 }
 
 static void bt_song_on_length_changed(BtSequence * const sequence, GParamSpec * const arg, gconstpointer user_data) {
-  bt_song_update_play_seek_event(BT_SONG(user_data));
+  bt_song_update_play_seek_event_and_play_pos(BT_SONG(user_data));
 }
 
 static void bt_song_on_tempo_changed(BtSongInfo * const song_info, GParamSpec * const arg, gconstpointer user_data) {
-  bt_song_update_play_seek_event(BT_SONG(user_data));
+  bt_song_update_play_seek_event_and_play_pos(BT_SONG(user_data));
 }
 
 
@@ -618,30 +700,12 @@ BtSong *bt_song_new(const BtApplication * const app) {
 
 //-- methods
 
-/**
- * bt_song_set_unsaved:
- * @self: the song that should be changed
- * @unsaved: the new state of the songs unsaved flag
- *
- * Use this method instead of directly setting the state via g_object_set() to
- * avoid double notifies, if the state is unchanged.
- */
-void bt_song_set_unsaved(const BtSong * const self, const gboolean unsaved) {
-  g_return_if_fail(BT_IS_SONG(self));
-
-  if(self->priv->unsaved!=unsaved) {
-    self->priv->unsaved=unsaved;
-    GST_INFO("unsaved = %d",unsaved);
-    g_object_notify(G_OBJECT(self),"unsaved");
-  }
-}
-
 /**
  * bt_song_play:
  * @self: the song that should be played
  *
  * Starts to play the specified song instance from beginning.
- * This methods emits the "play" signal.
+ * This methods toggles the #BtSong:is-playing property.
  *
  * Returns: %TRUE for success
  */
@@ -657,7 +721,7 @@ gboolean bt_song_play(const BtSong * const self) {
 
   GST_INFO("prepare playback");
   // update play-pos
-  bt_song_update_play_seek_event(self);
+  bt_song_update_play_seek_event_and_play_pos(self);
 
 #ifdef USE_READY_FOR_STOPPED
   if((res=gst_element_set_state(GST_ELEMENT(self->priv->bin),GST_STATE_READY))==GST_STATE_CHANGE_FAILURE) {
@@ -681,20 +745,22 @@ gboolean bt_song_play(const BtSong * const self) {
   bt_song_send_tags(self);
 
   res=gst_element_set_state(GST_ELEMENT(self->priv->bin),GST_STATE_PLAYING);
-  GST_DEBUG("->PAUSED state change returned '%s'",gst_element_state_change_return_get_name(res));
+  GST_DEBUG("->PLAYING state change returned '%s'",gst_element_state_change_return_get_name(res));
   switch(res) {
+    case GST_STATE_CHANGE_SUCCESS:
+      break;
     case GST_STATE_CHANGE_FAILURE:
       GST_WARNING("can't go to paused state");
-      bt_machine_dbg_print_parts(BT_MACHINE(self->priv->master));
+      bt_song_write_to_lowlevel_dot_file(self);
       return(FALSE);
       break;
     case GST_STATE_CHANGE_ASYNC:
-      GST_INFO("->PAUSED needs async wait");
+      GST_INFO("->PLAYING needs async wait");
       // start a short timeout that aborts playback if if get not started
       self->priv->paused_timeout_id=g_timeout_add(BT_SONG_STATE_CHANGE_TIMEOUT, on_song_paused_timeout, (gpointer)self);
       break;
     default:
-      GST_WARNING("unexpected state-change-return %d",res);
+      GST_WARNING("unexpected state-change-return %d:%s",res,gst_element_state_change_return_get_name(res));
       break;
   }
   GST_INFO("playback started");
@@ -716,7 +782,7 @@ gboolean bt_song_stop(const BtSong * const self) {
 
   GST_INFO("stopping playback, is_playing: %d, is_preparing: %d",self->priv->is_playing,self->priv->is_preparing);
 
-  if(!(self->priv->is_preparing || self->priv->is_playing)) {
+  if(!(self->priv->is_preparing || self->priv->is_playing) && (GST_STATE(self->priv->bin)==GST_STATE_NULL)) {
     GST_INFO("not playing");
     return(TRUE);
   }
@@ -732,6 +798,12 @@ gboolean bt_song_stop(const BtSong * const self) {
   GST_DEBUG("->NULL state change returned '%s'",gst_element_state_change_return_get_name(res));
 #endif
 
+  // kill a pending timeout
+  if(self->priv->paused_timeout_id) {
+    g_source_remove(self->priv->paused_timeout_id);
+    self->priv->paused_timeout_id=0;
+  }
+
   // do not stop if not playing or not preparing
   if(self->priv->is_preparing) {
     self->priv->is_preparing=FALSE;
@@ -781,14 +853,14 @@ gboolean bt_song_continue(const BtSong * const self) {
  * bt_song_update_playback_position:
  * @self: the song that should update its playback-pos counter
  *
- * Updates the playback-position counter to fire all notify::playback-pos
+ * Updates the playback-position counter to fire all #BtSong:play-pos notify
  * handlers.
  *
  * Returns: %FALSE if the song is not playing
  */
 gboolean bt_song_update_playback_position(const BtSong * const self) {
   g_return_val_if_fail(BT_IS_SONG(self),FALSE);
-  g_assert(GST_IS_BIN(self->priv->bin));
+  g_assert(GST_IS_BIN(self->priv->master_bin));
   g_assert(GST_IS_QUERY(self->priv->position_query));
   //GST_INFO("query playback-pos");
 
@@ -815,46 +887,11 @@ gboolean bt_song_update_playback_position(const BtSong * const self) {
   else {
     GST_WARNING("query playback-pos: failed");
   }
+  // don't return FALSE in the WARNING case above, we use the return value to
+  // return from time-out handlers
   return(TRUE);
 }
 
-/**
- * bt_song_write_to_xml_file:
- * @self: the song that should be written
- *
- * To aid debugging applications one can use this method to write out the whole
- * network of gstreamer elements that form the song into an XML file.
- * The file will be written to '/tmp' and will be named according the 'name'
- * property of the #BtSongInfo.
- * This XML file can be loaded into gst-editor.
- */
-void bt_song_write_to_xml_file(const BtSong * const self) {
-#ifndef GST_DISABLE_LOADSAVE
-  FILE *out;
-  BtSongInfo * const song_info;
-  gchar * const song_name;
-  char ts[10];
-  time_t t;
-
-  g_return_if_fail(BT_IS_SONG(self));
-
-  g_object_get(G_OBJECT(self),"song-info",&song_info,NULL);
-  g_object_get(song_info,"name",&song_name,NULL);
-  gchar * const file_name=g_alloca(strlen(song_name)+20);
-  // not overwrite files during a run by adding current time
-  t = time(NULL);
-  strftime(ts, sizeof(ts), "%T", localtime(&t));
-  g_sprintf(file_name,"/tmp/%s_%s.xml",song_name,ts);
-
-  if((out=fopen(file_name,"wb"))) {
-    gst_xml_write_file(GST_ELEMENT(self->priv->bin),out);
-    fclose(out);
-  }
-  g_free(song_name);
-  g_object_unref(song_info);
-#endif
-}
-
 /**
  * bt_song_write_to_highlevel_dot_file:
  * @self: the song that should be written
@@ -878,13 +915,6 @@ void bt_song_write_to_highlevel_dot_file(const BtSong * const self) {
   gchar * const file_name=g_alloca(strlen(song_name)+10);
   g_sprintf(file_name,"/tmp/%s_highlevel.dot",song_name);
 
-  /* @idea: improve dot output
-   * - make border of main element in machine black, other borders gray
-   * - get caps for each wire segment
-   *   - use colors for float/int
-   *   - use line style for mono/stereo/quadro
-   */
-
   if((out=fopen(file_name,"wb"))) {
     GList * const list,*node,*sublist,*subnode;
     BtMachine * const src,* const dst;
@@ -1024,6 +1054,8 @@ void bt_song_write_to_lowlevel_dot_file(const BtSong * const self) {
   g_object_get(self->priv->song_info,"name",&song_name,NULL);
   g_strcanon(song_name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-_", '_');
 
+  // @todo: check g_getenv("GST_DEBUG_DUMP_DOT_DIR") for the path
+  // and skip if ""
   gchar * const file_name=g_alloca(strlen(song_name)+10);
   g_sprintf(file_name,"/tmp/%s_lowlevel.dot",song_name);
 
@@ -1225,7 +1257,7 @@ bt_song_child_proxy_init(gpointer g_iface, gpointer iface_data)
 
 //-- io interface
 
-static xmlNodePtr bt_song_persistence_save(const BtPersistence * const persistence, xmlNodePtr const parent_node, const BtPersistenceSelection * const selection) {
+static xmlNodePtr bt_song_persistence_save(const BtPersistence * const persistence, xmlNodePtr const parent_node) {
   const BtSong * const self = BT_SONG(persistence);
   xmlNodePtr node=NULL;
 
@@ -1236,15 +1268,15 @@ static xmlNodePtr bt_song_persistence_save(const BtPersistence * const persisten
     xmlNewProp(node,XML_CHAR_PTR("xmlns:xsd"),XML_CHAR_PTR("http://www.w3.org/2001/XMLSchema-instance"));
     xmlNewProp(node,XML_CHAR_PTR("xsd:noNamespaceSchemaLocation"),XML_CHAR_PTR("buzztard.xsd"));
 
-    bt_persistence_save(BT_PERSISTENCE(self->priv->song_info),node,NULL);
-    bt_persistence_save(BT_PERSISTENCE(self->priv->setup),node,NULL);
-    bt_persistence_save(BT_PERSISTENCE(self->priv->sequence),node,NULL);
-    bt_persistence_save(BT_PERSISTENCE(self->priv->wavetable),node,NULL);
+    bt_persistence_save(BT_PERSISTENCE(self->priv->song_info),node);
+    bt_persistence_save(BT_PERSISTENCE(self->priv->setup),node);
+    bt_persistence_save(BT_PERSISTENCE(self->priv->sequence),node);
+    bt_persistence_save(BT_PERSISTENCE(self->priv->wavetable),node);
   }
   return(node);
 }
 
-static BtPersistence *bt_song_persistence_load(const GType type, const BtPersistence * const persistence, xmlNodePtr node, const BtPersistenceLocation * const location, GError **err, va_list var_args) {
+static BtPersistence *bt_song_persistence_load(const GType type, const BtPersistence * const persistence, xmlNodePtr node, GError **err, va_list var_args) {
   const BtSong * const self = BT_SONG(persistence);
   // @todo: this is a bit inconsistent
   // gtk is getting labels with progressbar, we could use this then
@@ -1258,22 +1290,22 @@ static BtPersistence *bt_song_persistence_load(const GType type, const BtPersist
     if(!xmlNodeIsText(node)) {
       if(!strncmp((gchar *)node->name,"meta\0",5)) {
         //g_sprintf(status,msg,"metadata");g_object_set(G_OBJECT(self->priv->song_io),"status",status,NULL);
-        if(!bt_persistence_load(BT_TYPE_SONG_INFO,BT_PERSISTENCE(self->priv->song_info),node,NULL,NULL,NULL))
+        if(!bt_persistence_load(BT_TYPE_SONG_INFO,BT_PERSISTENCE(self->priv->song_info),node,NULL,NULL))
           goto Error;
       }
       else if(!strncmp((gchar *)node->name,"setup\0",6)) {
         //g_sprintf(status,msg,"setup");g_object_set(G_OBJECT(self->priv->song_io),"status",status,NULL);
-        if(!bt_persistence_load(BT_TYPE_SETUP,BT_PERSISTENCE(self->priv->setup),node,NULL,NULL,NULL))
+        if(!bt_persistence_load(BT_TYPE_SETUP,BT_PERSISTENCE(self->priv->setup),node,NULL,NULL))
           goto Error;
       }
       else if(!strncmp((gchar *)node->name,"sequence\0",9)) {
         //g_sprintf(status,msg,"sequence");g_object_set(G_OBJECT(self->priv->song_io),"status",status,NULL);
-        if(!bt_persistence_load(BT_TYPE_SEQUENCE,BT_PERSISTENCE(self->priv->sequence),node,NULL,NULL,NULL))
+        if(!bt_persistence_load(BT_TYPE_SEQUENCE,BT_PERSISTENCE(self->priv->sequence),node,NULL,NULL))
           goto Error;
       }
       else if(!strncmp((gchar *)node->name,"wavetable\0",10)) {
         //g_sprintf(status,msg,"wavetable");g_object_set(G_OBJECT(self->priv->song_io),"status",status,NULL);
-        if(!bt_persistence_load(BT_TYPE_WAVETABLE,BT_PERSISTENCE(self->priv->wavetable),node,NULL,NULL,NULL))
+        if(!bt_persistence_load(BT_TYPE_WAVETABLE,BT_PERSISTENCE(self->priv->wavetable),node,NULL,NULL))
           goto Error;
       }
     }
@@ -1302,22 +1334,24 @@ static void bt_song_persistence_interface_init(gpointer const g_iface, gpointer
 static void bt_song_constructed(GObject *object) {
   BtSong *self=BT_SONG(object);
   
-  if(G_OBJECT_CLASS(parent_class)->constructed)
-    G_OBJECT_CLASS(parent_class)->constructed(object);
+  if(G_OBJECT_CLASS(bt_song_parent_class)->constructed)
+    G_OBJECT_CLASS(bt_song_parent_class)->constructed(object);
 
   g_return_if_fail(BT_IS_APPLICATION(self->priv->app));
   
-  g_object_get(G_OBJECT(self->priv->app),"bin",&self->priv->bin,NULL);
-  
+  g_object_get((gpointer)(self->priv->app),"bin",&self->priv->bin,NULL);
+    
   GstBus * const bus=gst_element_get_bus(GST_ELEMENT(self->priv->bin));
-  GST_DEBUG("listen to bus messages (%p)",bus);
-  gst_bus_add_signal_watch_full(bus, G_PRIORITY_HIGH);
-  g_signal_connect(bus, "message::segment-done", G_CALLBACK(on_song_segment_done), (gpointer)self);
-  g_signal_connect(bus, "message::eos", G_CALLBACK(on_song_eos), (gpointer)self);
-  g_signal_connect(bus, "message::state-changed", G_CALLBACK(on_song_state_changed), (gpointer)self);
-  g_signal_connect(bus, "message::async-done", G_CALLBACK(on_song_async_done), (gpointer)self);
-  g_signal_connect(bus, "message::clock-lost", G_CALLBACK(on_song_clock_lost), (gpointer)self);
-  gst_object_unref(bus);
+  if (bus) {
+    GST_DEBUG("listen to bus messages (%p)",bus);
+    gst_bus_add_signal_watch_full(bus, G_PRIORITY_HIGH);
+    g_signal_connect(bus, "message::segment-done", G_CALLBACK(on_song_segment_done), (gpointer)self);
+    g_signal_connect(bus, "message::eos", G_CALLBACK(on_song_eos), (gpointer)self);
+    g_signal_connect(bus, "message::state-changed", G_CALLBACK(on_song_state_changed), (gpointer)self);
+    g_signal_connect(bus, "message::async-done", G_CALLBACK(on_song_async_done), (gpointer)self);
+    g_signal_connect(bus, "message::clock-lost", G_CALLBACK(on_song_clock_lost), (gpointer)self);
+    gst_object_unref(bus);
+  }
 
   /* don't change the order */
   self->priv->song_info=bt_song_info_new(self);
@@ -1334,7 +1368,7 @@ static void bt_song_constructed(GObject *object) {
   g_signal_connect(self->priv->song_info,"notify::bpm",G_CALLBACK(bt_song_on_tempo_changed),(gpointer)self);
   GST_DEBUG("  tempo-signals connected");
 
-  bt_song_update_play_seek_event(BT_SONG(self));
+  bt_song_update_play_seek_event_and_play_pos(BT_SONG(self));
   GST_INFO("  new song created: %p",self);
 }
 
@@ -1364,12 +1398,12 @@ static void bt_song_get_property(GObject * const object, const guint property_id
     case SONG_WAVETABLE: {
       g_value_set_object(value, self->priv->wavetable);
     } break;
-    case SONG_UNSAVED: {
-      g_value_set_boolean(value, self->priv->unsaved);
-    } break;
     case SONG_PLAY_POS: {
       g_value_set_ulong(value, self->priv->play_pos);
     } break;
+    case SONG_PLAY_RATE: {
+      g_value_set_double(value, self->priv->play_rate);
+    } break;
     case SONG_IS_PLAYING: {
       g_value_set_boolean(value, self->priv->is_playing);
     } break;
@@ -1395,20 +1429,12 @@ static void bt_song_set_property(GObject * const object, const guint property_id
       g_object_try_weak_ref(self->priv->app);
       GST_DEBUG("set the app for the song: %p",self->priv->app);
     } break;
-    /*case SONG_BIN: {
-      self->priv->bin=GST_BIN(g_value_dup_object(value));
-      GST_DEBUG("set the bin for the song: %p",self->priv->bin);
-    } break;*/
     case SONG_MASTER: {
       g_object_try_weak_unref(self->priv->master);
       self->priv->master = BT_SINK_MACHINE(g_value_get_object(value));
       g_object_try_weak_ref(self->priv->master);
-      g_object_get(G_OBJECT(self->priv->master),"machine",&self->priv->master_bin,NULL);
-      GST_DEBUG("set the master for the song: %p (machine-refs: %d)",self->priv->master,(G_OBJECT(self->priv->master))->ref_count);
-    } break;
-    case SONG_UNSAVED: {
-      self->priv->unsaved = g_value_get_boolean(value);
-      GST_WARNING("set the unsaved flag for the song: %d (please use bt_song_set_unsaved())",self->priv->unsaved);
+      g_object_get((gpointer)(self->priv->master),"machine",&self->priv->master_bin,NULL);
+      GST_DEBUG("set the master for the song: %p (machine-ref_ct=%d)",self->priv->master,G_OBJECT_REF_COUNT(self->priv->master));
     } break;
     case SONG_PLAY_POS: {
       self->priv->play_pos=bt_sequence_limit_play_pos(self->priv->sequence,g_value_get_ulong(value));
@@ -1416,6 +1442,12 @@ static void bt_song_set_property(GObject * const object, const guint property_id
       // seek on playpos changes (if playing)
       bt_song_seek_to_play_pos(self);
     } break;
+    case SONG_PLAY_RATE: {
+      self->priv->play_rate=g_value_get_double(value);
+      GST_DEBUG("set the play-rate: %lf",self->priv->play_rate);
+      // update rate (if playing)
+      bt_song_change_play_rate(self);
+    } break;
     case SONG_IS_IDLE: {
       self->priv->is_idle=g_value_get_boolean(value);
       if(!self->priv->is_playing) {
@@ -1444,10 +1476,6 @@ static void bt_song_dispose(GObject * const object) {
   return_if_disposed();
   self->priv->dispose_has_run = TRUE;
 
-  //DEBUG
-  //bt_song_write_to_xml_file(self);
-  //DEBUG
-
   GST_DEBUG("!!!! self=%p",self);
   
   if(self->priv->playback_timeout_id)
@@ -1484,24 +1512,24 @@ static void bt_song_dispose(GObject * const object) {
     g_signal_handlers_disconnect_matched(self->priv->song_info,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,bt_song_on_tempo_changed,(gpointer)self);
   }
 
-  if(self->priv->master) GST_DEBUG("sink-machine-refs: %d",(G_OBJECT(self->priv->master))->ref_count);
+  GST_DEBUG("sink-machine-ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->master));
   if(self->priv->master_bin) gst_object_unref(self->priv->master_bin);
   g_object_try_weak_unref(self->priv->master);
   
   if(self->priv->song_info) {
-    GST_DEBUG("song_info->refs: %d",(G_OBJECT(self->priv->song_info))->ref_count);
+    GST_DEBUG("song_info->ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->song_info));
     g_object_unref(self->priv->song_info);
   }
   if(self->priv->sequence) {
-    GST_DEBUG("sequence->refs: %d",(G_OBJECT(self->priv->sequence))->ref_count);
+    GST_DEBUG("sequence->ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->sequence));
     g_object_unref(self->priv->sequence);
   }
   if(self->priv->setup) {
-    GST_DEBUG("setup->refs: %d",(G_OBJECT(self->priv->setup))->ref_count);
+    GST_DEBUG("setup->ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->setup));
     g_object_unref(self->priv->setup);
   }
   if(self->priv->wavetable) {
-    GST_DEBUG("wavetable->refs: %d",(G_OBJECT(self->priv->wavetable))->ref_count);
+    GST_DEBUG("wavetable->ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->wavetable));
     g_object_unref(self->priv->wavetable);
   }
 
@@ -1510,32 +1538,36 @@ static void bt_song_dispose(GObject * const object) {
   if(self->priv->loop_seek_event) gst_event_unref(self->priv->loop_seek_event);
   if(self->priv->idle_seek_event) gst_event_unref(self->priv->idle_seek_event);
   if(self->priv->idle_loop_seek_event) gst_event_unref(self->priv->idle_loop_seek_event);
-  if(self->priv->bin) gst_object_unref(self->priv->bin);
+  if(self->priv->bin) {
+    GST_DEBUG("bin-ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->bin));
+    gst_object_unref(self->priv->bin);
+  }
   g_object_try_weak_unref(self->priv->app);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_song_parent_class)->dispose(object);
   GST_DEBUG("  done");
 }
 
+#ifndef GST_DISABLE_GST_DEBUG
 static void bt_song_finalize(GObject * const object) {
   const BtSong * const self = BT_SONG(object);
 
   GST_DEBUG("!!!! self=%p",self);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(bt_song_parent_class)->finalize(object);
   GST_DEBUG("  done");
 }
+#endif
 
 //-- class internals
 
-static void bt_song_init(const GTypeInstance * const instance, gconstpointer const g_class) {
-  BtSong * const self = BT_SONG(instance);
-
+static void bt_song_init(BtSong *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_SONG, BtSongPrivate);
 
   self->priv->position_query=gst_query_new_position(GST_FORMAT_TIME);
+  self->priv->play_rate=1.0;
 
   self->priv->idle_seek_event=gst_event_new_seek(1.0, GST_FORMAT_TIME,
     GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT,
@@ -1561,14 +1593,15 @@ static void bt_song_init(const GTypeInstance * const instance, gconstpointer con
 static void bt_song_class_init(BtSongClass * const klass) {
   GObjectClass * const gobject_class = G_OBJECT_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtSongPrivate));
 
   gobject_class->constructed  = bt_song_constructed;
   gobject_class->set_property = bt_song_set_property;
   gobject_class->get_property = bt_song_get_property;
   gobject_class->dispose      = bt_song_dispose;
+#ifndef GST_DISABLE_GST_DEBUG
   gobject_class->finalize     = bt_song_finalize;
+#endif
 
   g_object_class_install_property(gobject_class,SONG_APP,
                                   g_param_spec_object("app",
@@ -1619,13 +1652,6 @@ static void bt_song_class_init(BtSongClass * const klass) {
                                      BT_TYPE_WAVETABLE, /* object type */
                                      G_PARAM_READABLE|G_PARAM_STATIC_STRINGS));
 
-  g_object_class_install_property(gobject_class,SONG_UNSAVED,
-                                  g_param_spec_boolean("unsaved",
-                                     "unsaved prop",
-                                     "tell wheter the current state of the song has been saved",
-                                     TRUE,
-                                     G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
-
   g_object_class_install_property(gobject_class,SONG_PLAY_POS,
                                   g_param_spec_ulong("play-pos",
                                      "play-pos prop",
@@ -1635,14 +1661,23 @@ static void bt_song_class_init(BtSongClass * const klass) {
                                      0,
                                      G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property(gobject_class,SONG_PLAY_RATE,
+                                  g_param_spec_double("play-rate",
+                                     "play-rate prop",
+                                     "playback rate of the sequence",
+                                     -5.0,
+                                     5.0,
+                                     1.0,
+                                     G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property(gobject_class,SONG_IS_PLAYING,
                                   g_param_spec_boolean("is-playing",
                                      "is-playing prop",
-                                     "tell wheter the song is playing right now or not",
+                                     "tell whether the song is playing right now or not",
                                      FALSE,
                                      G_PARAM_READABLE|G_PARAM_STATIC_STRINGS));
 
-    g_object_class_install_property(gobject_class,SONG_IS_IDLE,
+  g_object_class_install_property(gobject_class,SONG_IS_IDLE,
                                   g_param_spec_boolean("is-idle",
                                      "is-idle prop",
                                      "request that the song should idle-loop if not playing",
@@ -1657,39 +1692,3 @@ static void bt_song_class_init(BtSongClass * const klass) {
                                      G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 }
 
-GType bt_song_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtSongClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_song_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtSong),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_song_init, // instance_init
-      NULL // value_table
-    };
-    const GInterfaceInfo persistence_interface_info = {
-      (GInterfaceInitFunc) bt_song_persistence_interface_init,  // interface_init
-      NULL, // interface_finalize
-      NULL  // interface_data
-    };
-#if 0
-    static const GInterfaceInfo child_proxy_info = {
-      (GInterfaceInitFunc) bt_song_child_proxy_init,
-      NULL,
-      NULL
-    };
-#endif
-
-    type = g_type_register_static(G_TYPE_OBJECT,"BtSong",&info,0);
-    g_type_add_interface_static(type, BT_TYPE_PERSISTENCE, &persistence_interface_info);
-#if 0
-    g_type_add_interface_static(type, GST_TYPE_CHILD_PROXY, &child_proxy_info);
-#endif
-  }
-  return type;
-}
