@@ -1,4 +1,4 @@
-/* $Id: wave.c 2550 2009-09-03 18:41:47Z ensonic $
+/* $Id: wave.c 3675 2011-10-04 21:33:12Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -90,18 +90,26 @@ struct _BtWavePrivate {
   guint channels; /* number of channels (1,2) */
 
   GList *wavelevels;    // each entry points to a BtWavelevel
-  
+
   /* wave loader */
   GstElement *pipeline,*fmt;
   gint fd,ext_fd;
+  FILE *tf,*ext_tf;
 };
 
 static GQuark error_domain=0;
 
-static GObjectClass *parent_class=NULL;
-
 static guint signals[LAST_SIGNAL]={0,};
 
+//-- the class
+
+static void bt_wave_persistence_interface_init(gpointer const g_iface, gpointer const iface_data);
+
+G_DEFINE_TYPE_WITH_CODE (BtWave, bt_wave, G_TYPE_OBJECT,
+  G_IMPLEMENT_INTERFACE (BT_TYPE_PERSISTENCE,
+    bt_wave_persistence_interface_init));
+
+
 //-- enums
 
 GType bt_wave_loop_mode_get_type(void) {
@@ -123,12 +131,12 @@ GType bt_wave_loop_mode_get_type(void) {
 static void on_wave_io_error(const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
   GError *err = NULL;
   gchar *dbg = NULL;
-  
+
   gst_message_parse_error(message, &err, &dbg);
-  GST_WARNING ("ERROR: %s (%s)", err->message, (dbg) ? dbg : "no details");
-  g_error_free (err);
-  g_free (dbg);
-  
+  GST_WARNING_OBJECT(GST_MESSAGE_SRC(message),"ERROR: %s (%s)", err->message, (dbg ? dbg : "no details"));
+  g_error_free(err);
+  g_free(dbg);
+
   //g_signal_emit(G_OBJECT(self),signals[LOADING_DONE_EVENT], 0, FALSE);
   //wave_loader_free(self);
 }
@@ -136,12 +144,12 @@ static void on_wave_io_error(const GstBus * const bus, GstMessage *message, gcon
 static void on_wave_io_warning(const GstBus * const bus, GstMessage * message, gconstpointer user_data) {
   GError *err = NULL;
   gchar *dbg = NULL;
-  
+
   gst_message_parse_warning(message, &err, &dbg);
-  GST_WARNING ("WARNING: %s (%s)", err->message, (dbg) ? dbg : "no details");
-  g_error_free (err);
-  g_free (dbg);
-  
+  GST_WARNING_OBJECT(GST_MESSAGE_SRC(message),"WARNING: %s (%s)", err->message, (dbg ? dbg : "no details"));
+  g_error_free(err);
+  g_free(dbg);
+
   //g_signal_emit(G_OBJECT(self),signals[LOADING_DONE_EVENT], 0, FALSE);
   //wave_loader_free(self);
 }
@@ -149,28 +157,40 @@ static void on_wave_io_warning(const GstBus * const bus, GstMessage * message, g
 static void wave_loader_free(const BtWave *self) {
   if(self->priv->pipeline) {
     GstBus *bus;
-    
+
     bus=gst_element_get_bus(self->priv->pipeline);
     gst_bus_remove_signal_watch(bus);
     gst_object_unref(bus);
 
+    gst_element_set_state(self->priv->pipeline,GST_STATE_NULL);
     gst_object_unref(self->priv->pipeline);
     self->priv->pipeline=NULL;
     self->priv->fmt=NULL;
   }
-  if(self->priv->ext_fd!=-1) {
+  if(self->priv->ext_tf) {
+    // ext_fd is fileno() of a ext_tf
+    fclose(self->priv->ext_tf);
+    self->priv->ext_tf=NULL;
+    self->priv->ext_fd=-1;
+  } else if(self->priv->ext_fd!=-1) {
     close(self->priv->ext_fd);
     self->priv->ext_fd=-1;
   }
-  if(self->priv->fd!=-1) {
+  if(self->priv->tf) {
+    // fd is fileno() of a tf
+    fclose(self->priv->tf);
+    self->priv->tf=NULL;
+    self->priv->fd=-1;
+  } else if(self->priv->fd!=-1) {
     close(self->priv->fd);
     self->priv->fd=-1;
   }
 }
 
 static void on_wave_loader_new_pad(GstElement *bin,GstPad *pad,gboolean islast,gpointer user_data) {
+  // @todo: if we pass the pad in user_data we can use gst_pad_link()
   if(!gst_element_link(bin,GST_ELEMENT(user_data))) {
-    GST_WARNING("Can't link output of wave decoder.");
+    GST_WARNING("Can't link output of wave decoder to converter.");
   }
 }
 
@@ -185,20 +205,20 @@ static void on_wave_loader_eos(const GstBus * const bus, const GstMessage * cons
   GstFormat format=GST_FORMAT_TIME;
   gpointer data=NULL;
   struct stat buf={0,};
-  
+
   GST_INFO("sample loading done, eos from %s",GST_MESSAGE_SRC_NAME(message));
-  
+
   // query length and convert to samples
   if(!gst_element_query_duration(self->priv->pipeline, &format, &duration)) {
     GST_WARNING("getting sample duration failed");
   }
-  
-  // get caps for sample rate and channels 
+
+  // get caps for sample rate and channels
   if((pad=gst_element_get_static_pad(self->priv->fmt,"src"))) {
     caps=GST_PAD_CAPS(pad);
     if(caps && GST_CAPS_IS_SIMPLE(caps)) {
       GstStructure *structure = gst_caps_get_structure(caps,0);
-      
+
       gst_structure_get_int(structure,"channels",&channels);
       gst_structure_get_int(structure,"rate",&rate);
       length=gst_util_uint64_scale(duration,(guint64)rate,GST_SECOND);
@@ -211,7 +231,7 @@ static void on_wave_loader_eos(const GstBus * const bus, const GstMessage * cons
 
   GST_INFO("sample decoded: channels=%d, rate=%d, length=%"GST_TIME_FORMAT,
     channels, rate, GST_TIME_ARGS(duration));
-  
+
   if(!(fstat(self->priv->fd, &buf))) {
     if((data=g_try_malloc(buf.st_size))) {
       /* mmap is unsave for removable drives :(
@@ -219,12 +239,12 @@ static void on_wave_loader_eos(const GstBus * const bus, const GstMessage * cons
        */
       if(lseek(self->priv->fd,0,SEEK_SET) == 0) {
         ssize_t bytes;
-        
+
         bytes=read(self->priv->fd,data,buf.st_size);
-        
+
         self->priv->channels=channels;
         g_object_notify(G_OBJECT(self),"channels");
-    
+
         wavelevel=bt_wavelevel_new(self->priv->song,self,
           BT_WAVELEVEL_DEFAULT_ROOT_NOTE,(gulong)length,
           -1,-1, /* loop */
@@ -232,23 +252,23 @@ static void on_wave_loader_eos(const GstBus * const bus, const GstMessage * cons
         g_object_unref(wavelevel);
         /* emit signal so that UI can redraw */
         GST_INFO("sample loaded (%"G_GSSIZE_FORMAT"/%ld bytes)",bytes,buf.st_size);
-        g_signal_emit(G_OBJECT(self),signals[LOADING_DONE_EVENT], 0, TRUE);
+        g_signal_emit(self,signals[LOADING_DONE_EVENT], 0, TRUE);
       }
       else {
         GST_WARNING("can't seek to start of sample data");
-        g_signal_emit(G_OBJECT(self),signals[LOADING_DONE_EVENT], 0, FALSE);
+        g_signal_emit(self,signals[LOADING_DONE_EVENT], 0, FALSE);
       }
     }
     else {
       GST_WARNING("sample is too long (%ld bytes), not trying to load",buf.st_size);
-      g_signal_emit(G_OBJECT(self),signals[LOADING_DONE_EVENT], 0, FALSE);
+      g_signal_emit(self,signals[LOADING_DONE_EVENT], 0, FALSE);
     }
   }
   else {
     GST_WARNING("can't stat() sample");
-    g_signal_emit(G_OBJECT(self),signals[LOADING_DONE_EVENT], 0, FALSE);
+    g_signal_emit(self,signals[LOADING_DONE_EVENT], 0, FALSE);
   }
-  
+
   gst_element_set_state(self->priv->pipeline,GST_STATE_NULL);
 }
 
@@ -268,6 +288,8 @@ static gboolean bt_wave_load_from_uri(const BtWave * const self, const gchar * c
   GstCaps *caps;
 
   GST_INFO("about to load sample %s / %s",self->priv->uri,uri);
+  // this leaks!
+  //GST_INFO("current dir is %s", g_get_current_dir());
 
   // check if the url is valid
   // if(!uri) goto invalid_uri;
@@ -280,7 +302,7 @@ static gboolean bt_wave_load_from_uri(const BtWave * const self, const gchar * c
   conv=gst_element_factory_make("audioconvert",NULL);
   self->priv->fmt=gst_element_factory_make("capsfilter",NULL);
   sink=gst_element_factory_make("fdsink",NULL);
-  
+
   // configure elements
   caps=gst_caps_new_simple("audio/x-raw-int",
     "rate", GST_TYPE_INT_RANGE,1,G_MAXINT,
@@ -291,8 +313,13 @@ static gboolean bt_wave_load_from_uri(const BtWave * const self, const gchar * c
     NULL);
   g_object_set(self->priv->fmt,"caps",caps,NULL);
   gst_caps_unref(caps);
-  
-  self->priv->fd=fileno(tmpfile()); // or mkstemp("...XXXXXX")
+
+  if(!(self->priv->tf=tmpfile())) {
+    res=FALSE;
+    GST_WARNING ("Can't create tempfile.");
+    goto Error;
+  }
+  self->priv->fd=fileno(self->priv->tf);
   g_object_set(sink,"fd",self->priv->fd,"sync",FALSE,NULL);
 
   // add and link
@@ -307,7 +334,7 @@ static gboolean bt_wave_load_from_uri(const BtWave * const self, const gchar * c
     GST_WARNING ("Can't link wave loader pipeline (conf ! fmt ! sink).");
     goto Error;
   }
-  g_signal_connect(G_OBJECT(dec),"new-decoded-pad",G_CALLBACK(on_wave_loader_new_pad),(gpointer)conv);
+  g_signal_connect(dec,"new-decoded-pad",G_CALLBACK(on_wave_loader_new_pad),(gpointer)conv);
 
   /* @todo: during loading wave-data (into wavelevels)
    * - use statusbar for loader progress ("status" property like in song_io)
@@ -316,14 +343,14 @@ static gboolean bt_wave_load_from_uri(const BtWave * const self, const gchar * c
    *   - if so, how to get real/virtual memory sizes?
    *     mallinfo() not enough, sysconf()?
    */
-  
+
   bus=gst_element_get_bus(self->priv->pipeline);
   gst_bus_add_signal_watch_full(bus, G_PRIORITY_HIGH);
   g_signal_connect(bus, "message::error", G_CALLBACK(on_wave_io_error), (gpointer)self);
   g_signal_connect(bus, "message::warning", G_CALLBACK(on_wave_io_warning), (gpointer)self);
   g_signal_connect(bus, "message::eos", G_CALLBACK(on_wave_loader_eos), (gpointer)self);
   gst_object_unref(bus);
-  
+
   // play and wait for EOS
   if(gst_element_set_state(self->priv->pipeline,GST_STATE_PLAYING)==GST_STATE_CHANGE_FAILURE) {
     GST_WARNING ("Can't set wave loader pipeline for %s / %s to playing",self->priv->uri,uri);
@@ -333,10 +360,12 @@ static gboolean bt_wave_load_from_uri(const BtWave * const self, const gchar * c
   else {
     GST_INFO("loading sample ...");
   }
-  
+
   /* no need to *wait* for eos, we fire loading done signal */
 
 Error:
+  if(!res)
+    wave_loader_free(self);
   return(res);
 }
 
@@ -355,16 +384,24 @@ static gboolean bt_wave_save_to_fd(const BtWave * const self) {
   //gchar *fn_wav;
   gulong srate,length,size,written;
   gint16 *data;
+  FILE *tf=NULL;
   gint fd;
-  
+
   // we should have a wave loaded
   g_assert(self->priv->wavelevels->data);
 
-  //fn_wav=g_strdup_printf("%s" G_DIR_SEPARATOR_S "XXXXXX",g_get_tmp_dir());
-  fd=fileno(tmpfile());
-  self->priv->ext_fd=fileno(tmpfile());
-  //self->priv->ext_fd=mkstemp(fn_wav);
-  //g_free(fn_wav);
+  if(!(tf=tmpfile())) {
+    res=FALSE;
+    GST_WARNING ("Can't create tempfile.");
+    goto Error;
+  }
+  fd=fileno(tf);
+  if(!(self->priv->ext_tf=tmpfile())) {
+    res=FALSE;
+    GST_WARNING ("Can't create tempfile.");
+    goto Error;
+  }
+  self->priv->ext_fd=fileno(self->priv->ext_tf);
 
   // the data is in the wave-level :/
   wavelevel=BT_WAVELEVEL(self->priv->wavelevels->data);
@@ -418,7 +455,7 @@ static gboolean bt_wave_save_to_fd(const BtWave * const self) {
   GST_INFO("wrote %d, todo %d",written, size);
   lseek(fd,0,SEEK_SET);
 
-  // play and wait for EOS 
+  // play and wait for EOS
   if(gst_element_set_state(pipeline,GST_STATE_PLAYING)==GST_STATE_CHANGE_FAILURE) {
     GST_WARNING ("Can't set wave saver pipeline for to playing");
     gst_element_set_state(pipeline,GST_STATE_NULL);
@@ -441,11 +478,15 @@ static gboolean bt_wave_save_to_fd(const BtWave * const self) {
 
   gst_object_unref(bus);
   gst_element_set_state(pipeline,GST_STATE_NULL);
-  close(fd);
+  gst_object_unref(pipeline);
 
   GST_INFO("sample saved");
 
 Error:
+  if(tf)
+    fclose(tf);
+  if(!res)
+    wave_loader_free(self);
   return(res);
 }
 
@@ -462,10 +503,10 @@ static gboolean bt_wave_save_from_fd(const BtWave * const self, BtSongIONative *
   fp=g_strdup_printf("wavetable/%s",fn);
 
   GST_INFO("saving external uri=%s,%s -> zip=%s",uri,self->priv->uri,fp);
-  
+
   bt_song_io_native_bzt_copy_from_uri(BT_SONG_IO_NATIVE_BZT(song_io),fp,uri);
   g_free(fp);
-  
+
   return(TRUE);
 }
 
@@ -479,7 +520,7 @@ static gboolean bt_wave_save_from_fd(const BtWave * const self, BtSongIONative *
  * @uri: the location of the sample data
  * @index: the list slot for the new wave
  * @volume: the volume of the wave
- * @loop-mode: loop playback mode
+ * @loop_mode: loop playback mode
  * @channels: number of audio channels
  *
  * Create a new instance
@@ -512,9 +553,8 @@ gboolean bt_wave_add_wavelevel(const BtWave * const self, const BtWavelevel * co
 
   if(!g_list_find(self->priv->wavelevels,wavelevel)) {
     ret=TRUE;
-    self->priv->wavelevels=g_list_append(self->priv->wavelevels,g_object_ref(G_OBJECT(wavelevel)));
-    //g_signal_emit(G_OBJECT(self),signals[WAVELEVEL_ADDED_EVENT], 0, wavelevel);
-    bt_song_set_unsaved(self->priv->song,TRUE);
+    self->priv->wavelevels=g_list_append(self->priv->wavelevels,g_object_ref((gpointer)wavelevel));
+    //g_signal_emit((gpointer)self,signals[WAVELEVEL_ADDED_EVENT], 0, wavelevel);
   }
   else {
     GST_WARNING("trying to add wavelevel again");
@@ -531,11 +571,11 @@ gboolean bt_wave_add_wavelevel(const BtWave * const self, const BtWavelevel * co
  * The wavelevel must have been added previously to this wave with bt_wave_add_wavelevel().
  * Unref the wavelevel, when done with it.
  *
- * Returns: #BtWaveLevel instance or %NULL if not found
+ * Returns: #BtWavelevel instance or %NULL if not found
  */
 BtWavelevel *bt_wave_get_level_by_index(const BtWave * const self,const gulong index) {
   BtWavelevel *wavelevel;
-  
+
   if((wavelevel=g_list_nth_data(self->priv->wavelevels,index))) {
     return(g_object_ref(wavelevel));
   }
@@ -544,7 +584,7 @@ BtWavelevel *bt_wave_get_level_by_index(const BtWave * const self,const gulong i
 
 //-- io interface
 
-static xmlNodePtr bt_wave_persistence_save(const BtPersistence * const persistence, const xmlNodePtr const parent_node, const BtPersistenceSelection * const selection) {
+static xmlNodePtr bt_wave_persistence_save(const BtPersistence * const persistence, const xmlNodePtr const parent_node) {
   const BtWave * const self = BT_WAVE(persistence);
   xmlNodePtr node=NULL;
   xmlNodePtr child_node;
@@ -563,7 +603,7 @@ static xmlNodePtr bt_wave_persistence_save(const BtPersistence * const persisten
     xmlNewProp(node,XML_CHAR_PTR("uri"),XML_CHAR_PTR(self->priv->uri));
     xmlNewProp(node,XML_CHAR_PTR("volume"),XML_CHAR_PTR(bt_persistence_strfmt_double(self->priv->volume)));
     xmlNewProp(node,XML_CHAR_PTR("loop-mode"),XML_CHAR_PTR(bt_persistence_strfmt_enum(BT_TYPE_WAVE_LOOP_MODE,self->priv->loop_mode)));
-    
+
     // check if we need to save external data
     g_object_get(self->priv->song,"song-io",&song_io,NULL);
     if (song_io) {
@@ -589,7 +629,7 @@ static xmlNodePtr bt_wave_persistence_save(const BtPersistence * const persisten
   return(node);
 }
 
-static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersistence * const persistence, xmlNodePtr node, const BtPersistenceLocation * const location, GError **err, va_list var_args) {
+static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersistence * const persistence, xmlNodePtr node, GError **err, va_list var_args) {
   BtWave *self;
   BtPersistence *result;
   BtSongIONative *song_io;
@@ -607,7 +647,7 @@ static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersist
   xmlChar * const loop_mode_str=xmlGetProp(node,XML_CHAR_PTR("loop-mode"));
   gint loop_mode=bt_persistence_parse_enum(BT_TYPE_WAVE_LOOP_MODE,(char *)loop_mode_str);
   if(loop_mode==-1) loop_mode=BT_WAVE_LOOP_MODE_OFF;
-  
+
   if(!persistence) {
     BtSong *song=NULL;
     gchar *param_name;
@@ -624,20 +664,20 @@ static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersist
       }
       param_name=va_arg(var_args,gchar*);
     }
-    
+
     /* @todo: ugly hack, don't pass uri_str yet, this would trigger loading
      * which we do below (see also: bt_wave_constructed) */
     self=bt_wave_new(song,(gchar*)name,NULL,index,volume,loop_mode,0);
     result=BT_PERSISTENCE(self);
-    g_object_set(G_OBJECT(self),"uri",uri_str,NULL);
+    g_object_set(self,"uri",uri_str,NULL);
   }
   else {
     self=BT_WAVE(persistence);
     result=BT_PERSISTENCE(self);
 
-    g_object_set(G_OBJECT(self),"index",index,"name",name,"uri",uri_str,"volume",volume,"loop-mode",loop_mode,NULL);
-  }   
-  
+    g_object_set(self,"index",index,"name",name,"uri",uri_str,"volume",volume,"loop-mode",loop_mode,NULL);
+  }
+
   xmlFree(index_str);
   xmlFree(name);
   xmlFree(volume_str);
@@ -656,16 +696,21 @@ static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersist
       else fn++;
       fp=g_strdup_printf("wavetable/%s",fn);
 
-      GST_INFO("loading external uri=%s -> zip=%s",(gchar *)uri_str,fp); 
-      
+      GST_INFO("loading external uri=%s -> zip=%s",(gchar *)uri_str,fp);
+
       // we need to copy the files from zip and change the uri to "fd://%d"
-      self->priv->ext_fd=fileno(tmpfile()); // or mkstemp("...XXXXXX")
-      if(bt_song_io_native_bzt_copy_to_fd(BT_SONG_IO_NATIVE_BZT(song_io),fp,self->priv->ext_fd)) {
-        uri=g_strdup_printf("fd://%d",self->priv->ext_fd);
-      }
-      else {
-        close(self->priv->ext_fd);
-        self->priv->ext_fd=-1;
+      if((self->priv->ext_tf=tmpfile())) {
+        self->priv->ext_fd=fileno(self->priv->ext_tf);
+        if(bt_song_io_native_bzt_copy_to_fd(BT_SONG_IO_NATIVE_BZT(song_io),fp,self->priv->ext_fd)) {
+          uri=g_strdup_printf("fd://%d",self->priv->ext_fd);
+        }
+        else {
+          fclose(self->priv->ext_tf);
+          self->priv->ext_tf=NULL;
+          self->priv->ext_fd=-1;
+          unpack_failed=TRUE;
+        }
+      } else {
         unpack_failed=TRUE;
       }
       g_free(fp);
@@ -688,7 +733,7 @@ static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersist
   }
   else {
     GList *lnode=self->priv->wavelevels;
-  
+
     for(node=node->children;node;node=node->next) {
       if((!xmlNodeIsText(node)) && (!strncmp((gchar *)node->name,"wavelevel\0",10))) {
         /* loading the wave already created wave-levels,
@@ -697,7 +742,7 @@ static BtPersistence *bt_wave_persistence_load(const GType type, const BtPersist
         if(lnode) {
           BtWavelevel * const wave_level=BT_WAVELEVEL(lnode->data);
 
-          bt_persistence_load(BT_TYPE_WAVELEVEL,BT_PERSISTENCE(wave_level),node,NULL,NULL,NULL);
+          bt_persistence_load(BT_TYPE_WAVELEVEL,BT_PERSISTENCE(wave_level),node,NULL,NULL);
           lnode=g_list_next(lnode);
         }
       }
@@ -739,12 +784,12 @@ static void bt_wave_constructed(GObject *object) {
   BtWave *self=BT_WAVE(object);
   gboolean okay=TRUE;
 
-  if(G_OBJECT_CLASS(parent_class)->constructed)
-    G_OBJECT_CLASS(parent_class)->constructed(object);
+  if(G_OBJECT_CLASS(bt_wave_parent_class)->constructed)
+    G_OBJECT_CLASS(bt_wave_parent_class)->constructed(object);
 
   g_return_if_fail(BT_IS_SONG(self->priv->song));
 
-  // some SongIO loaders load the wave themself and set the data 
+  // some SongIO loaders load the wave themself and set the data
   if(self->priv->uri) {
     // @todo: move stuff from bt_wave_persistence_load() into bt_wave_load_from_uri()
     // try to load wavedata
@@ -756,7 +801,7 @@ static void bt_wave_constructed(GObject *object) {
   if(okay) {
     BtWavetable *wavetable;
     // add the wave to the wavetable of the song
-    g_object_get(G_OBJECT(self->priv->song),"wavetable",&wavetable,NULL);
+    g_object_get((gpointer)(self->priv->song),"wavetable",&wavetable,NULL);
     bt_wavetable_add_wave(wavetable,self);
     g_object_unref(wavetable);
   }
@@ -787,7 +832,7 @@ static void bt_wave_get_property(GObject * const object, const guint property_id
     } break;
     case WAVE_LOOP_MODE: {
       g_value_set_enum(value, self->priv->loop_mode);
-    } break;    
+    } break;
     case WAVE_CHANNELS: {
       g_value_set_uint(value, self->priv->channels);
     } break;
@@ -826,7 +871,7 @@ static void bt_wave_set_property(GObject * const object, const guint property_id
     } break;
     case WAVE_LOOP_MODE: {
       self->priv->loop_mode = g_value_get_enum(value);
-    } break;    
+    } break;
     case WAVE_CHANNELS: {
       self->priv->channels = g_value_get_uint(value);
       GST_DEBUG("set the channels for wave: %d",self->priv->channels);
@@ -850,20 +895,15 @@ static void bt_wave_dispose(GObject * const object) {
   // unref list of wavelevels
   if(self->priv->wavelevels) {
     for(node=self->priv->wavelevels;node;node=g_list_next(node)) {
-      {
-        const GObject * const obj=node->data;
-        GST_DEBUG("  free wavelevels : %p (%d)",obj,obj->ref_count);
-      }
+      GST_DEBUG("  free wavelevels : %p (%d)",node->data,G_OBJECT_REF_COUNT(node->data));
       g_object_try_unref(node->data);
       node->data=NULL;
     }
   }
-  
+
   wave_loader_free(self);
 
-  if(G_OBJECT_CLASS(parent_class)->dispose) {
-    (G_OBJECT_CLASS(parent_class)->dispose)(object);
-  }
+  G_OBJECT_CLASS(bt_wave_parent_class)->dispose(object);
 }
 
 static void bt_wave_finalize(GObject * const object) {
@@ -879,16 +919,12 @@ static void bt_wave_finalize(GObject * const object) {
   g_free(self->priv->name);
   g_free(self->priv->uri);
 
-  if(G_OBJECT_CLASS(parent_class)->finalize) {
-    (G_OBJECT_CLASS(parent_class)->finalize)(object);
-  }
+  G_OBJECT_CLASS(bt_wave_parent_class)->finalize(object);
 }
 
 //-- class internals
 
-static void bt_wave_init(GTypeInstance * const instance, gconstpointer const g_class) {
-  BtWave * const self = BT_WAVE(instance);
-
+static void bt_wave_init(BtWave *self) {
   self->priv=G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_WAVE, BtWavePrivate);
   self->priv->volume=1.0;
   self->priv->fd=-1;
@@ -898,10 +934,7 @@ static void bt_wave_init(GTypeInstance * const instance, gconstpointer const g_c
 static void bt_wave_class_init(BtWaveClass * const klass) {
   GObjectClass * const gobject_class = G_OBJECT_CLASS(klass);
 
-  // @idea: g_type_qname(BT_TYPE_WAVE);
-  error_domain=g_quark_from_static_string("BtWave");
-
-  parent_class=g_type_class_peek_parent(klass);
+  error_domain=g_type_qname(BT_TYPE_WAVE);
   g_type_class_add_private(klass,sizeof(BtWavePrivate));
 
   gobject_class->constructed  = bt_wave_constructed;
@@ -909,9 +942,7 @@ static void bt_wave_class_init(BtWaveClass * const klass) {
   gobject_class->get_property = bt_wave_get_property;
   gobject_class->dispose      = bt_wave_dispose;
   gobject_class->finalize     = bt_wave_finalize;
-  
-  klass->loading_done_event = NULL;
-  
+
   /**
    * BtWave::loading-done:
    * @self: the setup object that emitted the signal
@@ -922,7 +953,7 @@ static void bt_wave_class_init(BtWaveClass * const klass) {
   signals[LOADING_DONE_EVENT] = g_signal_new("loading-done",
                                         G_TYPE_FROM_CLASS(klass),
                                         G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                                        (guint)G_STRUCT_OFFSET(BtWaveClass,loading_done_event),
+                                        0,
                                         NULL, // accumulator
                                         NULL, // acc data
                                         g_cclosure_marshal_VOID__BOOLEAN,
@@ -930,7 +961,7 @@ static void bt_wave_class_init(BtWaveClass * const klass) {
                                         1, // n_params
                                         G_TYPE_BOOLEAN // param data
                                         );
-  
+
 
   g_object_class_install_property(gobject_class,WAVE_SONG,
                                   g_param_spec_object("song",
@@ -995,28 +1026,3 @@ static void bt_wave_class_init(BtWaveClass * const klass) {
                                      G_PARAM_CONSTRUCT|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 }
 
-GType bt_wave_get_type(void) {
-  static GType type = 0;
-  if (type == 0) {
-    const GTypeInfo info = {
-      sizeof(BtWaveClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_wave_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtWave),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_wave_init, // instance_init
-      NULL // value_table
-    };
-    const GInterfaceInfo persistence_interface_info = {
-      (GInterfaceInitFunc) bt_wave_persistence_interface_init,  // interface_init
-      NULL, // interface_finalize
-      NULL  // interface_data
-    };
-    type = g_type_register_static(G_TYPE_OBJECT,"BtWave",&info,0);
-    g_type_add_interface_static(type, BT_TYPE_PERSISTENCE, &persistence_interface_info);
-  }
-  return type;
-}
