@@ -1,4 +1,4 @@
-/* $Id: input-device.c 2202 2009-01-16 23:03:16Z ensonic $
+/* $Id: input-device.c 3705 2011-11-08 22:08:15Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2007 Buzztard team <buzztard-devel@lists.sf.net>
@@ -59,18 +59,16 @@ struct _BtIcInputDevicePrivate {
   /* io channel */
   GIOChannel *io_channel;
   gint io_source;
-
-  /* type|code -> control lookup */
-  GHashTable *controls;
 };
 
-static GObjectClass *parent_class=NULL;
+//-- the class
+
+G_DEFINE_TYPE (BtIcInputDevice, btic_input_device, BTIC_TYPE_DEVICE);
 
 //-- helper
 #define test_bit(bit, array)    (array[bit>>3] & (1<<(bit&0x7)))
 
 static gboolean register_trigger_controls(const BtIcInputDevice * const self,int fd) {
-  BtIcTriggerControl *control;
   guint ix;
   guint8 key_bitmask[KEY_MAX/8 + 1];
   const gchar *name = NULL;
@@ -145,10 +143,10 @@ static gboolean register_trigger_controls(const BtIcInputDevice * const self,int
            GST_INFO("Unknown key: %d",ix);
       }
       if(name) {
+        // BtIcTriggerControl *control;
         // create controller instances and register them
-        control = btic_trigger_control_new(BTIC_DEVICE(self),name);
-        key=(((gulong)EV_KEY)<<16)|(gulong)ix;
-        g_hash_table_insert(self->priv->controls,GUINT_TO_POINTER(key),(gpointer)control);
+        key=(((guint)EV_KEY)<<16)|(guint)ix;
+        /*control = */btic_trigger_control_new(BTIC_DEVICE(self),name,key);
       }
     }
   }
@@ -156,7 +154,6 @@ static gboolean register_trigger_controls(const BtIcInputDevice * const self,int
 }
 
 static gboolean register_abs_range_controls(const BtIcInputDevice * const self,int fd) {
-  BtIcAbsRangeControl *control;
   guint ix;
   guint8 abs_bitmask[ABS_MAX/8 + 1];
   struct input_absinfo abs_features;
@@ -200,7 +197,8 @@ static gboolean register_abs_range_controls(const BtIcInputDevice * const self,i
         default:
           GST_INFO("Unknown absolute axis : 0x%02x ", ix);
       }
-      if(ioctl(fd, EVIOCGABS(ix), &abs_features)) {
+      memset(&abs_features, 0, sizeof(abs_features));
+      if(ioctl(fd, EVIOCGABS(ix), &abs_features) < 0) {
         GST_WARNING("evdev EVIOCGABS ioctl : %s",g_strerror(errno));
       }
       else {
@@ -210,11 +208,13 @@ static gboolean register_abs_range_controls(const BtIcInputDevice * const self,i
            abs_features.fuzz  // tolerance
         );
         if(name) {
+          // BtIcAbsRangeControl *control;
           // create controller instances and register them
-          control = btic_abs_range_control_new(BTIC_DEVICE(self),name,
-            abs_features.minimum,abs_features.maximum,abs_features.flat);
-          key=(((gulong)EV_ABS)<<16)|(gulong)ix;
-          g_hash_table_insert(self->priv->controls,GUINT_TO_POINTER(key),(gpointer)control);
+          key=(((guint)EV_ABS)<<16)|(guint)ix;
+          /*control = */btic_abs_range_control_new(BTIC_DEVICE(self),name,key,
+            (glong)abs_features.minimum,
+            (glong)abs_features.maximum,
+            (glong)abs_features.flat);
         }
       }
     }
@@ -228,57 +228,58 @@ static gboolean register_controls(const BtIcInputDevice * const self) {
   guint ix;
   guint8 evtype_bitmask[EV_MAX/8 + 1];
 
+  GST_INFO("checking controls : %s",self->priv->devnode);
+
   if ((fd = open(self->priv->devnode, O_RDONLY)) < 0) {
     GST_WARNING("evdev open failed on device %s: %s",self->priv->devnode,g_strerror(errno));
     return(FALSE);
   }
   GST_INFO("opened device : %s",self->priv->devnode);
 
-  // @todo: query capabillities and register controllers
-
+  // query capabillities and register controllers
+  // FIXME: doing that for e.g. js0/js1 fails slowly 
   memset(evtype_bitmask, 0, sizeof(evtype_bitmask));
   if(ioctl(fd, EVIOCGBIT(0, sizeof(evtype_bitmask)), evtype_bitmask) < 0) {
     GST_WARNING("evdev ioctl : %s",g_strerror(errno));
-  }
-
-  // check supported event types
-  for (ix = 0; ix < EV_MAX; ix++) {
-    if (test_bit(ix, evtype_bitmask)) {
-      switch (ix) {
-        case EV_SYN:
-          break;
-        case EV_KEY:
-          GST_INFO("Keys or Buttons");
-          register_trigger_controls(self,fd);
-          break;
-        case EV_REL :
-          GST_INFO("Relative Axes");
-          break;
-        case EV_ABS:
-          GST_INFO("Absolute Axes");
-          register_abs_range_controls(self,fd);
-          break;
-        case EV_MSC:
-          GST_INFO("Miscellaneous");
-          break;
-        case EV_LED:
-          GST_INFO("LEDs");
-          break;
-        case EV_SND:
-          GST_INFO("Sounds");
-          break;
-        case EV_REP:
-          GST_INFO("Repeat");
-          break;
-        case EV_FF:
-          GST_INFO("Force Feedback");
-          break;
-        default:
-          GST_INFO("Unknown event type: 0x%04hx", ix);
-       }
+  } else {
+    // check supported event types
+    for (ix = 0; ix < EV_MAX; ix++) {
+      if (test_bit(ix, evtype_bitmask)) {
+        switch (ix) {
+          case EV_SYN:
+            break;
+          case EV_KEY:
+            GST_INFO("Keys or Buttons");
+            register_trigger_controls(self,fd);
+            break;
+          case EV_REL :
+            GST_INFO("Relative Axes");
+            break;
+          case EV_ABS:
+            GST_INFO("Absolute Axes");
+            register_abs_range_controls(self,fd);
+            break;
+          case EV_MSC:
+            GST_INFO("Miscellaneous");
+            break;
+          case EV_LED:
+            GST_INFO("LEDs");
+            break;
+          case EV_SND:
+            GST_INFO("Sounds");
+            break;
+          case EV_REP:
+            GST_INFO("Repeat");
+            break;
+          case EV_FF:
+            GST_INFO("Force Feedback");
+            break;
+          default:
+            GST_INFO("Unknown event type: 0x%04hx", ix);
+         }
+      }
     }
   }
-
   close(fd);
   return(TRUE);
 }
@@ -290,7 +291,7 @@ static gboolean io_handler(GIOChannel *channel,GIOCondition condition,gpointer u
   BtIcControl *control;
   GError *error=NULL;
   struct input_event ev;
-  gulong key;
+  guint key;
   gboolean res=TRUE;
 
   //GST_INFO("io handler : %d",condition);
@@ -301,8 +302,8 @@ static gboolean io_handler(GIOChannel *channel,GIOCondition condition,gpointer u
       g_error_free(error);
     }
     else {
-      key=(((gulong)ev.type)<<16)|(gulong)ev.code;
-      if((control=(BtIcControl *)g_hash_table_lookup(self->priv->controls,GUINT_TO_POINTER(key)))) {
+      key=(((guint)ev.type)<<16)|(guint)ev.code;
+      if((control=btic_device_get_control_by_id(BTIC_DEVICE(self),key))) {
         switch(ev.type) {
           case EV_KEY:
             //GST_INFO("key/button event: value %d, code 0x%x",ev.value,ev.code);
@@ -310,7 +311,7 @@ static gboolean io_handler(GIOChannel *channel,GIOCondition condition,gpointer u
             break;
           case EV_ABS:
             //GST_INFO("abs axis event: value %d, code 0x%x",ev.value,ev.code);
-            g_object_set(control,"value",ev.value,NULL);
+            g_object_set(control,"value",(glong)ev.value,NULL);
             break;
           default:
             GST_INFO("unhandled control event: type 0x%x, value %d, code 0x%x",ev.type,ev.value,ev.code);
@@ -341,14 +342,7 @@ static gboolean io_handler(GIOChannel *channel,GIOCondition condition,gpointer u
  * Returns: the new instance or %NULL in case of an error
  */
 BtIcInputDevice *btic_input_device_new(const gchar *udi,const gchar *name,const gchar *devnode) {
-  BtIcInputDevice *self=BTIC_INPUT_DEVICE(g_object_new(BTIC_TYPE_INPUT_DEVICE,"udi",udi,"name",name,"devnode",devnode,NULL));
-  if(!self) {
-    goto Error;
-  }
-  return(self);
-Error:
-  g_object_try_unref(self);
-  return(NULL);
+  return(BTIC_INPUT_DEVICE(g_object_new(BTIC_TYPE_INPUT_DEVICE,"udi",udi,"name",name,"devnode",devnode,NULL)));
 }
 
 //-- methods
@@ -421,6 +415,7 @@ static void btic_input_device_set_property(GObject * const object, const guint p
   switch (property_id) {
     case DEVICE_DEVNODE: {
       self->priv->devnode = g_value_dup_string(value);
+      //g_assert(!btic_device_has_controls(BTIC_DEVICE(self)));
       register_controls(self);
     } break;
     default: {
@@ -435,11 +430,11 @@ static void btic_input_device_dispose(GObject * const object) {
   return_if_disposed();
   self->priv->dispose_has_run = TRUE;
 
-  GST_DEBUG("!!!! self=%p, self->ref_ct=%d",self,G_OBJECT(self)->ref_count);
+  GST_DEBUG("!!!! self=%p, self->ref_ct=%d",self,G_OBJECT_REF_COUNT(self));
   btic_input_device_stop(self);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(btic_input_device_parent_class)->dispose(object);
   GST_DEBUG("  done");
 }
 
@@ -449,26 +444,20 @@ static void btic_input_device_finalize(GObject * const object) {
   GST_DEBUG("!!!! self=%p",self);
 
   g_free(self->priv->devnode);
-  g_hash_table_destroy(self->priv->controls);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(btic_input_device_parent_class)->finalize(object);
   GST_DEBUG("  done");
 }
 
-static void btic_input_device_init(const GTypeInstance * const instance, gconstpointer const g_class) {
-  BtIcInputDevice * const self = BTIC_INPUT_DEVICE(instance);
-
+static void btic_input_device_init(BtIcInputDevice *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BTIC_TYPE_INPUT_DEVICE, BtIcInputDevicePrivate);
-
-  self->priv->controls=g_hash_table_new_full(NULL,NULL,NULL,(GDestroyNotify)g_object_unref);
 }
 
 static void btic_input_device_class_init(BtIcInputDeviceClass * const klass) {
   GObjectClass * const gobject_class = G_OBJECT_CLASS(klass);
   BtIcDeviceClass * const bticdevice_class = BTIC_DEVICE_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtIcInputDevicePrivate));
 
   gobject_class->set_property = btic_input_device_set_property;
@@ -487,22 +476,3 @@ static void btic_input_device_class_init(BtIcInputDeviceClass * const klass) {
                                      G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 }
 
-GType btic_input_device_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      (guint16)(sizeof(BtIcInputDeviceClass)),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)btic_input_device_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      (guint16)(sizeof(BtIcInputDevice)),
-      0,   // n_preallocs
-      (GInstanceInitFunc)btic_input_device_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(BTIC_TYPE_DEVICE,"BtIcInputDevice",&info,0);
-  }
-  return type;
-}
