@@ -1,4 +1,4 @@
-/* $Id: cmd-application.c 2530 2009-07-23 15:15:28Z ensonic $
+/* $Id: cmd-application.c 3367 2011-05-13 07:24:26Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -50,14 +50,26 @@ struct _BtCmdApplicationPrivate {
   /* used to validate if dispose has run */
   gboolean dispose_has_run;
 
-  /* do no output on stdout */
+  /* no output on stdout */
   gboolean quiet;
+
+  /* error flag from bus handler */
+  gboolean has_error;
+
+  /* runtime data */
+  const BtSong *song;
+  gboolean res;
+
+  /* main loop */
+  GMainLoop *loop;
 };
 
-static BtApplicationClass *parent_class=NULL;
-
 static gboolean is_playing=FALSE;
 
+//-- the class
+
+G_DEFINE_TYPE (BtCmdApplication, bt_cmd_application, BT_TYPE_APPLICATION);
+
 //-- helper methods
 
 /*
@@ -66,47 +78,118 @@ static gboolean is_playing=FALSE;
  * playback status signal callback function
  */
 static void on_song_is_playing_notify(const BtSong *song, GParamSpec *arg, gpointer user_data) {
-  g_object_get(G_OBJECT(song),"is-playing",&is_playing,NULL);
+  g_object_get((gpointer)song,"is-playing",&is_playing,NULL);
   GST_INFO("%s playing - invoked per signal : song=%p, user_data=%p",
     (is_playing?"started":"stopped"),song,user_data);
 }
 
+
+static void on_song_error(const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
+  const BtCmdApplication *self=BT_CMD_APPLICATION(user_data);
+  GError *err = NULL;
+  gchar *dbg = NULL;
+
+  GST_INFO("received %s bus message from %s",
+    GST_MESSAGE_TYPE_NAME(message), GST_OBJECT_NAME(GST_MESSAGE_SRC(message)));
+
+  // @todo: check domain and code
+  gst_message_parse_error(message, &err, &dbg);
+  GST_WARNING_OBJECT(GST_MESSAGE_SRC(message),"ERROR: %s (%s)", err->message, (dbg ? dbg : "no details"));
+
+  g_error_free(err);
+  g_free(dbg);
+
+  self->priv->has_error = TRUE;
+}
+
+static void on_song_warning(const GstBus * const bus, GstMessage *message, gconstpointer user_data) {
+  //const BtCmdApplication *self=BT_CMD_APPLICATION(user_data);
+  GError *err = NULL;
+  gchar *dbg = NULL;
+
+  GST_INFO("received %s bus message from %s",
+    GST_MESSAGE_TYPE_NAME(message), GST_OBJECT_NAME(GST_MESSAGE_SRC(message)));
+
+  // @todo: check domain and code
+  gst_message_parse_warning(message, &err, &dbg);
+  GST_WARNING_OBJECT(GST_MESSAGE_SRC(message),"WARNING: %s (%s)", err->message, (dbg ? dbg : "no details"));
+
+  g_error_free(err);
+  g_free(dbg);
+}
+
+/*
+ * bt_cmd_application_song_init:
+ *
+ * Create the song an attach error handlers to the bus.
+ */
+static BtSong *bt_cmd_application_song_init(const BtCmdApplication *self) {
+  BtSong *song;
+  GstBin *bin;
+  GstBus *bus;
+
+  song=bt_song_new(BT_APPLICATION(self));
+
+  g_object_get((gpointer)song,"bin",&bin,NULL);
+  bus=gst_element_get_bus(GST_ELEMENT(bin));
+  g_signal_connect(bus, "message::error", G_CALLBACK(on_song_error), (gpointer)self);
+  g_signal_connect(bus, "message::warning", G_CALLBACK(on_song_warning), (gpointer)self);
+  //g_signal_connect(bus, "message::element", G_CALLBACK(on_song_element_msg), (gpointer)self);
+
+  gst_object_unref(bus);
+  gst_object_unref(bin);
+  return(song);
+}
+
 /*
  * bt_cmd_application_play_song:
  *
  * start playback, used by play and record
  */
-static gboolean bt_cmd_application_play_song(const BtCmdApplication *self,const BtSong *song) {
+static void bt_cmd_application_idle_play_song(const BtCmdApplication *self) {
   gboolean res=FALSE;
+  const BtSong *song=self->priv->song;
   BtSequence *sequence=NULL;
   gulong cmsec,csec,cmin,tmsec,tsec,tmin;
   gulong length,pos=0;
   GstClockTime bar_time;
+  GstBin *bin;
 
   // DEBUG
   //bt_song_write_to_highlevel_dot_file(song);
+  //bt_song_write_to_lowlevel_dot_file(song);
   // DEBUG
 
-  g_object_get(G_OBJECT(song),"sequence",&sequence,NULL);
-  g_object_get(G_OBJECT(sequence),"length",&length,NULL);
+  g_object_get((gpointer)song,"sequence",&sequence,"bin",&bin,NULL);
+  if(!GST_BIN_NUMCHILDREN(bin)) {
+    GST_INFO("song is empty");
+    goto Error;
+  }
+  g_object_get(sequence,"length",&length,NULL);
 
   bar_time=bt_sequence_get_bar_time(sequence);
   tmsec=(gulong)((length*bar_time)/G_USEC_PER_SEC);
   tmin=(gulong)(tmsec/60000);tmsec-=(tmin*60000);
   tsec=(gulong)(tmsec/ 1000);tmsec-=(tsec* 1000);
-  
+
   // connection play and stop signals
-  g_signal_connect(G_OBJECT(song), "notify::is-playing", G_CALLBACK(on_song_is_playing_notify), (gpointer)self);
+  g_signal_connect((gpointer)song, "notify::is-playing", G_CALLBACK(on_song_is_playing_notify), (gpointer)self);
   if(bt_song_play(song)) {
     GST_INFO("playing is starting, is_playing=%d",is_playing);
+    /* FIXME: this is a bad idea, now that we have a main loop
+     * we should start a g_timeout_add() from on_song_is_playing_notify()
+     * and quit the main-loop there on eos
+     */
     while(!is_playing) {
       while(g_main_context_pending(NULL)) g_main_context_iteration(NULL,FALSE);
       g_usleep(G_USEC_PER_SEC/10);
     }
     GST_INFO("playing has started, is_playing=%d",is_playing);
-    while(is_playing && (pos<length)) {
-      bt_song_update_playback_position(song);
-      g_object_get(G_OBJECT(song),"play-pos",&pos,NULL);
+    while(is_playing && (pos<length) && !self->priv->has_error) {
+      if (!bt_song_update_playback_position(song)) {
+        bt_song_stop(song);
+      }
+      g_object_get((gpointer)song,"play-pos",&pos,NULL);
 
       if(!self->priv->quiet) {
         // get song->play-pos and print progress
@@ -130,8 +213,20 @@ static gboolean bt_cmd_application_play_song(const BtCmdApplication *self,const
   }
   is_playing=FALSE;
 Error:
+  gst_object_unref(bin);
   g_object_unref(sequence);
-  return(res);
+  self->priv->res=res;
+  g_main_loop_quit(self->priv->loop);
+}
+
+static gboolean bt_cmd_application_play_song(const BtCmdApplication *self,const BtSong *song) {
+
+  self->priv->song=song;
+
+  g_idle_add((GSourceFunc)bt_cmd_application_idle_play_song,(gpointer)self);
+  g_main_loop_run(self->priv->loop);
+
+  return(self->priv->res);
 }
 
 /*
@@ -143,7 +238,6 @@ static gboolean bt_cmd_application_prepare_encoding(const BtCmdApplication *self
   gboolean ret=FALSE;
   BtSetup *setup;
   BtMachine *machine;
-  BtSinkBin *sink_bin;
   BtSinkBinRecordFormat format;
   gchar *lc_file_name,*file_name=NULL;
   GEnumClass *enum_class;
@@ -151,7 +245,7 @@ static gboolean bt_cmd_application_prepare_encoding(const BtCmdApplication *self
   guint i;
   gboolean matched=FALSE;
 
-  g_object_get(G_OBJECT(song),"setup",&setup,NULL);
+  g_object_get((gpointer)song,"setup",&setup,NULL);
 
   lc_file_name=g_ascii_strdown(output_file_name,-1);
 
@@ -172,7 +266,10 @@ static gboolean bt_cmd_application_prepare_encoding(const BtCmdApplication *self
 
   // lookup the audio-sink machine and change mode
   if((machine=bt_setup_get_machine_by_type(setup,BT_TYPE_SINK_MACHINE))) {
-    g_object_get(G_OBJECT(machine),"machine",&sink_bin,NULL);
+    GstElement *convert;
+    BtSinkBin *sink_bin;
+
+    g_object_get(machine,"machine",&sink_bin,"adder-convert",&convert,NULL);
 
     /* @todo eventually have a method for the sink bin to only update once
      * after the changes, right now keep the order as it is, as sink-bin only
@@ -183,9 +280,13 @@ static gboolean bt_cmd_application_prepare_encoding(const BtCmdApplication *self
       "record-format",format,
       "record-file-name",(file_name?file_name:output_file_name),
       NULL);
-    ret=TRUE;
+    /* see comments in edit/render-progress.c */
+    g_object_set(convert,"dithering",2,"noise-shaping",3,NULL);
+
+    ret=!self->priv->has_error;
 
     g_free(file_name);
+    gst_object_unref(convert);
     gst_object_unref(sink_bin);
     g_object_unref(machine);
   }
@@ -229,8 +330,8 @@ gboolean bt_cmd_application_play(const BtCmdApplication *self, const gchar *inpu
   GST_INFO("application.play(%s) launched",input_file_name);
 
   // prepare song and song-io
-  song=bt_song_new(BT_APPLICATION(self));
-  if(!(loader=bt_song_io_make(input_file_name))) {
+  song=bt_cmd_application_song_init(self);
+  if(!(loader=bt_song_io_from_file(input_file_name))) {
     goto Error;
   }
 
@@ -241,10 +342,10 @@ gboolean bt_cmd_application_play(const BtCmdApplication *self, const gchar *inpu
     BtWavetable *wavetable;
     GList *node,*missing_machines,*missing_waves;
 
-    g_object_get(G_OBJECT(song),"setup",&setup,"wavetable",&wavetable,NULL);
+    g_object_get((gpointer)song,"setup",&setup,"wavetable",&wavetable,NULL);
     // get missing element info
-    g_object_get(G_OBJECT(setup),"missing-machines",&missing_machines,NULL);
-    g_object_get(G_OBJECT(wavetable),"missing-waves",&missing_waves,NULL);
+    g_object_get(setup,"missing-machines",&missing_machines,NULL);
+    g_object_get(wavetable,"missing-waves",&missing_waves,NULL);
 
     if(missing_machines || missing_waves) {
       printf("could not load all of song\"%s\"\n",input_file_name);
@@ -264,6 +365,7 @@ gboolean bt_cmd_application_play(const BtCmdApplication *self, const gchar *inpu
     g_object_unref(setup);
     g_object_unref(wavetable);
 
+    GST_INFO("start playback");
     if(bt_cmd_application_play_song(self,song)) {
       res=TRUE;
     }
@@ -308,11 +410,14 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
   if (!BT_IS_STRING(output_file_name)) {
     output_file=stdout;
   } else {
-    output_file = fopen(output_file_name,"wb");
+    if(!(output_file = fopen(output_file_name,"wb"))) {
+      GST_ERROR("cannot open output file \"%s\"",output_file_name);
+      goto Error;
+    }
   }
   // prepare song and song-io
-  song=bt_song_new(BT_APPLICATION(self));
-  if(!(loader=bt_song_io_make(input_file_name))) {
+  song=bt_cmd_application_song_init(self);
+  if(!(loader=bt_song_io_from_file(input_file_name))) {
     goto Error;
   }
 
@@ -338,13 +443,13 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
     //bt_song_write_to_highlevel_dot_file(song);
     //DEBUG
 
-    g_object_get(G_OBJECT(song),"song-info",&song_info,"sequence",&sequence,"setup",&setup,"wavetable",&wavetable,NULL);
+    g_object_get((gpointer)song,"song-info",&song_info,"sequence",&sequence,"setup",&setup,"wavetable",&wavetable,NULL);
     // get missing element info
-    g_object_get(G_OBJECT(setup),"missing-machines",&missing_machines,NULL);
-    g_object_get(G_OBJECT(wavetable),"missing-waves",&missing_waves,NULL);
+    g_object_get(setup,"missing-machines",&missing_machines,NULL);
+    g_object_get(wavetable,"missing-waves",&missing_waves,NULL);
 
     // print some info about the song
-    g_object_get(G_OBJECT(song_info),
+    g_object_get(song_info,
       "name",&name,"author",&author,"genre",&genre,"info",&info,
       "bpm",&beats_per_minute,"tpb",&ticks_per_beat,
       "create-dts",&create_dts,"change-dts",&change_dts,
@@ -358,7 +463,7 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
     g_fprintf(output_file,"song.song_info.created: \"%s\"\n",create_dts);g_free(create_dts);
     g_fprintf(output_file,"song.song_info.changed: \"%s\"\n",change_dts);g_free(change_dts);
     // print some info about the sequence
-    g_object_get(G_OBJECT(sequence),"length",&length,"tracks",&tracks,"loop",&loop,"loop-start",&loop_start,"loop-end",&loop_end,NULL);
+    g_object_get(sequence,"length",&length,"tracks",&tracks,"loop",&loop,"loop-start",&loop_start,"loop-end",&loop_end,NULL);
     g_fprintf(output_file,"song.sequence.length: %lu\n",length);
     g_fprintf(output_file,"song.sequence.tracks: %lu\n",tracks);
     g_fprintf(output_file,"song.sequence.loop: %s\n",(loop?"yes":"no"));
@@ -371,13 +476,14 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
     g_fprintf(output_file,"song.sequence.playing_time: %02lu:%02lu.%03lu\n",min,sec,msec);
 
     // print some statistics about the song (number of machines, wires, patterns)
-    g_object_get(G_OBJECT(setup),"machines",&machines,"wires",&wires,NULL);
+    g_object_get(setup,"machines",&machines,"wires",&wires,NULL);
     g_fprintf(output_file,"song.setup.number_of_machines: %u\n",g_list_length(machines));
     g_fprintf(output_file,"song.setup.number_of_wires: %u\n",g_list_length(wires));
     for(node=machines;node;node=g_list_next(node)) {
-      g_object_get(G_OBJECT(node->data),"patterns",&patterns,NULL);
+      g_object_get(node->data,"patterns",&patterns,NULL);
       // @todo: this include internal ones
       n_patterns+=g_list_length(patterns);
+      g_list_foreach(patterns,(GFunc)g_object_unref,NULL);
       g_list_free(patterns);
     }
     g_fprintf(output_file,"song.setup.number_of_patterns: %lu\n",n_patterns);
@@ -387,20 +493,20 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
     }
     g_list_free(machines);
     g_list_free(wires);
-    g_object_get(G_OBJECT(wavetable),"waves",&waves,NULL);
+    g_object_get(wavetable,"waves",&waves,NULL);
     g_fprintf(output_file,"song.wavetable.number_of_waves: %u\n",g_list_length(waves));
     g_fprintf(output_file,"song.wavetable.number_of_missing_waves: %u\n",g_list_length(missing_waves));
     for(node=missing_waves;node;node=g_list_next(node)) {
       g_fprintf(output_file,"  %s\n",(gchar *)(node->data));
     }
     g_list_free(waves);
-    g_object_get(G_OBJECT(self),"bin",&bin,NULL);
+    g_object_get((gpointer)self,"bin",&bin,NULL);
     g_fprintf(output_file,"app.bin.number_of_elements: %u\n",GST_BIN_NUMCHILDREN(bin));
     gst_object_unref(bin);
 
     // lookup the audio-sink machine and print some info about it
     if((machine=bt_setup_get_machine_by_type(setup,BT_TYPE_SINK_MACHINE))) {
-      g_object_get(G_OBJECT(machine),"id",&id,"plugin_name",&name,NULL);
+      g_object_get(machine,"id",&id,"plugin_name",&name,NULL);
       g_fprintf(output_file,"machine.id: \"%s\"\n",id);g_free(id);
       g_fprintf(output_file,"machine.plugin_name: \"%s\"\n",name);g_free(name);
       g_object_unref(machine);
@@ -412,7 +518,7 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
     g_object_unref(setup);
     g_object_unref(wavetable);
     res=TRUE;
-    GST_INFO("finished succesfully");
+    GST_INFO("finished successfully");
   }
   else {
     GST_ERROR("could not load song \"%s\"",input_file_name);
@@ -421,7 +527,7 @@ gboolean bt_cmd_application_info(const BtCmdApplication *self, const gchar *inpu
 Error:
   g_object_try_unref(song);
   g_object_try_unref(loader);
-  if (BT_IS_STRING(output_file_name)) {
+  if (output_file) {
     fclose(output_file);
   }
   return(res);
@@ -448,11 +554,11 @@ gboolean bt_cmd_application_convert(const BtCmdApplication *self, const gchar *i
   g_return_val_if_fail(BT_IS_STRING(output_file_name),FALSE);
 
   // prepare song and song-io
-  song=bt_song_new(BT_APPLICATION(self));
-  if(!(loader=bt_song_io_make(input_file_name))) {
+  song=bt_cmd_application_song_init(self);
+  if(!(loader=bt_song_io_from_file(input_file_name))) {
     goto Error;
   }
-  if(!(saver=bt_song_io_make(output_file_name))) {
+  if(!(saver=bt_song_io_from_file(output_file_name))) {
     goto Error;
   }
 
@@ -500,8 +606,8 @@ gboolean bt_cmd_application_encode(const BtCmdApplication *self, const gchar *in
   GST_INFO("application.play launched");
 
   // prepare song and song-io
-  song=bt_song_new(BT_APPLICATION(self));
-  if(!(loader=bt_song_io_make(input_file_name))) {
+  song=bt_cmd_application_song_init(self);
+  if(!(loader=bt_song_io_from_file(input_file_name))) {
     goto Error;
   }
 
@@ -509,6 +615,7 @@ gboolean bt_cmd_application_encode(const BtCmdApplication *self, const gchar *in
 
   if(bt_song_io_load(loader,song)) {
     if(bt_cmd_application_prepare_encoding(self,song,output_file_name)) {
+      GST_INFO("start encoding");
       if(bt_cmd_application_play_song(self,song)) {
         res=TRUE;
       }
@@ -536,30 +643,8 @@ Error:
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_cmd_application_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
-  BtCmdApplication *self = BT_CMD_APPLICATION(object);
-  return_if_disposed();
-  switch (property_id) {
-    case CMD_APP_QUIET: {
-      g_value_set_boolean(value, self->priv->quiet);
-    } break;
-    default: {
-       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
 /* sets the given properties for this object */
-static void bt_cmd_application_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
+static void bt_cmd_application_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
   BtCmdApplication *self = BT_CMD_APPLICATION(object);
   return_if_disposed();
   switch (property_id) {
@@ -572,20 +657,6 @@ static void bt_cmd_application_set_property(GObject      *object,
   }
 }
 
-#if 0
-static GObject* bt_cmd_application_construct (GType type, guint n_construct_params, GObjectConstructParam *construct_params) {
-  GObject *self;
-
-  //GST_DEBUG("<<<");
-  self = G_OBJECT_CLASS (parent_class)->constructor (type, n_construct_params, construct_params);
-  GST_INFO("new cmd app instantiated");
-  // do post construct here
-  //GST_DEBUG(">>>");
-  return(self);
-}
-#endif
-
-
 static void bt_cmd_application_dispose(GObject *object) {
   BtCmdApplication *self = BT_CMD_APPLICATION(object);
 
@@ -594,7 +665,7 @@ static void bt_cmd_application_dispose(GObject *object) {
 
   GST_DEBUG("!!!! self=%p",self);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_cmd_application_parent_class)->dispose(object);
 }
 
 static void bt_cmd_application_finalize(GObject *object) {
@@ -602,28 +673,28 @@ static void bt_cmd_application_finalize(GObject *object) {
 
   GST_DEBUG("!!!! self=%p",self);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  // this would exit the mainloop from a different thread :/
+  //g_main_loop_quit(self->priv->loop);
+  g_main_loop_unref(self->priv->loop);
+
+  G_OBJECT_CLASS(bt_cmd_application_parent_class)->finalize(object);
 }
 
-static void bt_cmd_application_init(GTypeInstance *instance, gpointer g_class) {
-  BtCmdApplication *self = BT_CMD_APPLICATION(instance);
-
+static void bt_cmd_application_init(BtCmdApplication *self) {
   GST_DEBUG("!!!! self=%p",self);
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_CMD_APPLICATION, BtCmdApplicationPrivate);
+
+  self->priv->loop=g_main_loop_new(NULL,FALSE);
+  //self->priv->loop_thread=g_thread_create((GThreadFunc)g_main_loop_run,self->priv->loop,FALSE,NULL);
 }
 
 static void bt_cmd_application_class_init(BtCmdApplicationClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
   GST_DEBUG("!!!!");
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtCmdApplicationPrivate));
 
-#if 0
-  gobject_class->constructor  = bt_cmd_application_construct;
-#endif
   gobject_class->set_property = bt_cmd_application_set_property;
-  gobject_class->get_property = bt_cmd_application_get_property;
   gobject_class->dispose      = bt_cmd_application_dispose;
   gobject_class->finalize     = bt_cmd_application_finalize;
 
@@ -632,26 +703,7 @@ static void bt_cmd_application_class_init(BtCmdApplicationClass *klass) {
                                      "quiet prop",
                                      "tell wheter the app should do output or not",
                                      FALSE,
-                                     G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
+                                     G_PARAM_WRITABLE|G_PARAM_STATIC_STRINGS));
 
 }
 
-GType bt_cmd_application_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtCmdApplicationClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_cmd_application_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtCmdApplication),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_cmd_application_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(BT_TYPE_APPLICATION,"BtCmdApplication",&info,0);
-  }
-  return type;
-}
