@@ -1,4 +1,4 @@
-/* $Id: edit-application.c 2510 2009-07-06 23:10:03Z ensonic $
+/* $Id: edit-application.c 3648 2011-09-20 20:29:03Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -25,6 +25,19 @@
  *
  * Opens the #BtMainWindow and provide application level function like load,
  * save, run and exit.
+ *
+ * It also provides functions to invoke some dialogs like about and tips.
+ *
+ * The application instance will have exactly one active 
+ * #BtEditApplication:song at a time. I tracks undo/redo-able changes to the
+ * song via #BtChangeLog and simple flagged changes via the
+ * #BtEditApplication:unsaved property.
+ */
+/* @todo: unsaved flagging
+ * - various parts in the UI call bt_edit_application_set_song_unsaved()
+ * - this sometimes happens as a side effect from doing something undo/redoable
+ * - can we include flagging unsaved in the changelog group, so hat if we undo
+ *   it the flagging gets undone?
  */
 
 #define BT_EDIT
@@ -39,7 +52,8 @@
 enum {
   EDIT_APPLICATION_SONG=1,
   EDIT_APPLICATION_MAIN_WINDOW,
-  EDIT_APPLICATION_IC_REGISTRY
+  EDIT_APPLICATION_IC_REGISTRY,
+  EDIT_APPLICATION_UNSAVED
 };
 
 // this needs to be here because of gtk-doc and unit-tests
@@ -56,6 +70,12 @@ struct _BtEditApplicationPrivate {
   /* the top-level window of our app */
   BtMainWindow *main_window;
 
+  /* editor change log for undo/redo */
+  BtChangeLog *change_log;
+  /* flag for smaller changes to make the song-saveable */
+  gboolean unsaved;
+  gboolean need_dts_reset;
+
   /* remote playback controller */
   BtPlaybackControllerSocket *pb_controller;
 
@@ -63,7 +83,11 @@ struct _BtEditApplicationPrivate {
   BtIcRegistry *ic_registry;
 };
 
-static BtApplicationClass *parent_class=NULL;
+static BtEditApplication *singleton=NULL;
+
+//-- the class
+
+G_DEFINE_TYPE (BtEditApplication, bt_edit_application, BT_TYPE_APPLICATION);
 
 //-- event handler
 
@@ -72,9 +96,6 @@ static void on_songio_status_changed(BtSongIO *songio,GParamSpec *arg,gpointer u
   BtMainStatusbar *statusbar;
   gchar *str;
 
-  g_assert(BT_IS_SONG_IO(songio));
-  g_assert(user_data);
-
   /* @idea: bind properties */
   g_object_get(self->priv->main_window,"statusbar",&statusbar,NULL);
   g_object_get(songio,"status",&str,NULL);
@@ -84,6 +105,21 @@ static void on_songio_status_changed(BtSongIO *songio,GParamSpec *arg,gpointer u
   g_free(str);
 }
 
+static void on_changelog_can_undo_changed(BtChangeLog *change_log,GParamSpec *arg,gpointer user_data) {
+  BtEditApplication *self=BT_EDIT_APPLICATION(user_data);
+
+  if(self->priv->need_dts_reset) {
+    BtSongInfo *song_info;
+
+    self->priv->need_dts_reset=FALSE;
+    g_object_get(self->priv->song,"song-info",&song_info,NULL);
+    // this updates the time-stamp (we need that to show the since when we have
+    // unsaved changes, if some one closes the song)
+    g_object_set(song_info,"change-dts",NULL,NULL);
+    g_object_unref(song_info);
+  }
+}
+
 //-- helper methods
 
 /*
@@ -141,8 +177,8 @@ static gboolean bt_edit_application_check_missing(const BtEditApplication *self)
   if(missing) {
     GtkWidget *dialog;
 
-    if((dialog=GTK_WIDGET(bt_missing_framework_elements_dialog_new(self,missing_core_elements, missing_edit_elements)))) {
-      gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(self->priv->main_window));
+    if((dialog=GTK_WIDGET(bt_missing_framework_elements_dialog_new(missing_core_elements, missing_edit_elements)))) {
+      bt_edit_application_attach_child_window(self,GTK_WINDOW(dialog));
       gtk_widget_show_all(dialog);
 
       gtk_dialog_run(GTK_DIALOG(dialog));
@@ -150,7 +186,8 @@ static gboolean bt_edit_application_check_missing(const BtEditApplication *self)
       gtk_widget_destroy(dialog);
     }
   }
-  g_list_free(missing_core_elements);
+  // don't free. its static
+  //g_list_free(missing_core_elements);
   g_list_free(missing_edit_elements);
   return(res);
 }
@@ -168,30 +205,41 @@ static gboolean bt_edit_application_check_missing(const BtEditApplication *self)
 static gboolean bt_edit_application_run_ui(const BtEditApplication *self) {
   BtSettings *settings;
   guint version;
-  gboolean res;
+  gboolean res,show_tips;
 
   g_assert(self);
   g_assert(self->priv->main_window);
 
   GST_INFO("application.run_ui launched");
 
-  g_object_get(G_OBJECT(self),"settings",&settings,NULL);
-  g_object_get(G_OBJECT(settings),"news-seen",&version,NULL);
+  g_object_get((gpointer)self,"settings",&settings,NULL);
+  g_object_get(settings,"news-seen",&version,"show-tips",&show_tips,NULL);
 
   if(PACKAGE_VERSION_NUMBER>version) {
     // show about
     bt_edit_application_show_about(self);
     // store new version
     version=PACKAGE_VERSION_NUMBER;
-    g_object_set(G_OBJECT(settings),"news-seen",version,NULL);
+    g_object_set(settings,"news-seen",version,NULL);
+  } else if(show_tips) {
+    // show tip-of-the-day
+    bt_edit_application_show_tip(self);
   }
   g_object_unref(settings);
 
   // check for missing elements
-  if((res=bt_edit_application_check_missing(self))) {
-    res=bt_main_window_run(self->priv->main_window);
-  }
+  if(!(res=bt_edit_application_check_missing(self)))
+    goto Error;
 
+  // check for recoverable songs
+  bt_edit_application_crash_log_recover(self);
+
+  GST_INFO("before running the UI");
+  gtk_main();
+  GST_INFO("after running the UI");
+  res=TRUE;
+
+Error:
   GST_INFO("application.run_ui finished : %d",res);
   return(res);
 }
@@ -201,9 +249,9 @@ static gboolean bt_edit_application_run_ui(const BtEditApplication *self) {
 /**
  * bt_edit_application_new:
  *
- * Create a new instance
+ * Create a new instance on first call and return a reference later on.
  *
- * Returns: the new instance or %NULL in case of an error
+ * Returns: the new signleton instance
  */
 BtEditApplication *bt_edit_application_new(void) {
   return(BT_EDIT_APPLICATION(g_object_new(BT_TYPE_EDIT_APPLICATION,NULL)));
@@ -225,8 +273,10 @@ gboolean bt_edit_application_new_song(const BtEditApplication *self) {
   BtSong *song;
   BtSetup *setup;
   BtSequence *sequence;
+  BtSongInfo *song_info;
   BtMachine *machine;
   gchar *id;
+  gulong bars;
   GError *err=NULL;
 
   g_return_val_if_fail(BT_IS_EDIT_APPLICATION(self),FALSE);
@@ -234,16 +284,17 @@ gboolean bt_edit_application_new_song(const BtEditApplication *self) {
   // create new song
   song=bt_song_new(BT_APPLICATION(self));
 
-  g_object_get(song,"setup",&setup,"sequence",&sequence,NULL);
-  // add some initial timelines
-  g_object_set(sequence,"length",SEQUENCE_ROW_ADDITION_INTERVAL,NULL);
+  g_object_get(song,"setup",&setup,"sequence",&sequence,"song-info",&song_info,NULL);
+  // make initial song length 4 timelines
+  g_object_get(song_info,"bars",&bars,NULL);
+  g_object_set(sequence,"length",bars*4,NULL);
   // add audiosink
   id=bt_setup_get_unique_machine_id(setup,"master");
   machine=BT_MACHINE(bt_sink_machine_new(song,id,&err));
   if(err==NULL) {
     GHashTable *properties;
 
-    GST_DEBUG("sink-machine-refs: %d",(G_OBJECT(machine))->ref_count);
+    GST_DEBUG("sink-machine=%p,ref_ct=%d",machine,G_OBJECT_REF_COUNT(machine));
     g_object_get(machine,"properties",&properties,NULL);
     if(properties) {
       gchar str[G_ASCII_DTOSTR_BUF_SIZE];
@@ -251,16 +302,15 @@ gboolean bt_edit_application_new_song(const BtEditApplication *self) {
       g_hash_table_insert(properties,g_strdup("ypos"),g_strdup(g_ascii_dtostr(str,G_ASCII_DTOSTR_BUF_SIZE,0.0)));
     }
     if(bt_machine_enable_input_post_level(machine)) {
-      GST_DEBUG("sink-machine-refs: %d",(G_OBJECT(machine))->ref_count);
+      GST_DEBUG("sink-machine-ref_ct=%d",G_OBJECT_REF_COUNT(machine));
       // set new song in application
-      bt_song_set_unsaved(song,FALSE);
-      g_object_set(G_OBJECT(self),"song",song,NULL);
+      g_object_set((gpointer)self,"song",song,NULL);
       res=TRUE;
     }
     else {
       GST_WARNING("Can't add input level/gain element in sink machine");
     }
-    GST_DEBUG("sink-machine-refs: %d",(G_OBJECT(machine))->ref_count);
+    GST_DEBUG("sink-machine-ref_ct=%d",G_OBJECT_REF_COUNT(machine));
   }
   else {
     GST_WARNING("Can't create sink machine: %s",err->message);
@@ -269,7 +319,11 @@ gboolean bt_edit_application_new_song(const BtEditApplication *self) {
   g_object_unref(machine);
   g_free(id);
 
+  self->priv->unsaved=FALSE;
+  g_object_notify(G_OBJECT(self),"unsaved");
+
   // release references
+  g_object_unref(song_info);
   g_object_unref(setup);
   g_object_unref(sequence);
   g_object_unref(song);
@@ -294,41 +348,37 @@ gboolean bt_edit_application_load_song(const BtEditApplication *self,const char
 
   GST_INFO("song name = %s",file_name);
 
-  if((loader=bt_song_io_make(file_name))) {
-    GdkCursor *cursor=gdk_cursor_new(GDK_WATCH);
-    GdkWindow *window=GTK_WIDGET(self->priv->main_window)->window;
+  if((loader=bt_song_io_from_file(file_name))) {
     BtSetup *setup;
     BtWavetable *wavetable;
     GList *missing_machines,*missing_waves;
 
-    gdk_window_set_cursor(window,cursor);
-    gdk_cursor_unref(cursor);
-    gtk_widget_set_sensitive(GTK_WIDGET(self->priv->main_window),FALSE);
+    bt_edit_application_ui_lock(self);
+    g_signal_connect(loader,"notify::status",G_CALLBACK(on_songio_status_changed),(gpointer)self);
 
-    g_signal_connect(G_OBJECT(loader),"notify::status",G_CALLBACK(on_songio_status_changed),(gpointer)self);
-    while(gtk_events_pending()) gtk_main_iteration();
-
-    // create new song
+    // create new song and release the previous one
     song=bt_song_new(BT_APPLICATION(self));
-    g_object_set(G_OBJECT(self),"song",NULL,NULL);
-    
+    g_object_set((gpointer)self,"song",NULL,NULL);
+
+#ifdef USE_DEBUG
     // do sanity check that bin is empty
     {
       GstBin *bin;
-      g_object_get(G_OBJECT(self),"bin",&bin,NULL);
+      g_object_get((gpointer)self,"bin",&bin,NULL);
 
-      if(GST_BIN_NUMCHILDREN(bin)) {  
+      if(GST_BIN_NUMCHILDREN(bin)) {
         GST_WARNING("bin.num_children=%d has left-overs",GST_BIN_NUMCHILDREN(bin));
         GList *node=GST_BIN_CHILDREN(bin);
 
         while(node) {
-          GST_INFO_OBJECT(node->data,"removing object (ref-ct=%d)",G_OBJECT(node->data)->ref_count);
+          GST_INFO_OBJECT(node->data,"removing object (ref-ct=%d)",G_OBJECT_REF_COUNT(node->data));
           gst_bin_remove(bin,GST_ELEMENT(node->data));
           node=GST_BIN_CHILDREN(bin);
         }
       }
       gst_object_unref(bin);
     }
+#endif
 
     if(bt_song_io_load(loader,song)) {
       BtMachine *machine;
@@ -341,7 +391,7 @@ gboolean bt_edit_application_load_song(const BtEditApplication *self,const char
           //bt_song_write_to_highlevel_dot_file(song);
           // DEBUG
           // set new song
-          g_object_set(G_OBJECT(self),"song",song,NULL);
+          g_object_set((gpointer)self,"song",song,NULL);
           res=TRUE;
           GST_INFO("new song activated");
         }
@@ -359,19 +409,21 @@ gboolean bt_edit_application_load_song(const BtEditApplication *self,const char
     else {
       GST_ERROR("could not load song \"%s\"",file_name);
     }
-    gtk_widget_set_sensitive(GTK_WIDGET(self->priv->main_window),TRUE);
-    gdk_window_set_cursor(window,NULL);
+    self->priv->unsaved=FALSE;
+    g_object_notify(G_OBJECT(self),"unsaved");
+
+    bt_edit_application_ui_unlock(self);
 
     // get missing element info
     g_object_get(song,"setup",&setup,"wavetable",&wavetable,NULL);
-    g_object_get(G_OBJECT(setup),"missing-machines",&missing_machines,NULL);
-    g_object_get(G_OBJECT(wavetable),"missing-waves",&missing_waves,NULL);
+    g_object_get(setup,"missing-machines",&missing_machines,NULL);
+    g_object_get(wavetable,"missing-waves",&missing_waves,NULL);
     // tell about missing machines and/or missing waves
     if(missing_machines || missing_waves) {
       GtkWidget *dialog;
 
-      if((dialog=GTK_WIDGET(bt_missing_song_elements_dialog_new(self,missing_machines,missing_waves)))) {
-        gtk_window_set_transient_for(GTK_WINDOW(dialog),GTK_WINDOW(self->priv->main_window));
+      if((dialog=GTK_WIDGET(bt_missing_song_elements_dialog_new(missing_machines,missing_waves)))) {
+        bt_edit_application_attach_child_window(self,GTK_WINDOW(dialog));
         gtk_widget_show_all(dialog);
 
         gtk_dialog_run(GTK_DIALOG(dialog));
@@ -406,22 +458,15 @@ gboolean bt_edit_application_save_song(const BtEditApplication *self,const char
 
   GST_INFO("song name = %s",file_name);
 
-  if((saver=bt_song_io_make(file_name))) {
-    GdkCursor *cursor=gdk_cursor_new(GDK_WATCH);
-    GdkWindow *window=GTK_WIDGET(self->priv->main_window)->window;
+  if((saver=bt_song_io_from_file(file_name))) {
     BtSongInfo *song_info;
     gchar *old_file_name=NULL,*bak_file_name=NULL;
 
-    gdk_window_set_cursor(window,cursor);
-    gdk_cursor_unref(cursor);
-    
-    gtk_widget_set_sensitive(GTK_WIDGET(self->priv->main_window),FALSE);
+    bt_edit_application_ui_lock(self);
+    g_signal_connect(saver,"notify::status",G_CALLBACK(on_songio_status_changed),(gpointer)self);
 
-    g_signal_connect(G_OBJECT(saver),"notify::status",G_CALLBACK(on_songio_status_changed),(gpointer)self);
-    while(gtk_events_pending()) gtk_main_iteration();
-    
-    g_object_get(G_OBJECT(self->priv->song),"song-info",&song_info,NULL);
-    g_object_get(G_OBJECT(song_info),"file-name",&old_file_name,NULL);
+    g_object_get(self->priv->song,"song-info",&song_info,NULL);
+    g_object_get(song_info,"file-name",&old_file_name,NULL);
     g_object_unref(song_info);
 
     /* save file saving (bak files)
@@ -438,7 +483,7 @@ gboolean bt_edit_application_save_song(const BtEditApplication *self,const char
      *     - move to <existing>.bak
      *       - save newfile
      *       - if saving failed, move <existing>.bak back
-     * save-as 
+     * save-as
      *   new file (!old_file_name)
      *     like save of a new-file
      *   existing file
@@ -458,7 +503,7 @@ gboolean bt_edit_application_save_song(const BtEditApplication *self,const char
     if(g_file_test(file_name,G_FILE_TEST_EXISTS)) {
       bak_file_name=g_strconcat(file_name,".bak",NULL);
       g_rename(file_name,bak_file_name);
-    }  
+    }
     if(bt_song_io_save(saver,self->priv->song)) {
       res=TRUE;
       if(!old_file_name || strcmp(old_file_name,file_name)) {
@@ -476,9 +521,11 @@ gboolean bt_edit_application_save_song(const BtEditApplication *self,const char
       }
     }
     GST_INFO("saving done");
-
-    gtk_widget_set_sensitive(GTK_WIDGET(self->priv->main_window),TRUE);
-    gdk_window_set_cursor(window,NULL);
+    self->priv->unsaved=FALSE;
+    g_object_notify(G_OBJECT(self),"unsaved");
+    self->priv->need_dts_reset=TRUE;
+    
+    bt_edit_application_ui_unlock(self);
 
     g_free(old_file_name);
     g_free(bak_file_name);
@@ -494,9 +541,9 @@ gboolean bt_edit_application_save_song(const BtEditApplication *self,const char
  * bt_edit_application_run:
  * @self: the application instance to run
  *
- * start the gtk based editor application
+ * Start the gtk based editor application
  *
- * Returns: true for success
+ * Returns: %TRUE for success
  */
 gboolean bt_edit_application_run(const BtEditApplication *self) {
   gboolean res=FALSE;
@@ -541,6 +588,31 @@ gboolean bt_edit_application_load_and_run(const BtEditApplication *self, const g
   return(res);
 }
 
+/**
+ * bt_edit_application_quit:
+ * @self: the application instance to quit
+ *
+ * End the application. Eventualy asks the user for confirmation.
+ *
+ * Returns: %TRUE it ending the application was confirmed
+ */
+gboolean bt_edit_application_quit(const BtEditApplication *self) {
+  if(bt_main_window_check_quit(self->priv->main_window)) {
+    BtSettings *settings;
+    gint x, y, w, h;
+
+    // remember window size
+    g_object_get((gpointer)self,"settings",&settings,NULL);
+    gtk_window_get_position(GTK_WINDOW(self->priv->main_window),&x,&y);
+    gtk_window_get_size(GTK_WINDOW(self->priv->main_window),&w,&h);
+    g_object_set(settings,"window-xpos",x,"window-ypos",y,"window-width",w,"window-height",h,NULL);
+    g_object_unref(settings);
+
+    return(TRUE);
+  }
+  return(FALSE);
+}
+
 /**
  * bt_edit_application_show_about:
  * @self: the application instance
@@ -550,31 +622,148 @@ gboolean bt_edit_application_load_and_run(const BtEditApplication *self, const g
 void bt_edit_application_show_about(const BtEditApplication *self) {
   GtkWidget *dialog;
 
-  GST_INFO("menu about event occurred");
-
-  if((dialog=GTK_WIDGET(bt_about_dialog_new(self)))) {
-    // set parent relationship
-    gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(self->priv->main_window));
-    gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
-    gtk_widget_show_all(GTK_WIDGET(dialog));
+  if((dialog=GTK_WIDGET(bt_about_dialog_new()))) {
+    bt_edit_application_attach_child_window(self,GTK_WINDOW(dialog));
+    gtk_widget_show_all(dialog);
 
     gtk_dialog_run(GTK_DIALOG(dialog));
     gtk_widget_destroy(dialog);
   }
 }
 
+/**
+ * bt_edit_application_show_tip:
+ * @self: the application instance
+ *
+ * Shows the tip of the day window
+ *
+ * Since: 0.6
+ */
+void bt_edit_application_show_tip(const BtEditApplication *self) {
+  GtkWidget *dialog;
+
+  if((dialog=GTK_WIDGET(bt_tip_dialog_new()))) {
+    bt_edit_application_attach_child_window(self,GTK_WINDOW(dialog));
+    gtk_widget_show_all(dialog);
+
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+  }
+}
+
+/**
+ * bt_edit_application_crash_log_recover:
+ * @self: the application instance
+ *
+ * Shows the crash-log recover window if we have pending crash logs.
+ *
+ * Since: 0.6
+ */
+void bt_edit_application_crash_log_recover(const BtEditApplication *self) {
+  GList *crash_logs;
+
+  g_object_get(self->priv->change_log,"crash-logs",&crash_logs, NULL);
+  if(crash_logs) {
+    GtkWidget *dialog;
+
+    GST_INFO("have found crash logs");
+    if((dialog=GTK_WIDGET(bt_crash_recover_dialog_new(crash_logs)))) {
+      bt_edit_application_attach_child_window(self,GTK_WINDOW(dialog));
+      gtk_widget_show_all(dialog);
+
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(dialog);
+    }
+  }
+}
+
+
+/**
+ * bt_edit_application_attach_child_window:
+ * @self: the application instance
+ * @window: a child window (e.g. dialog)
+ *
+ * The parent and transient relation ship to the applications main-window.
+ *
+ * Since: 0.6
+ */
+void bt_edit_application_attach_child_window(const BtEditApplication *self, GtkWindow *window) {
+  gtk_window_set_transient_for(window, GTK_WINDOW(self->priv->main_window));
+  gtk_window_set_destroy_with_parent(window, TRUE);
+}
+
+
+/**
+ * bt_edit_application_ui_lock:
+ * @self: the application instance
+ *
+ * Sets the main window insensitive and show a wait cursor.
+ */
+void bt_edit_application_ui_lock(const BtEditApplication *self) {
+  GdkCursor *cursor=gdk_cursor_new(GDK_WATCH);
+
+  gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(self->priv->main_window)),cursor);
+  gdk_cursor_unref(cursor);
+  gtk_widget_set_sensitive(GTK_WIDGET(self->priv->main_window),FALSE);
+
+  while(gtk_events_pending()) gtk_main_iteration();
+}
+
+/**
+ * bt_edit_application_ui_unlock:
+ * @self: the application instance
+ *
+ * Sets the main window sensitive again and unset the wait cursor.
+ */
+void bt_edit_application_ui_unlock(const BtEditApplication *self) {
+  gtk_widget_set_sensitive(GTK_WIDGET(self->priv->main_window),TRUE);
+  gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(self->priv->main_window)),NULL);
+}
+
+/**
+ * bt_edit_application_is_song_unsaved:
+ * @self: the application instance
+ *
+ * Check if the song has unsaved changes.
+ *
+ * Returns: %TRUE if there are pending changes
+ */
+gboolean bt_edit_application_is_song_unsaved(const BtEditApplication *self) {
+  gboolean unsaved=self->priv->unsaved;
+
+  if(!unsaved)
+    g_object_get(self->priv->change_log,"can-undo",&unsaved,NULL);
+  return(unsaved);
+}
+
+/**
+ * bt_edit_application_set_song_unsaved:
+ * @self: the application instance
+ *
+ * Flag unsaved changes in the applications song.
+ */
+void bt_edit_application_set_song_unsaved(const BtEditApplication *self) {
+  /* this is not succesfully preventing setting the flag when we create an
+   * undo-able change
+  gboolean can_undo;
+
+  g_object_get(self->priv->change_log,"can-undo",&can_undo,NULL);
+  if((!self->priv->unsaved) && (!can_undo)) {
+  */
+  if(!self->priv->unsaved) {
+    self->priv->unsaved=TRUE;
+    g_object_notify(G_OBJECT(self),"unsaved");
+  }
+
+}
+
 //-- wrapper
 
 //-- default signal handler
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_edit_application_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
+static void bt_edit_application_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
   BtEditApplication *self = BT_EDIT_APPLICATION(object);
   return_if_disposed();
   switch (property_id) {
@@ -587,56 +776,46 @@ static void bt_edit_application_get_property(GObject      *object,
     case EDIT_APPLICATION_IC_REGISTRY: {
       g_value_set_object(value, self->priv->ic_registry);
     } break;
+    case EDIT_APPLICATION_UNSAVED: {
+      g_value_set_boolean(value, self->priv->unsaved);
+    } break;
     default: {
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
     } break;
   }
 }
 
-/* sets the given properties for this object */
-static void bt_edit_application_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
+static void bt_edit_application_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
   BtEditApplication *self = BT_EDIT_APPLICATION(object);
   return_if_disposed();
   switch (property_id) {
     case EDIT_APPLICATION_SONG: {
-      // DEBUG
+      BtSong *song=self->priv->song;
+
+#ifdef USE_DEBUG
       GstElement *bin;
       g_object_get(self,"bin",&bin,NULL);
       GST_INFO("bin->num_children=%d",GST_BIN_NUMCHILDREN(bin));
-      // DEBUG
+      gst_object_unref(bin);
 
-      if(self->priv->song) {
-        if(G_OBJECT(self->priv->song)->ref_count>1) {
-          GST_WARNING("old song->ref_ct=%d!",G_OBJECT(self->priv->song)->ref_count);
+      if(song) {
+        if(G_OBJECT_REF_COUNT(song)>1) {
+          GST_WARNING("old song->ref_ct=%d!",G_OBJECT_REF_COUNT(song));
         }
         else {
-          GST_INFO("old song->ref_ct=%d",G_OBJECT(self->priv->song)->ref_count);
+          GST_INFO("old song->ref_ct=%d",G_OBJECT_REF_COUNT(song));
         }
-        g_object_unref(self->priv->song);
-        // DEBUG - if new song is NULL, it should be empty now
-#if 0
-        {
-          gint num=GST_BIN_NUMCHILDREN(bin);
-          GList *node=GST_BIN_CHILDREN(bin);
-
-          GST_INFO("bin->num_children=%d",num);
-          for(;node;node=g_list_next(node)) {
-            GST_INFO("  %p, ref_ct=%d, '%s'",node->data,G_OBJECT(node->data)->ref_count,GST_ELEMENT_NAME(node->data));
-          }
-        }
-#endif
-        // DEBUG
+        g_object_unref(song);
       }
-      // DEBUG
-      gst_object_unref(bin);
-      // DEBUG
-
+#else
+      g_object_try_unref(song);
+#endif
       self->priv->song=BT_SONG(g_value_dup_object(value));
-      if(self->priv->song) GST_DEBUG("new song: %p, song->ref_ct=%d",self->priv->song,G_OBJECT(self->priv->song)->ref_count);
+      GST_DEBUG("new song: %p, song->ref_ct=%d",self->priv->song,G_OBJECT_REF_COUNT(self->priv->song));
+    } break;
+    case EDIT_APPLICATION_UNSAVED: {
+      self->priv->unsaved = g_value_get_boolean(value);
+      GST_INFO("set the unsaved flag for the song",self->priv->unsaved);
     } break;
     default: {
       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
@@ -644,38 +823,43 @@ static void bt_edit_application_set_property(GObject      *object,
   }
 }
 
-static GObject* bt_edit_application_construct (GType type, guint n_construct_params, GObjectConstructParam *construct_params) {
-  BtEditApplication *self;
-  
-  //GST_DEBUG("<<<");
-  self = BT_EDIT_APPLICATION (G_OBJECT_CLASS (parent_class)->constructor (type, n_construct_params, construct_params));
-  GST_INFO("new edit app instantiated");
-  // create or ref the shared ui ressources
-  if(!(self->priv->ui_resources=bt_ui_resources_new())) {
-    goto Error;
-  }
-  // create the playback controller
-  self->priv->pb_controller=bt_playback_controller_socket_new(self);
-  // create the interaction controller registry
-  self->priv->ic_registry=btic_registry_new();
-  // create main window
-  GST_INFO("new edit app created, app->ref_ct=%d",G_OBJECT(self)->ref_count);
-  if(!(self->priv->main_window=bt_main_window_new(self))) {
-    goto Error;
-  }
-  // warning: dereferencing type-punned pointer will break strict-aliasing rules
-  g_object_add_weak_pointer(G_OBJECT(self->priv->main_window),(gpointer*)(gpointer)&self->priv->main_window);
+static GObject* bt_edit_application_constructor(GType type, guint n_construct_params, GObjectConstructParam *construct_params) {
+  GObject *object;
+
+  if(G_UNLIKELY(!singleton)) {
+    object=G_OBJECT_CLASS(bt_edit_application_parent_class)->constructor(type,n_construct_params,construct_params);
+    singleton=BT_EDIT_APPLICATION(object);
+    g_object_add_weak_pointer(object,(gpointer*)(gpointer)&singleton);
+
+    //GST_DEBUG("<<<");
+    GST_INFO("new edit app instantiated");
+    // create or ref the shared ui ressources
+    singleton->priv->ui_resources=bt_ui_resources_new();
+    // create the playback controller
+    singleton->priv->pb_controller=bt_playback_controller_socket_new();
+    // create the interaction controller registry
+    singleton->priv->ic_registry=btic_registry_new();
+    // create the editor change log
+    singleton->priv->change_log=bt_change_log_new();
+    g_signal_connect(singleton->priv->change_log,"notify::can-undo",G_CALLBACK(on_changelog_can_undo_changed),(gpointer)singleton);
+
+    // create main window
+    GST_INFO("new edit app created, app->ref_ct=%d",G_OBJECT_REF_COUNT(singleton));
+    singleton->priv->main_window=bt_main_window_new();
+
+    // warning: dereferencing type-punned pointer will break strict-aliasing rules
+    g_object_add_weak_pointer(G_OBJECT(singleton->priv->main_window),(gpointer*)(gpointer)&singleton->priv->main_window);
 #ifdef USE_HILDON
-  hildon_program_add_window(HILDON_PROGRAM(hildon_program_get_instance()),
-    HILDON_WINDOW(self->priv->main_window));
+    hildon_program_add_window(HILDON_PROGRAM(hildon_program_get_instance()),
+      HILDON_WINDOW(singleton->priv->main_window));
 #endif
-  GST_INFO("new edit app window created, app->ref_ct=%d",G_OBJECT(self)->ref_count);
-  //GST_DEBUG(">>>");
-  return(G_OBJECT(self));
-Error:
-  g_object_try_unref(self);
-  //GST_DEBUG(">>>");
-  return NULL;
+    GST_INFO("new edit app created, app->ref_ct=%d",G_OBJECT_REF_COUNT(singleton));
+    //GST_DEBUG(">>>");
+  }
+  else {
+    object=g_object_ref(singleton);
+  }
+  return object;
 }
 
 
@@ -690,16 +874,17 @@ static void bt_edit_application_dispose(GObject *object) {
    * strong reference to the app.
    * Solution 1: Only use weak refs when reffing upstream objects
    */
-  GST_DEBUG("!!!! self=%p, self->ref_ct=%d",self,G_OBJECT(self)->ref_count);
+  GST_DEBUG("!!!! self=%p, self->ref_ct=%d",self,G_OBJECT_REF_COUNT(self));
 
   if(self->priv->song) {
-    GST_INFO("song->ref_ct=%d",G_OBJECT(self->priv->song)->ref_count);
+    GST_INFO("song->ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->song));
     bt_song_stop(self->priv->song);
     g_object_unref(self->priv->song);
+    self->priv->song=NULL;
   }
 
   if(self->priv->main_window) {
-    GST_INFO("main_window->ref_ct=%d",G_OBJECT(self->priv->main_window)->ref_count);
+    GST_INFO("main_window->ref_ct=%d",G_OBJECT_REF_COUNT(self->priv->main_window));
     //main-menu.c::on_menu_quit_activate
     gtk_widget_destroy(GTK_WIDGET(self->priv->main_window));
     // we get this if we unref
@@ -711,45 +896,33 @@ static void bt_edit_application_dispose(GObject *object) {
   g_object_try_unref(self->priv->ui_resources);
   g_object_try_unref(self->priv->pb_controller);
   g_object_try_unref(self->priv->ic_registry);
+  g_object_try_unref(self->priv->change_log);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_edit_application_parent_class)->dispose(object);
   GST_DEBUG("  done");
 }
 
-static void bt_edit_application_finalize(GObject *object) {
-  //BtEditApplication *self = BT_EDIT_APPLICATION(object);
-
-  //GST_DEBUG("!!!! self=%p",self);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
-  GST_DEBUG("  done");
-}
-
-static void bt_edit_application_init(GTypeInstance *instance, gpointer g_class) {
-  BtEditApplication *self = BT_EDIT_APPLICATION(instance);
-
+static void bt_edit_application_init(BtEditApplication *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_EDIT_APPLICATION, BtEditApplicationPrivate);
 }
 
 static void bt_edit_application_class_init(BtEditApplicationClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtEditApplicationPrivate));
 
-  gobject_class->constructor  = bt_edit_application_construct;
+  gobject_class->constructor  = bt_edit_application_constructor;
   gobject_class->set_property = bt_edit_application_set_property;
   gobject_class->get_property = bt_edit_application_get_property;
   gobject_class->dispose      = bt_edit_application_dispose;
-  gobject_class->finalize     = bt_edit_application_finalize;
 
   klass->song_changed = NULL;
 
   g_object_class_install_property(gobject_class,EDIT_APPLICATION_SONG,
                                   g_param_spec_object("song",
-                                     "song construct prop",
-                                     "the song object, the wire belongs to",
+                                     "song prop",
+                                     "the current song object",
                                      BT_TYPE_SONG, /* object type */
                                      G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 
@@ -766,24 +939,13 @@ static void bt_edit_application_class_init(BtEditApplicationClass *klass) {
                                      "the interaction controller registry of this application",
                                      BTIC_TYPE_REGISTRY, /* object type */
                                      G_PARAM_READABLE|G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property(gobject_class,EDIT_APPLICATION_UNSAVED,
+                                  g_param_spec_boolean("unsaved",
+                                     "unsaved prop",
+                                     "tell whether the current state of the song has been saved",
+                                     TRUE,
+                                     G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
+
 }
 
-GType bt_edit_application_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtEditApplicationClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_edit_application_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtEditApplication),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_edit_application_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(BT_TYPE_APPLICATION,"BtEditApplication",&info,0);
-  }
-  return type;
-}
