@@ -1,4 +1,4 @@
-/* $Id: machine-actions.h 2127 2008-12-07 11:44:16Z ensonic $
+/* $Id: machine-actions.h 2966 2010-09-06 13:07:39Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -24,7 +24,6 @@
 
 #include "main-window.h"
 
-void bt_machine_action_help(GtkWidget *widget, GstElement *machine);
-void bt_machine_action_about(GstElement *machine,BtMainWindow *main_window);
+void bt_machine_action_about(BtMachine *machine,BtMainWindow *main_window);
 
 #endif // BT_MACHINE_ACTIONS_H
