@@ -1,4 +1,4 @@
-/* $Id: machine-preferences-dialog.c 2513 2009-07-16 12:44:40Z ensonic $
+/* $Id: machine-preferences-dialog.c 3738 2011-12-04 14:27:11Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -27,6 +27,14 @@
  */
 
 /* @todo: filter certain properties (tempo iface, ...)
+ *
+ * @todo: we have a few notifies, but not for all types
+ * - do we need them at all? who else could change things?
+ *
+ * @todo: we should only be able to have one preferences for each machien type
+ *
+ * @todo: save the chosen settings somewhere
+ * - gconf would need some sort of schema
  */
 
 #define BT_EDIT
@@ -37,8 +45,7 @@
 //-- property ids
 
 enum {
-  MACHINE_PREFERENCES_DIALOG_APP=1,
-  MACHINE_PREFERENCES_DIALOG_MACHINE
+  MACHINE_PREFERENCES_DIALOG_MACHINE=1
 };
 
 struct _BtMachinePreferencesDialogPrivate {
@@ -52,19 +59,11 @@ struct _BtMachinePreferencesDialogPrivate {
   BtMachine *machine;
 };
 
-static GtkDialogClass *parent_class=NULL;
-
 static GQuark widget_parent_quark=0;
 
-//-- event handler helper
+//-- the class
 
-static void mark_song_as_changed(const BtMachinePreferencesDialog *self) {
-  BtSong *song;
-  
-  g_object_get(self->priv->app,"song",&song,NULL);
-  bt_song_set_unsaved(song,TRUE);
-  g_object_unref(song);
-}
+G_DEFINE_TYPE (BtMachinePreferencesDialog, bt_machine_preferences_dialog, GTK_TYPE_WINDOW);
 
 //-- event handler
 
@@ -72,14 +71,10 @@ static void on_range_property_notify(const GstElement *machine,GParamSpec *prope
   GtkWidget *widget=GTK_WIDGET(user_data);
   gdouble value;
 
-  g_assert(user_data);
-
   //GST_INFO("preferences value notify received for: '%s'",property->name);
 
-  g_object_get(G_OBJECT(machine),property->name,&value,NULL);
-  //gdk_threads_enter();
+  g_object_get((gpointer)machine,property->name,&value,NULL);
   gtk_range_set_value(GTK_RANGE(widget),value);
-  //gdk_threads_leave();
 }
 
 static void on_double_entry_property_notify(const GstElement *machine,GParamSpec *property,gpointer user_data) {
@@ -87,15 +82,11 @@ static void on_double_entry_property_notify(const GstElement *machine,GParamSpec
   gdouble value;
   gchar *str_value;
 
-  g_assert(user_data);
-
   //GST_INFO("preferences value notify received for: '%s'",property->name);
 
-  g_object_get(G_OBJECT(machine),property->name,&value,NULL);
-  str_value=g_strdup_printf("%f",value);
-  //gdk_threads_enter();
+  g_object_get((gpointer)machine,property->name,&value,NULL);
+  str_value=g_strdup_printf("%7.2lf",value);
   gtk_entry_set_text(GTK_ENTRY(widget),str_value);
-  //gdk_threads_leave();
   g_free(str_value);
 }
 
@@ -105,9 +96,7 @@ static void on_combobox_property_notify(const GstElement *machine,GParamSpec *pr
   GtkTreeModel *store;
   GtkTreeIter iter;
 
-  g_assert(user_data);
-
-  g_object_get(G_OBJECT(machine),property->name,&nvalue,NULL);
+  g_object_get((gpointer)machine,property->name,&nvalue,NULL);
   store=gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
   gtk_tree_model_get_iter_first(store,&iter);
   do {
@@ -118,16 +107,35 @@ static void on_combobox_property_notify(const GstElement *machine,GParamSpec *pr
   gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget),&iter);
 }
 
+
+static void on_entry_property_changed(GtkEditable *editable,gpointer user_data) {
+  GstElement *machine=GST_ELEMENT(user_data);
+  const gchar *name=gtk_widget_get_name(GTK_WIDGET(editable));
+  BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(g_object_get_qdata(G_OBJECT(editable),widget_parent_quark));
+
+  //GST_INFO("preferences value change received for: '%s'",name);
+  g_object_set(machine,name,gtk_entry_get_text(GTK_ENTRY(editable)),NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
+}
+
+static void on_checkbox_property_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
+  GstElement *machine=GST_ELEMENT(user_data);
+  const gchar *name=gtk_widget_get_name(GTK_WIDGET(togglebutton));
+  BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(g_object_get_qdata(G_OBJECT(togglebutton),widget_parent_quark));
+
+  //GST_INFO("preferences value change received for: '%s'",name);
+  g_object_set(machine,name,gtk_toggle_button_get_active(togglebutton),NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
+}
+
 static void on_range_property_changed(GtkRange *range,gpointer user_data) {
   GstElement *machine=GST_ELEMENT(user_data);
   const gchar *name=gtk_widget_get_name(GTK_WIDGET(range));
   BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(g_object_get_qdata(G_OBJECT(range),widget_parent_quark));
 
-  g_assert(user_data);
-
   //GST_INFO("preferences value change received for: '%s'",name);
   g_object_set(machine,name,gtk_range_get_value(range),NULL);
-  mark_song_as_changed(self);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_double_entry_property_changed(GtkEditable *editable,gpointer user_data) {
@@ -136,12 +144,10 @@ static void on_double_entry_property_changed(GtkEditable *editable,gpointer user
   const gchar *name=gtk_widget_get_name(GTK_WIDGET(editable));
   BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(g_object_get_qdata(G_OBJECT(editable),widget_parent_quark));
 
-  g_assert(user_data);
-
   //GST_INFO("preferences value change received for: '%s'",name);
   value=g_ascii_strtod(gtk_entry_get_text(GTK_ENTRY(editable)),NULL);
   g_object_set(machine,name,value,NULL);
-  mark_song_as_changed(self);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_spinbutton_property_changed(GtkSpinButton *spinbutton,gpointer user_data) {
@@ -150,12 +156,10 @@ static void on_spinbutton_property_changed(GtkSpinButton *spinbutton,gpointer us
   const gchar *name=gtk_widget_get_name(GTK_WIDGET(spinbutton));
   BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(g_object_get_qdata(G_OBJECT(spinbutton),widget_parent_quark));
 
-  g_assert(user_data);
-
   GST_INFO("preferences value change received for: '%s'",name);
   value=gtk_spin_button_get_value_as_int(spinbutton);
   g_object_set(machine,name,value,NULL);
-  mark_song_as_changed(self);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_combobox_property_changed(GtkComboBox *combobox, gpointer user_data) {
@@ -166,8 +170,6 @@ static void on_combobox_property_changed(GtkComboBox *combobox, gpointer user_da
   GtkTreeModel *store;
   GtkTreeIter iter;
 
-  g_assert(user_data);
-
   GST_INFO("preferences value change received for: '%s'",name);
   //value=gtk_combo_box_get_active(combobox);
   store=gtk_combo_box_get_model(combobox);
@@ -175,7 +177,7 @@ static void on_combobox_property_changed(GtkComboBox *combobox, gpointer user_da
     gtk_tree_model_get(store,&iter,0,&value,-1);
     g_object_set(machine,name,value,NULL);
   }
-  mark_song_as_changed(self);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 /*
@@ -185,7 +187,7 @@ static void on_combobox_property_changed(GtkComboBox *combobox, gpointer user_da
  */
 static void on_table_size_request(GtkWidget *widget,GtkRequisition *requisition,gpointer user_data) {
   //BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(user_data);
-  GtkWidget *parent=widget->parent->parent;
+  GtkWidget *parent=gtk_widget_get_parent(gtk_widget_get_parent(widget));
   gint height=requisition->height,width=-1;
   gint max_height=gdk_screen_get_height(gdk_screen_get_default());
   gint available_heigth;
@@ -204,6 +206,16 @@ static void on_table_size_request(GtkWidget *widget,GtkRequisition *requisition,
   gtk_widget_set_size_request(parent,width,height + 2);
 }
 
+static void on_machine_id_changed(const BtMachine *machine,GParamSpec *arg,gpointer user_data) {
+  BtMachinePreferencesDialog *self=BT_MACHINE_PREFERENCES_DIALOG(user_data);
+  gchar *id,*title;
+
+  g_object_get((GObject *)machine,"id",&id,NULL);
+  title=g_strdup_printf(_("%s preferences"),id);
+  gtk_window_set_title(GTK_WINDOW(self),title);
+  g_free(id);g_free(title);
+}
+
 //-- helper methods
 
 static gboolean skip_property(GstElement *element,GParamSpec *pspec) {
@@ -217,28 +229,44 @@ static gboolean skip_property(GstElement *element,GParamSpec *pspec) {
   else if(pspec->owner_type==GST_TYPE_BASE_SRC) return(TRUE);
   else if(pspec->owner_type==GST_TYPE_BASE_TRANSFORM) return(TRUE);
   else if(pspec->owner_type==GST_TYPE_BASE_SINK) return(TRUE);
+  else if(pspec->owner_type==GST_TYPE_BIN) return(TRUE);
   // skip know interface properties (tempo, childbin)
   else if(pspec->owner_type==GSTBT_TYPE_CHILD_BIN) return(TRUE);
   else if(pspec->owner_type==GSTBT_TYPE_HELP) return(TRUE);
   else if(pspec->owner_type==GSTBT_TYPE_TEMPO) return(TRUE);
   
   GST_INFO("property: %s, owner-type: %s",pspec->name,g_type_name(pspec->owner_type));
-  
+
   return(FALSE);
 }
 
-static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferencesDialog *self) {
+#define _MAKE_SPIN_BUTTON(t,T,p)                                               \
+	case G_TYPE_ ## T: {                                                         \
+		GParamSpec ## p *p=G_PARAM_SPEC_ ## T(property);                           \
+		g ## t value;                                                              \
+		gdouble step;                                                              \
+                                                                               \
+		g_object_get(machine,property->name,&value,NULL);                          \
+		step=(gdouble)(p->maximum-p->minimum)/1024.0;                              \
+		spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(                         \
+			  (gdouble)value,(gdouble)p->minimum, (gdouble)p->maximum,1.0,step,0.0));\
+		widget1=gtk_spin_button_new(spin_adjustment,1.0,0);                        \
+		gtk_widget_set_name(GTK_WIDGET(widget1),property->name);                   \
+		g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);  \
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);        \
+		widget2=NULL;                                                              \
+		g_signal_connect(widget1,"value-changed",G_CALLBACK(on_spinbutton_property_changed),(gpointer)machine); \
+	} break;
+
+
+static void bt_machine_preferences_dialog_init_ui(const BtMachinePreferencesDialog *self) {
   BtMainWindow *main_window;
   GtkWidget *label,*widget1,*widget2,*table,*scrolled_window;
   GtkAdjustment *spin_adjustment;
-  gchar *id,*title;
   GdkPixbuf *window_icon=NULL;
   GstElement *machine;
   GParamSpec **properties,*property;
   guint i,k,props,number_of_properties;
-#if !GTK_CHECK_VERSION(2,12,0)
-  GtkTooltips *tips=gtk_tooltips_new();
-#endif
 
   gtk_widget_set_name(GTK_WIDGET(self),"machine preferences");
 
@@ -256,15 +284,14 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
   ////gtk_widget_set_size_request(GTK_WIDGET(self),300,200);
   //gtk_window_set_default_size(GTK_WINDOW(self),300,-1);
 
-  // set a title
-  g_object_get(self->priv->machine,"id",&id,"machine",&machine,NULL);
-  title=g_strdup_printf(_("%s preferences"),id);
-  gtk_window_set_title(GTK_WINDOW(self),title);
-  g_free(id);g_free(title);
+  g_object_get(self->priv->machine,"machine",&machine,NULL);
+  // set dialog title
+  on_machine_id_changed(self->priv->machine,NULL,(gpointer)self);
 
   // get machine properties
   if((properties=g_object_class_list_properties(G_OBJECT_CLASS(GST_ELEMENT_GET_CLASS(machine)),&number_of_properties))) {
     gchar *signal_name;
+    gchar *tool_tip_text;
     GType param_type,base_type;
 
     GST_INFO("machine has %d properties",number_of_properties);
@@ -283,7 +310,7 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
       // add machine preferences into the table
       table=gtk_table_new(/*rows=*/props/*+1 (space eater)*/,/*columns=*/3,/*homogenous=*/FALSE);
       gtk_container_set_border_width(GTK_CONTAINER(table),6);
-      g_signal_connect(G_OBJECT(table),"size-request",G_CALLBACK(on_table_size_request),(gpointer)self);
+      g_signal_connect(table,"size-request",G_CALLBACK(on_table_size_request),(gpointer)self);
 
       for(i=0,k=0;i<number_of_properties;i++) {
         property=properties[i];
@@ -291,9 +318,12 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
         if(skip_property(machine,property)) continue;
 
         GST_INFO("property %p has name '%s'",property,property->name);
+        tool_tip_text=(gchar *)g_param_spec_get_blurb(property);
+
         // get name
         label=gtk_label_new(property->name);
         gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
+        gtk_widget_set_tooltip_text(label,tool_tip_text);
         gtk_table_attach(GTK_TABLE(table),label, 0, 1, k, k+1, GTK_FILL,GTK_SHRINK, 2,1);
 
         param_type=property->value_type;
@@ -306,129 +336,41 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
 
             g_object_get(machine,property->name,&value,NULL);
             widget1=gtk_entry_new();
+            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
+            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
             gtk_entry_set_text(GTK_ENTRY(widget1),safe_string(value));g_free(value);
             widget2=NULL;
+            // connect handlers
+            g_signal_connect(widget1, "changed", G_CALLBACK(on_entry_property_changed), (gpointer)machine);
           } break;
           case G_TYPE_BOOLEAN: {
             gboolean value;
 
             g_object_get(machine,property->name,&value,NULL);
             widget1=gtk_check_button_new();
+            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
+            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
             gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget1),value);
             widget2=NULL;
-          } break;
-          case G_TYPE_INT: {
-            GParamSpecInt *int_property=G_PARAM_SPEC_INT(property);
-            gint value;
-            gdouble step;
-
-            g_object_get(machine,property->name,&value,NULL);
-            step=(gdouble)(int_property->maximum-int_property->minimum)/1024.0;
-            GST_INFO("  int : %d...%d, step=%f",int_property->minimum,int_property->maximum,step);
-            spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new((gdouble)value,(gdouble)int_property->minimum, (gdouble)int_property->maximum,1.0,step,0.0));
-            widget1=gtk_spin_button_new(spin_adjustment,1.0,0);
-            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
-            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);
-            widget2=NULL;
             // connect handlers
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_spinbutton_property_changed), (gpointer)machine);
-          } break;
-          case G_TYPE_UINT: {
-            GParamSpecUInt *uint_property=G_PARAM_SPEC_UINT(property);
-            guint value;
-            gdouble step;
-
-            g_object_get(machine,property->name,&value,NULL);
-            step=(gdouble)(uint_property->maximum-uint_property->minimum)/1024.0;
-            GST_INFO("  uint : %u...%u, step=%f",uint_property->minimum,uint_property->maximum,step);
-            spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new((gdouble)value,(gdouble)uint_property->minimum, (gdouble)uint_property->maximum,1.0,step,0.0));
-            widget1=gtk_spin_button_new(spin_adjustment,1.0,0);
-            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
-            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);
-            widget2=NULL;
-            // connect handlers
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_spinbutton_property_changed), (gpointer)machine);
-          } break;
-          case G_TYPE_LONG: {
-            GParamSpecLong *long_property=G_PARAM_SPEC_LONG(property);
-            glong value;
-            gdouble step;
-
-            g_object_get(machine,property->name,&value,NULL);
-            step=(gdouble)(long_property->maximum-long_property->minimum)/1024.0;
-            GST_INFO("  long : %ld...%ld, step=%f",long_property->minimum,long_property->maximum,step);
-            spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new((gdouble)value,(gdouble)long_property->minimum, (gdouble)long_property->maximum,1.0,step,0.0));
-            widget1=gtk_spin_button_new(spin_adjustment,1.0,0);
-            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
-            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);
-            widget2=NULL;
-            // connect handlers
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_spinbutton_property_changed), (gpointer)machine);
-          } break;
-          case G_TYPE_ULONG: {
-            GParamSpecULong *ulong_property=G_PARAM_SPEC_ULONG(property);
-            gulong value;
-            gdouble step;
-
-            g_object_get(machine,property->name,&value,NULL);
-            step=(gdouble)(ulong_property->maximum-ulong_property->minimum)/1024.0;
-            GST_INFO("  ulong : %lu...%lu, step=%f",ulong_property->minimum,ulong_property->maximum,step);
-            spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new((gdouble)value,(gdouble)ulong_property->minimum, (gdouble)ulong_property->maximum,1.0,step,0.0));
-            widget1=gtk_spin_button_new(spin_adjustment,1.0,0);
-            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
-            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);
-            widget2=NULL;
-            // connect handlers
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_spinbutton_property_changed), (gpointer)machine);
-          } break;
-          case G_TYPE_INT64: {
-            GParamSpecInt64 *int64_property=G_PARAM_SPEC_INT64(property);
-            gint64 value;
-            gdouble step;
-
-            g_object_get(machine,property->name,&value,NULL);
-            step=(gdouble)(int64_property->maximum-int64_property->minimum)/1024.0;
-            GST_INFO("  int : %"G_GINT64_FORMAT"...%"G_GINT64_FORMAT", step=%f",int64_property->minimum,int64_property->maximum,step);
-            spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new((gdouble)value,(gdouble)int64_property->minimum, (gdouble)int64_property->maximum,1.0,step,0.0));
-            widget1=gtk_spin_button_new(spin_adjustment,1.0,0);
-            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
-            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);
-            widget2=NULL;
-            // connect handlers
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_spinbutton_property_changed), (gpointer)machine);
-          } break;
-          case G_TYPE_UINT64: {
-            GParamSpecUInt64 *uint64_property=G_PARAM_SPEC_UINT64(property);
-            guint64 value;
-            gdouble step;
-
-            g_object_get(machine,property->name,&value,NULL);
-            step=(gdouble)(uint64_property->maximum-uint64_property->minimum)/1024.0;
-            GST_INFO("  uint : %"G_GUINT64_FORMAT"...%"G_GUINT64_FORMAT", step=%f",uint64_property->minimum,uint64_property->maximum,step);
-            spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new((gdouble)value,(gdouble)uint64_property->minimum, (gdouble)uint64_property->maximum,1.0,step,0.0));
-            widget1=gtk_spin_button_new(spin_adjustment,1.0,0);
-            gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
-            g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget1),(gdouble)value);
-            widget2=NULL;
-            // connect handlers
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_spinbutton_property_changed), (gpointer)machine);
+            g_signal_connect(widget1, "toggled", G_CALLBACK(on_checkbox_property_toggled), (gpointer)machine);
           } break;
+          _MAKE_SPIN_BUTTON(int,INT,Int)
+          _MAKE_SPIN_BUTTON(uint,UINT,UInt)
+          _MAKE_SPIN_BUTTON(int64,INT64,Int64)
+          _MAKE_SPIN_BUTTON(uint64,UINT64,UInt64)
+          _MAKE_SPIN_BUTTON(long,LONG,Long)
+          _MAKE_SPIN_BUTTON(ulong,ULONG,ULong)
           case G_TYPE_DOUBLE: {
-            GParamSpecDouble *double_property=G_PARAM_SPEC_DOUBLE(property);
+            GParamSpecDouble *p=G_PARAM_SPEC_DOUBLE(property);
             gdouble step,value;
             gchar *str_value;
 
             g_object_get(machine,property->name,&value,NULL);
             // get max(max,-min), count digits -> to determine needed length of field
-            str_value=g_strdup_printf("%7.2f",value);
-            step=(double_property->maximum-double_property->minimum)/1024.0;
-            widget1=gtk_hscale_new_with_range(double_property->minimum,double_property->maximum,step);
+            str_value=g_strdup_printf("%7.2lf",value);
+            step=(p->maximum-p->minimum)/1024.0;
+            widget1=gtk_hscale_new_with_range(p->minimum,p->maximum,step);
             gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
             g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
             gtk_scale_set_draw_value(GTK_SCALE(widget1),FALSE);
@@ -440,10 +382,10 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
             g_object_set(widget2,"max-length",9,"width-chars",9,NULL);
             g_free(str_value);
             signal_name=g_strdup_printf("notify::%s",property->name);
-            g_signal_connect(G_OBJECT(machine), signal_name, G_CALLBACK(on_range_property_notify), (gpointer)widget1);
-            g_signal_connect(G_OBJECT(machine), signal_name, G_CALLBACK(on_double_entry_property_notify), (gpointer)widget2);
-            g_signal_connect(G_OBJECT(widget1), "value-changed", G_CALLBACK(on_range_property_changed), (gpointer)machine);
-            g_signal_connect(G_OBJECT(widget2), "changed", G_CALLBACK(on_double_entry_property_changed), (gpointer)machine);
+            g_signal_connect(machine, signal_name, G_CALLBACK(on_range_property_notify), (gpointer)widget1);
+            g_signal_connect(machine, signal_name, G_CALLBACK(on_double_entry_property_notify), (gpointer)widget2);
+            g_signal_connect(widget1, "value-changed", G_CALLBACK(on_range_property_changed), (gpointer)machine);
+            g_signal_connect(widget2, "changed", G_CALLBACK(on_double_entry_property_changed), (gpointer)machine);
             g_free(signal_name);
           } break;
           case G_TYPE_ENUM: {
@@ -462,7 +404,7 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
             for(value=enum_class->minimum;value<=enum_class->maximum;value++) {
               if((enum_value=g_enum_get_value(enum_class, value))) {
                 //GST_INFO("enum value: %d, '%s', '%s'",enum_value->value,enum_value->value_name,enum_value->value_nick);
-                //gtk_combo_box_append_text(GTK_COMBO_BOX(widget1),enum_value->value_nick);
+                //gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widget1),enum_value->value_nick);
                 gtk_list_store_append(store,&iter);
                 gtk_list_store_set(store,&iter,
                   0,enum_value->value,
@@ -483,14 +425,14 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
               gtk_tree_model_get((GTK_TREE_MODEL(store)),&iter,0,&ivalue,-1);
               if(ivalue==value) break;
             } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(store),&iter));
-            
+
             gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget1),&iter);
             gtk_widget_set_name(GTK_WIDGET(widget1),property->name);
             g_object_set_qdata(G_OBJECT(widget1),widget_parent_quark,(gpointer)self);
-            
+
             signal_name=g_strdup_printf("notify::%s",property->name);
-            g_signal_connect(G_OBJECT(machine), signal_name, G_CALLBACK(on_combobox_property_notify), (gpointer)widget1);
-            g_signal_connect(G_OBJECT(widget1), "changed", G_CALLBACK(on_combobox_property_changed), (gpointer)machine);
+            g_signal_connect(machine, signal_name, G_CALLBACK(on_combobox_property_notify), (gpointer)widget1);
+            g_signal_connect(widget1, "changed", G_CALLBACK(on_combobox_property_changed), (gpointer)machine);
             g_free(signal_name);
             widget2=NULL;
           } break;
@@ -500,12 +442,12 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
             widget2=NULL;
           }
         }
-        gtk_widget_set_tooltip_text(widget1,g_param_spec_get_blurb(property));
+        gtk_widget_set_tooltip_text(widget1,tool_tip_text);
         if(!widget2) {
           gtk_table_attach(GTK_TABLE(table),widget1, 1, 3, k, k+1, GTK_FILL|GTK_EXPAND,GTK_SHRINK, 2,1);
         }
         else {
-          gtk_widget_set_tooltip_text(widget2,g_param_spec_get_blurb(property));
+          gtk_widget_set_tooltip_text(widget2,tool_tip_text);
           gtk_table_attach(GTK_TABLE(table),widget1, 1, 2, k, k+1, GTK_FILL|GTK_EXPAND,GTK_SHRINK, 2,1);
           gtk_table_attach(GTK_TABLE(table),widget2, 2, 3, k, k+1, GTK_FILL,GTK_SHRINK, 2,1);
         }
@@ -527,37 +469,30 @@ static gboolean bt_machine_preferences_dialog_init_ui(const BtMachinePreferences
     gtk_container_add(GTK_CONTAINER(self),gtk_label_new(_("machine has no preferences")));
   }
 
+  // track machine name (keep window title up-to-date)
+  g_signal_connect(self->priv->machine,"notify::id",G_CALLBACK(on_machine_id_changed),(gpointer)self);
+
   g_object_unref(machine);
   g_object_unref(main_window);
-  return(TRUE);
 }
 
 //-- constructor methods
 
 /**
  * bt_machine_preferences_dialog_new:
- * @app: the application the dialog belongs to
  * @machine: the machine to create the dialog for
  *
  * Create a new instance
  *
- * Returns: the new instance or NULL in case of an error
+ * Returns: the new instance
  */
-BtMachinePreferencesDialog *bt_machine_preferences_dialog_new(const BtEditApplication *app,const BtMachine *machine) {
+BtMachinePreferencesDialog *bt_machine_preferences_dialog_new(const BtMachine *machine) {
   BtMachinePreferencesDialog *self;
 
-  if(!(self=BT_MACHINE_PREFERENCES_DIALOG(g_object_new(BT_TYPE_MACHINE_PREFERENCES_DIALOG,"app",app,"machine",machine,NULL)))) {
-    goto Error;
-  }
-  // generate UI
-  if(!bt_machine_preferences_dialog_init_ui(self)) {
-    goto Error;
-  }
+  self=BT_MACHINE_PREFERENCES_DIALOG(g_object_new(BT_TYPE_MACHINE_PREFERENCES_DIALOG,"machine",machine,NULL));
+  bt_machine_preferences_dialog_init_ui(self);
   gtk_widget_show_all(GTK_WIDGET(self));
   return(self);
-Error:
-  gtk_widget_destroy(GTK_WIDGET(self));
-  return(NULL);
 }
 
 //-- methods
@@ -566,44 +501,13 @@ Error:
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_machine_preferences_dialog_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
+static void bt_machine_preferences_dialog_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
   BtMachinePreferencesDialog *self = BT_MACHINE_PREFERENCES_DIALOG(object);
   return_if_disposed();
   switch (property_id) {
-    case MACHINE_PREFERENCES_DIALOG_APP: {
-      g_value_set_object(value, self->priv->app);
-    } break;
-    case MACHINE_PREFERENCES_DIALOG_MACHINE: {
-      g_value_set_object(value, self->priv->machine);
-    } break;
-    default: {
-       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
-/* sets the given preferences for this object */
-static void bt_machine_preferences_dialog_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
-  BtMachinePreferencesDialog *self = BT_MACHINE_PREFERENCES_DIALOG(object);
-  return_if_disposed();
-  switch (property_id) {
-    case MACHINE_PREFERENCES_DIALOG_APP: {
-      g_object_try_unref(self->priv->app);
-      self->priv->app = g_object_try_ref(g_value_get_object(value));
-      //GST_DEBUG("set the app for settings_dialog: %p",self->priv->app);
-    } break;
     case MACHINE_PREFERENCES_DIALOG_MACHINE: {
       g_object_try_unref(self->priv->machine);
-      self->priv->machine = g_object_try_ref(g_value_get_object(value));
+      self->priv->machine = g_value_dup_object(value);
     } break;
     default: {
       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
@@ -620,80 +524,42 @@ static void bt_machine_preferences_dialog_dispose(GObject *object) {
 
   GST_DEBUG("!!!! self=%p",self);
 
+  // disconnect handlers
+  g_signal_handlers_disconnect_matched(self->priv->machine,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_machine_id_changed,self);
   // disconnect handlers connected to machine properties
   g_object_get(self->priv->machine,"machine",&machine,NULL);
   g_signal_handlers_disconnect_matched(machine,G_SIGNAL_MATCH_FUNC,0,0,NULL,on_range_property_notify,NULL);
   g_signal_handlers_disconnect_matched(machine,G_SIGNAL_MATCH_FUNC,0,0,NULL,on_double_entry_property_notify,NULL);
   g_object_unref(machine);
 
-  g_object_try_unref(self->priv->app);
   g_object_try_unref(self->priv->machine);
+  g_object_unref(self->priv->app);
 
-  if(G_OBJECT_CLASS(parent_class)->dispose) {
-    (G_OBJECT_CLASS(parent_class)->dispose)(object);
-  }
+  G_OBJECT_CLASS(bt_machine_preferences_dialog_parent_class)->dispose(object);
 }
 
-static void bt_machine_preferences_dialog_finalize(GObject *object) {
-  //BtMachinePreferencesDialog *self = BT_MACHINE_PREFERENCES_DIALOG(object);
-
-  //GST_DEBUG("!!!! self=%p",self);
-
-  if(G_OBJECT_CLASS(parent_class)->finalize) {
-    (G_OBJECT_CLASS(parent_class)->finalize)(object);
-  }
-}
-
-static void bt_machine_preferences_dialog_init(GTypeInstance *instance, gpointer g_class) {
-  BtMachinePreferencesDialog *self = BT_MACHINE_PREFERENCES_DIALOG(instance);
-
+static void bt_machine_preferences_dialog_init(BtMachinePreferencesDialog *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_MACHINE_PREFERENCES_DIALOG, BtMachinePreferencesDialogPrivate);
+  GST_DEBUG("!!!! self=%p",self);
+  self->priv->app = bt_edit_application_new();
 }
 
 static void bt_machine_preferences_dialog_class_init(BtMachinePreferencesDialogClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
-  
+
   widget_parent_quark=g_quark_from_static_string("BtMachinePreferencesDialog::widget-parent");
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtMachinePreferencesDialogPrivate));
 
   gobject_class->set_property = bt_machine_preferences_dialog_set_property;
-  gobject_class->get_property = bt_machine_preferences_dialog_get_property;
   gobject_class->dispose      = bt_machine_preferences_dialog_dispose;
-  gobject_class->finalize     = bt_machine_preferences_dialog_finalize;
-
-  g_object_class_install_property(gobject_class,MACHINE_PREFERENCES_DIALOG_APP,
-                                  g_param_spec_object("app",
-                                     "app construct prop",
-                                     "Set application object, the dialog belongs to",
-                                     BT_TYPE_EDIT_APPLICATION, /* object type */
-                                     G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property(gobject_class,MACHINE_PREFERENCES_DIALOG_MACHINE,
                                   g_param_spec_object("machine",
                                      "machine construct prop",
                                      "Set machine object, the dialog handles",
                                      BT_TYPE_MACHINE, /* object type */
-                                     G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
+                                     G_PARAM_CONSTRUCT_ONLY|G_PARAM_WRITABLE|G_PARAM_STATIC_STRINGS));
 
 }
 
-GType bt_machine_preferences_dialog_get_type(void) {
-  static GType type = 0;
-  if (type == 0) {
-    const GTypeInfo info = {
-      sizeof (BtMachinePreferencesDialogClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_machine_preferences_dialog_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof (BtMachinePreferencesDialog),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_machine_preferences_dialog_init, // instance_init
-    };
-    type = g_type_register_static(GTK_TYPE_WINDOW,"BtMachinePreferencesDialog",&info,0);
-  }
-  return type;
-}
