@@ -1,4 +1,4 @@
-/* $Id: main-page-info.c 2418 2009-05-06 09:16:59Z ensonic $
+/* $Id: main-page-info.c 3738 2011-12-04 14:27:11Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -26,29 +26,40 @@
  * Provides an editor for the song meta data.
  */
 
-/* @todo
+/* @todo:
  * - use this tab as the first one?
- * - add time-stamps
- *   song created, song last changed
  * - add choice for metre (in german -> takt): beats (beats = bars / tpb)
+ *   or at least show the resulting metre
  */
-
+/* @todo: undo/redo
+ * - for text_view we would need to connect to insert_at_corsor and delete_from_cursor
+ *   signals to track changes
+ * - for text_enry the signals are inserted_text and deleted_text
+ * - optionally we just store the complete before/after
+ */
+/* @todo: changing tpb, beats
+ * - when changing tpb and beats the playback-speed of the current song is
+ *   affected, we might want to offer a tool to resample the patterns.
+ * - it would probably be easier to have some buttons for 
+ *   1/4x 1/3x 1/2x 2x, 3x, 4x tick resolution
+ * - when lowering the tick resolution, we need to handle loss of notes
+ *   when doing 1/4 we would merge 4 rows into one keeping the first note found
+ */
+ 
 #define BT_EDIT
 #define BT_MAIN_PAGE_INFO_C
 
 #include "bt-edit.h"
 
-enum {
-  MAIN_PAGE_INFO_APP=1,
-};
-
-
 struct _BtMainPageInfoPrivate {
   /* used to validate if dispose has run */
   gboolean dispose_has_run;
 
   /* the application */
-  G_POINTER_ALIAS(BtEditApplication *,app);
+  BtEditApplication *app;
+
+  /* the sequence we are showing */
+  BtSongInfo *song_info;
 
   /* name, genre, author of the song */
   GtkEntry *name,*genre,*author;
@@ -63,27 +74,24 @@ struct _BtMainPageInfoPrivate {
   GtkTextView *info;
 };
 
-static GtkVBoxClass *parent_class=NULL;
+//-- the class
+
+G_DEFINE_TYPE (BtMainPageInfo, bt_main_page_info, GTK_TYPE_VBOX);
+
 
 //-- event handler
 
-static gboolean on_page_switched_idle(gpointer user_data) {
-  BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-
-  gtk_widget_grab_focus_savely(GTK_WIDGET(self->priv->info));
-  return(FALSE);
-}
-
-static void on_page_switched(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointer user_data) {
+static void on_page_switched(GtkNotebook *notebook, GParamSpec *arg, gpointer user_data) {
   //BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
+  guint page_num;
   static gint prev_page_num=-1;
+  
+  g_object_get(notebook,"page",&page_num,NULL);
 
   if(page_num==BT_MAIN_PAGES_INFO_PAGE) {
     // only do this if the page really has changed
     if(prev_page_num != BT_MAIN_PAGES_INFO_PAGE) {
       GST_DEBUG("enter info page");
-      // delay the sequence_table grab
-      g_idle_add_full(G_PRIORITY_HIGH_IDLE,on_page_switched_idle,user_data,NULL);
     }
   }
   else {
@@ -97,147 +105,107 @@ static void on_page_switched(GtkNotebook *notebook, GtkNotebookPage *page, guint
 
 static void on_name_changed(GtkEditable *editable,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
-
-  g_assert(user_data);
 
   GST_INFO("name changed : self=%p -> \"%s\"",self,gtk_entry_get_text(GTK_ENTRY(editable)));
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
   // update info fields
-  g_object_set(G_OBJECT(song_info),"name",g_strdup(gtk_entry_get_text(GTK_ENTRY(editable))),NULL);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  g_object_set(self->priv->song_info,"name",g_strdup(gtk_entry_get_text(GTK_ENTRY(editable))),NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_genre_changed(GtkEditable *editable,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
-
-  g_assert(user_data);
 
   GST_INFO("genre changed : self=%p -> \"%s\"",self,gtk_entry_get_text(GTK_ENTRY(editable)));
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
   // update info fields
-  g_object_set(G_OBJECT(song_info),"genre",g_strdup(gtk_entry_get_text(GTK_ENTRY(editable))),NULL);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  g_object_set(self->priv->song_info,"genre",g_strdup(gtk_entry_get_text(GTK_ENTRY(editable))),NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_author_changed(GtkEditable *editable,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
-
-  g_assert(user_data);
 
   GST_INFO("author changed : self=%p -> \"%s\"",self,gtk_entry_get_text(GTK_ENTRY(editable)));
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
   // update info fields
-  g_object_set(G_OBJECT(song_info),"author",g_strdup(gtk_entry_get_text(GTK_ENTRY(editable))),NULL);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  g_object_set(self->priv->song_info,"author",g_strdup(gtk_entry_get_text(GTK_ENTRY(editable))),NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_bpm_changed(GtkSpinButton *spinbutton,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
-
-  g_assert(user_data);
 
   GST_INFO("bpm changed : self=%p -> %d",self,gtk_spin_button_get_value_as_int(spinbutton));
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
   // update info fields
-  g_object_set(G_OBJECT(song_info),"bpm",gtk_spin_button_get_value_as_int(spinbutton),NULL);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  g_object_set(self->priv->song_info,"bpm",gtk_spin_button_get_value_as_int(spinbutton),NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_tpb_changed(GtkSpinButton *spinbutton,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
   gulong tpb,beats,bars;
-
-  g_assert(user_data);
+  GParamSpecULong *pspec;
+  GObjectClass *song_info_class;
 
   GST_INFO("tpb changed : self=%p -> %d",self,gtk_spin_button_get_value_as_int(spinbutton));
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
+
+  song_info_class=g_type_class_ref(BT_TYPE_SONG_INFO);
+  pspec=(GParamSpecULong *)g_object_class_find_property(song_info_class,"bars");
+
   // update info fields
-  g_object_get(G_OBJECT(song_info),"bars",&bars,"tpb",&tpb,NULL);
-  beats = bars/tpb;
-  tpb = gtk_spin_button_get_value_as_int(spinbutton);
-  g_object_set(G_OBJECT(song_info),"tpb",tpb,"bars",beats*tpb,NULL);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  g_object_get(self->priv->song_info,"bars",&bars,"tpb",&tpb,NULL);
+  beats=bars/tpb;
+  tpb=gtk_spin_button_get_value_as_int(spinbutton);
+  bars=beats*tpb;
+  bars=CLAMP(bars,pspec->minimum,pspec->maximum);
+  tpb=bars/beats;
+  gtk_spin_button_set_value(spinbutton,tpb);
+
+  g_object_set(self->priv->song_info,"tpb",tpb,"bars",bars,NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_beats_changed(GtkSpinButton *spinbutton,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
-  gulong tpb,beats;
+  gulong tpb,beats,bars;
+  GParamSpecULong *pspec;
+  GObjectClass *song_info_class;
 
-  g_assert(user_data);
+  song_info_class=g_type_class_ref(BT_TYPE_SONG_INFO);
+  pspec=(GParamSpecULong *)g_object_class_find_property(song_info_class,"bars");
 
   GST_INFO("beats changed : self=%p -> %d",self,gtk_spin_button_get_value_as_int(spinbutton));
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
   // update info fields
-  g_object_get(G_OBJECT(song_info),"tpb",&tpb,NULL);
-  beats = gtk_spin_button_get_value_as_int(spinbutton);
-  g_object_set(G_OBJECT(song_info),"bars",beats*tpb,NULL);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  g_object_get(self->priv->song_info,"tpb",&tpb,NULL);
+  beats=gtk_spin_button_get_value_as_int(spinbutton);
+  bars=beats*tpb;
+  bars=CLAMP(bars,pspec->minimum,pspec->maximum);
+  beats=bars/tpb;
+  gtk_spin_button_set_value(spinbutton,beats);
+  
+  g_object_set(self->priv->song_info,"bars",bars,NULL);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_info_changed(GtkTextBuffer *textbuffer,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
-  BtSong *song;
-  BtSongInfo *song_info;
   gchar *str;
   GtkTextIter beg_iter,end_iter;
 
   GST_INFO("info changed : self=%p",self);
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
   // update info fields
   // get begin and end textiters :(, then get text
   gtk_text_buffer_get_iter_at_offset(textbuffer,&beg_iter,0);
   gtk_text_buffer_get_iter_at_offset(textbuffer,&end_iter,-1);
   str=gtk_text_buffer_get_text(textbuffer,&beg_iter,&end_iter,FALSE);
-  g_object_set(G_OBJECT(song_info),"info",str,NULL);
+  g_object_set(self->priv->song_info,"info",str,NULL);
   g_free(str);
-  // release the references
-  g_object_unref(song_info);
-  g_object_unref(song);
+  bt_edit_application_set_song_unsaved(self->priv->app);
 }
 
 static void on_name_notify(const BtSongInfo *song_info,GParamSpec *arg,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
   gchar *name;
 
-  g_object_get(G_OBJECT(song_info),"name",&name,NULL);
+  g_object_get((gpointer)song_info,"name",&name,NULL);
   g_signal_handlers_block_matched(self->priv->name,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_name_changed,(gpointer)self);
   gtk_entry_set_text(self->priv->name,safe_string(name));g_free(name);
   g_signal_handlers_unblock_matched(self->priv->name,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_name_changed,(gpointer)self);  
@@ -246,23 +214,25 @@ static void on_name_notify(const BtSongInfo *song_info,GParamSpec *arg,gpointer
 static void on_song_changed(const BtEditApplication *app,GParamSpec *arg,gpointer user_data) {
   BtMainPageInfo *self=BT_MAIN_PAGE_INFO(user_data);
   BtSong *song;
-  BtSongInfo *song_info;
   GtkTextBuffer *buffer;
   gchar *name,*genre,*author,*create_dts,*change_dts;
   gulong bpm,tpb,bars;
   gchar *info;
 
-  g_assert(user_data);
-
   GST_INFO("song has changed : app=%p, self=%p",app,self);
-  // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  if(!song) return;
-  GST_INFO("song->ref_ct=%d",G_OBJECT(song)->ref_count);
+  g_object_try_unref(self->priv->song_info);
 
-  g_object_get(G_OBJECT(song),"song-info",&song_info,NULL);
+  // get song from app
+  g_object_get(self->priv->app,"song",&song,NULL);
+  if(!song) {
+    self->priv->song_info=NULL;
+    return;
+  }
+  GST_INFO("song->ref_ct=%d",G_OBJECT_REF_COUNT(song));
+
+  g_object_get(song,"song-info",&self->priv->song_info,NULL);
   // update info fields
-  g_object_get(G_OBJECT(song_info),
+  g_object_get(self->priv->song_info,
     "name",&name,"genre",&genre,"author",&author,"info",&info,
     "bpm",&bpm,"tpb",&tpb,"bars",&bars,
     "create-dts",&create_dts,"change-dts",&change_dts,
@@ -306,30 +276,34 @@ static void on_song_changed(const BtEditApplication *app,GParamSpec *arg,gpointe
   gtk_entry_set_text(self->priv->date_changed,change_dts);g_free(change_dts);
   
   buffer=gtk_text_view_get_buffer(self->priv->info);
-  g_signal_handlers_block_matched(G_OBJECT(buffer),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_info_changed,(gpointer)self);
+  g_signal_handlers_block_matched(buffer,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_info_changed,(gpointer)self);
   gtk_text_buffer_set_text(buffer,safe_string(info),-1);g_free(info);
-  g_signal_handlers_unblock_matched(G_OBJECT(buffer),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_info_changed,(gpointer)self);
+  g_signal_handlers_unblock_matched(buffer,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_info_changed,(gpointer)self);
 
-  g_signal_connect(G_OBJECT(song_info), "notify::name", G_CALLBACK(on_name_notify), (gpointer)self);
+  g_signal_connect(self->priv->song_info, "notify::name", G_CALLBACK(on_name_notify), (gpointer)self);
   
   // release the references
-  g_object_unref(song_info);
   g_object_unref(song);
   GST_INFO("song has changed done");
 }
 
 //-- helper methods
 
-static gboolean bt_main_page_info_init_ui(const BtMainPageInfo *self,const BtMainPages *pages) {
+static void bt_main_page_info_init_ui(const BtMainPageInfo *self,const BtMainPages *pages) {
   GtkWidget *label,*frame,*box;
   GtkWidget *table;
   GtkWidget *scrolledwindow;
   GtkAdjustment *spin_adjustment;
+  GParamSpecULong *pspec,*pspec2;
+  GObjectClass *song_info_class;
   gchar *str;
+  gulong def,min,max;
 
   GST_DEBUG("!!!! self=%p",self);
 
   gtk_widget_set_name(GTK_WIDGET(self),"song information");
+  
+  song_info_class=g_type_class_ref(BT_TYPE_SONG_INFO);
 
   // first row of vbox
   frame=gtk_frame_new(NULL);
@@ -362,28 +336,28 @@ static gboolean bt_main_page_info_init_ui(const BtMainPageInfo *self,const BtMai
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 0, 1, GTK_FILL,GTK_SHRINK, 2,1);
   self->priv->name=GTK_ENTRY(gtk_entry_new());
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->name), 1, 2, 0, 1, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
-  g_signal_connect(G_OBJECT(self->priv->name), "changed", G_CALLBACK(on_name_changed), (gpointer)self);
+  g_signal_connect(self->priv->name, "changed", G_CALLBACK(on_name_changed), (gpointer)self);
 
   label=gtk_label_new(_("genre"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 1, 2, GTK_FILL,GTK_SHRINK, 2,1);
   self->priv->genre=GTK_ENTRY(gtk_entry_new());
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->genre), 1, 2, 1, 2, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
-  g_signal_connect(G_OBJECT(self->priv->genre), "changed", G_CALLBACK(on_genre_changed), (gpointer)self);
+  g_signal_connect(self->priv->genre, "changed", G_CALLBACK(on_genre_changed), (gpointer)self);
 
   label=gtk_label_new(_("author"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 2, 3, GTK_FILL,GTK_SHRINK, 2,1);
   self->priv->author=GTK_ENTRY(gtk_entry_new());
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->author), 1, 2, 2, 3, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
-  g_signal_connect(G_OBJECT(self->priv->author), "changed", G_CALLBACK(on_author_changed), (gpointer)self);
+  g_signal_connect(self->priv->author, "changed", G_CALLBACK(on_author_changed), (gpointer)self);
   
   label=gtk_label_new(_("created"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 3, 4, GTK_FILL,GTK_SHRINK, 2,1);
   self->priv->date_created=GTK_ENTRY(gtk_entry_new());
   gtk_editable_set_editable(GTK_EDITABLE(self->priv->date_created),FALSE);
-  GTK_WIDGET_UNSET_FLAGS(self->priv->date_created,GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus(GTK_WIDGET(self->priv->date_created),FALSE);
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->date_created), 1, 2, 3, 4, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
 
   // second column
@@ -393,33 +367,40 @@ static gboolean bt_main_page_info_init_ui(const BtMainPageInfo *self,const BtMai
   label=gtk_label_new(_("beats per minute"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 0, 1, GTK_FILL,GTK_SHRINK, 2,1);
-  spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(130.0, 20.0, 300.0, 1.0, 5.0, 0.0));
+  pspec=(GParamSpecULong *)g_object_class_find_property(song_info_class,"bpm");
+  spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(pspec->default_value, pspec->minimum, pspec->maximum, 1.0, 5.0, 0.0));
   self->priv->bpm=GTK_SPIN_BUTTON(gtk_spin_button_new(spin_adjustment,1.0,0));
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->bpm), 1, 2, 0, 1, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
-  g_signal_connect(G_OBJECT(self->priv->bpm), "value-changed", G_CALLBACK(on_bpm_changed), (gpointer)self);
+  g_signal_connect(self->priv->bpm, "value-changed", G_CALLBACK(on_bpm_changed), (gpointer)self);
 
   label=gtk_label_new(_("beats"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 1, 2, GTK_FILL,GTK_SHRINK, 2,1);
-  spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(8.0, 1.0, 32.0, 1.0, 4.0, 0.0));
+  pspec=(GParamSpecULong *)g_object_class_find_property(song_info_class,"tpb");
+  pspec2=(GParamSpecULong *)g_object_class_find_property(song_info_class,"bars");
+  def=pspec2->default_value/pspec->default_value;
+  min=pspec2->minimum/pspec->maximum;
+  max=pspec2->maximum/pspec->minimum;
+  if(min<1) min=1;
+  spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(def,min,max, 1.0, 4.0, 0.0));
   self->priv->beats=GTK_SPIN_BUTTON(gtk_spin_button_new(spin_adjustment,1.0,0));
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->beats), 1, 2, 1, 2, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
-  g_signal_connect(G_OBJECT(self->priv->beats), "value-changed", G_CALLBACK(on_beats_changed), (gpointer)self);
+  g_signal_connect(self->priv->beats, "value-changed", G_CALLBACK(on_beats_changed), (gpointer)self);
 
   label=gtk_label_new(_("ticks per beat"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 2, 3, GTK_FILL,GTK_SHRINK, 2,1);
-  spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(8.0, 1.0, 64.0, 1.0, 4.0, 0.0));
+  spin_adjustment=GTK_ADJUSTMENT(gtk_adjustment_new(pspec->default_value, pspec->minimum, pspec->maximum, 1.0, 4.0, 0.0));
   self->priv->tpb=GTK_SPIN_BUTTON(gtk_spin_button_new(spin_adjustment,1.0,0));
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->tpb), 1, 2, 2, 3, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
-  g_signal_connect(G_OBJECT(self->priv->tpb), "value-changed", G_CALLBACK(on_tpb_changed), (gpointer)self);
+  g_signal_connect(self->priv->tpb, "value-changed", G_CALLBACK(on_tpb_changed), (gpointer)self);
 
   label=gtk_label_new(_("last saved"));
   gtk_misc_set_alignment(GTK_MISC(label),1.0,0.5);
   gtk_table_attach(GTK_TABLE(table),label, 0, 1, 3, 4, GTK_FILL,GTK_SHRINK, 2,1);
   self->priv->date_changed=GTK_ENTRY(gtk_entry_new());
   gtk_editable_set_editable(GTK_EDITABLE(self->priv->date_changed),FALSE);
-  GTK_WIDGET_UNSET_FLAGS(self->priv->date_changed,GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus(GTK_WIDGET(self->priv->date_changed),FALSE);
   gtk_table_attach(GTK_TABLE(table),GTK_WIDGET(self->priv->date_changed), 1, 2, 3, 4, GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 2,1);
 
   // @idea have another field with subticks (GstController parameter smoothing)
@@ -469,44 +450,32 @@ static gboolean bt_main_page_info_init_ui(const BtMainPageInfo *self,const BtMai
   GST_DEBUG("  text view: %p",self->priv->info);
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(self->priv->info),GTK_WRAP_WORD);
   gtk_container_add(GTK_CONTAINER(scrolledwindow),GTK_WIDGET(self->priv->info));
-  g_signal_connect(G_OBJECT(gtk_text_view_get_buffer(self->priv->info)), "changed", G_CALLBACK(on_info_changed), (gpointer)self);
+  g_signal_connect(gtk_text_view_get_buffer(self->priv->info), "changed", G_CALLBACK(on_info_changed), (gpointer)self);
 
-  // set default widget
-  gtk_container_set_focus_child(GTK_CONTAINER(self),GTK_WIDGET(self->priv->info));
   // register event handlers
-  g_signal_connect(G_OBJECT(self->priv->app), "notify::song", G_CALLBACK(on_song_changed), (gpointer)self);
+  g_signal_connect(self->priv->app, "notify::song", G_CALLBACK(on_song_changed), (gpointer)self);
   // listen to page changes
-  g_signal_connect(G_OBJECT(pages), "switch-page", G_CALLBACK(on_page_switched), (gpointer)self);
+  g_signal_connect((gpointer)pages, "notify::page", G_CALLBACK(on_page_switched), (gpointer)self);
 
   GST_DEBUG("  done");
-  return(TRUE);
 }
 
 //-- constructor methods
 
 /**
  * bt_main_page_info_new:
- * @app: the application the window belongs to
  * @pages: the page collection
  *
  * Create a new instance
  *
- * Returns: the new instance or %NULL in case of an error
+ * Returns: the new instance
  */
-BtMainPageInfo *bt_main_page_info_new(const BtEditApplication *app,const BtMainPages *pages) {
+BtMainPageInfo *bt_main_page_info_new(const BtMainPages *pages) {
   BtMainPageInfo *self;
 
-  if(!(self=BT_MAIN_PAGE_INFO(g_object_new(BT_TYPE_MAIN_PAGE_INFO,"app",app,"spacing",6,NULL)))) {
-    goto Error;
-  }
-  // generate UI
-  if(!bt_main_page_info_init_ui(self,pages)) {
-    goto Error;
-  }
+  self=BT_MAIN_PAGE_INFO(g_object_new(BT_TYPE_MAIN_PAGE_INFO,"spacing",6,NULL));
+  bt_main_page_info_init_ui(self,pages);
   return(self);
-Error:
-  if(self) gtk_object_destroy(GTK_OBJECT(self));
-  return(NULL);
 }
 
 //-- methods
@@ -515,43 +484,12 @@ Error:
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_main_page_info_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
-  BtMainPageInfo *self = BT_MAIN_PAGE_INFO(object);
-  return_if_disposed();
-  switch (property_id) {
-    case MAIN_PAGE_INFO_APP: {
-      g_value_set_object(value, self->priv->app);
-    } break;
-    default: {
-       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
-/* sets the given properties for this object */
-static void bt_main_page_info_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
-  BtMainPageInfo *self = BT_MAIN_PAGE_INFO(object);
-  return_if_disposed();
-  switch (property_id) {
-    case MAIN_PAGE_INFO_APP: {
-      g_object_try_weak_unref(self->priv->app);
-      self->priv->app = BT_EDIT_APPLICATION(g_value_get_object(value));
-      g_object_try_weak_ref(self->priv->app);
-      //GST_DEBUG("set the app for main_page_info: %p",self->priv->app);
-    } break;
-    default: {
-      G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
+static gboolean bt_main_page_info_focus(GtkWidget *widget, GtkDirectionType direction) {
+  BtMainPageInfo *self=BT_MAIN_PAGE_INFO(widget);
+  
+  GST_DEBUG("focusing default widget");
+  gtk_widget_grab_focus_savely(GTK_WIDGET(self->priv->info));
+  return FALSE;
 }
 
 static void bt_main_page_info_dispose(GObject *object) {
@@ -561,62 +499,30 @@ static void bt_main_page_info_dispose(GObject *object) {
   
   GST_DEBUG("!!!! self=%p",self);
 
-  // @bug: http://bugzilla.gnome.org/show_bug.cgi?id=414712
-  gtk_container_set_focus_child(GTK_CONTAINER(self),NULL);
+  g_signal_handlers_disconnect_matched(self->priv->app,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_song_changed,(gpointer)self);
 
-  g_object_try_weak_unref(self->priv->app);
+  g_object_try_unref(self->priv->song_info);
+
+  g_object_unref(self->priv->app);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_main_page_info_parent_class)->dispose(object);
 }
 
-static void bt_main_page_info_finalize(GObject *object) {
-  //BtMainPageInfo *self = BT_MAIN_PAGE_INFO(object);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
-}
-
-static void bt_main_page_info_init(GTypeInstance *instance, gpointer g_class) {
-  BtMainPageInfo *self = BT_MAIN_PAGE_INFO(instance);
-
+static void bt_main_page_info_init(BtMainPageInfo *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_MAIN_PAGE_INFO, BtMainPageInfoPrivate);
+  GST_DEBUG("!!!! self=%p",self);
+  self->priv->app = bt_edit_application_new();
 }
 
 static void bt_main_page_info_class_init(BtMainPageInfoClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
+  GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtMainPageInfoPrivate));
 
-  gobject_class->set_property = bt_main_page_info_set_property;
-  gobject_class->get_property = bt_main_page_info_get_property;
   gobject_class->dispose      = bt_main_page_info_dispose;
-  gobject_class->finalize     = bt_main_page_info_finalize;
-
-  g_object_class_install_property(gobject_class,MAIN_PAGE_INFO_APP,
-                                  g_param_spec_object("app",
-                                     "app contruct prop",
-                                     "Set application object, the window belongs to",
-                                     BT_TYPE_EDIT_APPLICATION, /* object type */
-                                     G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
+  
+  gtkwidget_class->focus      = bt_main_page_info_focus;
 }
 
-GType bt_main_page_info_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtMainPageInfoClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_main_page_info_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtMainPageInfo),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_main_page_info_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(GTK_TYPE_VBOX,"BtMainPageInfo",&info,0);
-  }
-  return type;
-}
