@@ -1,4 +1,4 @@
-/* $Id: main-statusbar.c 2418 2009-05-06 09:16:59Z ensonic $
+/* $Id: main-statusbar.c 3765 2011-12-08 19:39:30Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -25,9 +25,6 @@
  * The statusbar shows some contextual help, as well as things like playback
  * status.
  */
-/* buzz uses 3 time counters
- * file:///windows/C/Programme/Jeskola/Buzz%20(work)/Help/Files/Time%20Window.htm
- */
 
 #define BT_EDIT
 #define BT_MAIN_STATUSBAR_C
@@ -35,17 +32,18 @@
 #include "bt-edit.h"
 
 enum {
-  MAIN_STATUSBAR_APP=1,
-  MAIN_STATUSBAR_STATUS
+  MAIN_STATUSBAR_STATUS=1
 };
 
+//#define USE_MAIN_LOOP_IDLE_TRACKER 1
+
 
 struct _BtMainStatusbarPrivate {
   /* used to validate if dispose has run */
   gboolean dispose_has_run;
 
   /* the application */
-  G_POINTER_ALIAS(BtEditApplication *,app);
+  BtEditApplication *app;
 
   /* main status bar */
   GtkStatusbar *status;
@@ -71,12 +69,22 @@ struct _BtMainStatusbarPrivate {
   GtkProgressBar *cpu_load;
   guint cpu_load_handler_id;
 
+#ifdef USE_MAIN_LOOP_IDLE_TRACKER
+  /* main-loop monitor */
+  guint main_loop_idle_handler_id;
+  GstClockTime ml_tlast,ml_tavg;
+  guint64 ml_ct;
+#endif
+
   /* total playtime */
   GstClockTime total_time;
   gulong last_pos, play_start;
 };
 
-static GtkHBoxClass *parent_class=NULL;
+//-- the class
+
+G_DEFINE_TYPE (BtMainStatusbar, bt_main_statusbar, GTK_TYPE_HBOX);
+
 
 //-- helper
 
@@ -105,10 +113,12 @@ static void on_song_play_pos_notify(const BtSong *song,GParamSpec *arg,gpointer
   gulong pos,msec,sec,min;
   GstClockTime bar_time;
 
-  g_assert(user_data);
   GST_DEBUG("tick update");
 
-  g_object_get(G_OBJECT(song),"sequence",&sequence,"play-pos",&pos,NULL);
+  g_object_get((gpointer)song,"sequence",&sequence,"play-pos",&pos,NULL);
+  if(!sequence)
+    return;
+
   //GST_INFO("sequence tick received : %d",pos);
   bar_time=bt_sequence_get_bar_time(sequence);
 
@@ -146,9 +156,7 @@ static void on_song_is_playing_notify(const BtSong *song,GParamSpec *arg,gpointe
   gboolean is_playing;
   gulong play_start;
 
-  g_assert(user_data);
-
-  g_object_get(G_OBJECT(song),"is-playing",&is_playing,"play-pos",&play_start,NULL);
+  g_object_get((gpointer)song,"is-playing",&is_playing,"play-pos",&play_start,NULL);
   self->priv->total_time=G_GINT64_CONSTANT(0);
   if(!is_playing) {
     GST_INFO("play_start=%lu",play_start);
@@ -169,9 +177,9 @@ static void on_song_info_rhythm_notify(const BtSongInfo *song_info,GParamSpec *a
   BtSequence *sequence;
 
   // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   g_return_if_fail(song);
-  g_object_get(G_OBJECT(song),"sequence",&sequence,NULL);
+  g_object_get(song,"sequence",&sequence,NULL);
 
   bt_main_statusbar_update_length(self,song,sequence);
 
@@ -185,7 +193,7 @@ static void on_sequence_loop_time_notify(const BtSequence *sequence,GParamSpec *
   BtSong *song;
 
   // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   g_return_if_fail(song);
 
   bt_main_statusbar_update_length(self,song,sequence);
@@ -200,26 +208,24 @@ static void on_song_changed(const BtEditApplication *app,GParamSpec *arg,gpointe
   BtSongInfo *song_info;
   BtSequence *sequence;
 
-  g_assert(user_data);
-
   GST_INFO("song has changed : app=%p, self=%p",app,self);
   // get song from app
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   if(!song) return;
 
-  g_object_get(G_OBJECT(song),"sequence",&sequence,"song-info",&song_info,NULL);
+  g_object_get(song,"sequence",&sequence,"song-info",&song_info,NULL);
   bt_main_statusbar_update_length(self,song,sequence);
   // subscribe to property changes in song
-  g_signal_connect(G_OBJECT(song), "notify::play-pos", G_CALLBACK(on_song_play_pos_notify), (gpointer)self);
-  g_signal_connect(G_OBJECT(song), "notify::is-playing", G_CALLBACK(on_song_is_playing_notify), (gpointer)self);
+  g_signal_connect(song, "notify::play-pos", G_CALLBACK(on_song_play_pos_notify), (gpointer)self);
+  g_signal_connect(song, "notify::is-playing", G_CALLBACK(on_song_is_playing_notify), (gpointer)self);
   // subscribe to property changes in song_info
-  g_signal_connect(G_OBJECT(song_info), "notify::bpm", G_CALLBACK(on_song_info_rhythm_notify), (gpointer)self);
-  g_signal_connect(G_OBJECT(song_info), "notify::tpb", G_CALLBACK(on_song_info_rhythm_notify), (gpointer)self);
+  g_signal_connect(song_info, "notify::bpm", G_CALLBACK(on_song_info_rhythm_notify), (gpointer)self);
+  g_signal_connect(song_info, "notify::tpb", G_CALLBACK(on_song_info_rhythm_notify), (gpointer)self);
   // subscribe to property changes in sequence
-  g_signal_connect(G_OBJECT(sequence), "notify::length", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
-  g_signal_connect(G_OBJECT(sequence), "notify::loop", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
-  g_signal_connect(G_OBJECT(sequence), "notify::loop-start", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
-  g_signal_connect(G_OBJECT(sequence), "notify::loop-end", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
+  g_signal_connect(sequence, "notify::length", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
+  g_signal_connect(sequence, "notify::loop", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
+  g_signal_connect(sequence, "notify::loop-start", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
+  g_signal_connect(sequence, "notify::loop-end", G_CALLBACK(on_sequence_loop_time_notify), (gpointer)self);
   // release the references
   g_object_unref(song_info);
   g_object_unref(sequence);
@@ -229,28 +235,48 @@ static void on_song_changed(const BtEditApplication *app,GParamSpec *arg,gpointe
 static gboolean on_cpu_load_update(gpointer user_data) {
   BtMainStatusbar *self=BT_MAIN_STATUSBAR(user_data);
   guint cpu_load=bt_cpu_load_get_current();
+#ifdef USE_MAIN_LOOP_IDLE_TRACKER
+  guint ml_lag=GST_TIME_AS_MSECONDS(self->priv->ml_tavg);
+  gchar str[strlen("CPU: 000 %, ML: 00000 ms") + 3];
+#else
   gchar str[strlen("CPU: 000 %") + 3];
+#endif
 
-  g_assert(user_data);
-
-  // @todo: we nned to handle multi core cpus and divide by num-cores  
-  // http://lxr.linux.no/linux/include/linux/cpumask.h
   if(cpu_load>100.0) cpu_load=100.0;
 
+#ifdef USE_MAIN_LOOP_IDLE_TRACKER
+  g_sprintf(str,"CPU: %d %%, ML: %d ms",cpu_load,ml_lag);
+#else
   g_sprintf(str,"CPU: %d %%",cpu_load);
+#endif
   gtk_progress_bar_set_fraction(self->priv->cpu_load,(gdouble)cpu_load/100.0);
   gtk_progress_bar_set_text(self->priv->cpu_load,str);
   return(TRUE);
 }
 
+#ifdef USE_MAIN_LOOP_IDLE_TRACKER
+static gboolean on_main_loop_idle(gpointer user_data) {
+  BtMainStatusbar *self=BT_MAIN_STATUSBAR(user_data);
+  GstClockTime tnow,tdiff;
+
+  tnow=gst_util_get_timestamp();
+  if(self->priv->ml_ct!=0) {
+    tdiff=self->priv->ml_tlast-tnow;
+    // moving cumulative average
+    self->priv->ml_tavg=self->priv->ml_tavg+((tdiff-self->priv->ml_tavg)/self->priv->ml_ct);
+    self->priv->ml_ct++;
+  }
+  self->priv->ml_tlast=tnow;
+
+  return TRUE;
+}
+#endif
+
 //-- helper methods
 
-static gboolean bt_main_statusbar_init_ui(const BtMainStatusbar *self, const BtEditApplication *app) {
+static void bt_main_statusbar_init_ui(const BtMainStatusbar *self) {
   GtkWidget *ev_box;
   gchar str[]="00:00.000";
-#if !GTK_CHECK_VERSION(2,12,0)
-  GtkTooltips *tips=gtk_tooltips_new();
-#endif
 
   gtk_widget_set_name(GTK_WIDGET(self),"status-bar");
   //#ifdef USE_HILDON /* has no effect */
@@ -260,27 +286,33 @@ static gboolean bt_main_statusbar_init_ui(const BtMainStatusbar *self, const BtE
 
   // context sensitive help statusbar
   self->priv->status=GTK_STATUSBAR(gtk_statusbar_new());
-  self->priv->status_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->status),_("default"));
+  self->priv->status_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->status),"default");
   gtk_statusbar_set_has_resize_grip(self->priv->status,FALSE);
   gtk_statusbar_push(GTK_STATUSBAR(self->priv->status),self->priv->status_context_id,_("Ready to rock!"));
   gtk_box_pack_start(GTK_BOX(self),GTK_WIDGET(self->priv->status),TRUE,TRUE,1);
 
   // cpu load
   // @todo: make this dependend on settings (view menu?)
+  // @todo: this pushes the height of the status-bar
+  //   (e.g. setting this to 10 instead of -1 saves pixels)
   self->priv->cpu_load=GTK_PROGRESS_BAR(gtk_progress_bar_new());
   // @idea: this could be used to avoid resizing when the label changes
   //gtk_progress_bar_set_ellipsize(self->priv->cpu_load,PANGO_ELLIPSIZE_END);
+  //gtk_label_set_single_line_mode(self->priv->cpu_load,TRUE);
   gtk_widget_set_tooltip_text(GTK_WIDGET(self->priv->cpu_load),_("CPU load"));
 
   gtk_box_pack_start(GTK_BOX(self),GTK_WIDGET(self->priv->cpu_load),FALSE,FALSE,1);
   self->priv->cpu_load_handler_id=g_timeout_add(1000, on_cpu_load_update, (gpointer)self);
+#ifdef USE_MAIN_LOOP_IDLE_TRACKER
+  self->priv->main_loop_idle_handler_id=g_idle_add_full(G_PRIORITY_LOW, on_main_loop_idle, (gpointer)self,NULL);
+#endif
 
   // timer status-bars
   ev_box=gtk_event_box_new();
   g_object_set(ev_box,"visible-window",FALSE,NULL);
   gtk_widget_set_tooltip_text(ev_box,_("Playback time"));
   self->priv->elapsed=GTK_STATUSBAR(gtk_statusbar_new());
-  self->priv->elapsed_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->elapsed),_("default"));
+  self->priv->elapsed_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->elapsed),"default");
   gtk_statusbar_set_has_resize_grip(self->priv->elapsed,FALSE);
   gtk_misc_set_alignment(GTK_MISC(GTK_STATUSBAR(self->priv->elapsed)->label),1.0,0.5);
   gtk_widget_set_size_request(GTK_WIDGET(self->priv->elapsed),100,-1);
@@ -292,7 +324,7 @@ static gboolean bt_main_statusbar_init_ui(const BtMainStatusbar *self, const BtE
   g_object_set(ev_box,"visible-window",FALSE,NULL);
   gtk_widget_set_tooltip_text(ev_box,_("Playback position"));
   self->priv->current=GTK_STATUSBAR(gtk_statusbar_new());
-  self->priv->current_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->current),_("default"));
+  self->priv->current_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->current),"default");
   gtk_statusbar_set_has_resize_grip(self->priv->current,FALSE);
   gtk_misc_set_alignment(GTK_MISC(GTK_STATUSBAR(self->priv->current)->label),1.0,0.5);
   gtk_widget_set_size_request(GTK_WIDGET(self->priv->current),100,-1);
@@ -304,7 +336,7 @@ static gboolean bt_main_statusbar_init_ui(const BtMainStatusbar *self, const BtE
   g_object_set(ev_box,"visible-window",FALSE,NULL);
   gtk_widget_set_tooltip_text(ev_box,_("Playback length"));
   self->priv->loop=GTK_STATUSBAR(gtk_statusbar_new());
-  self->priv->loop_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->loop),_("default"));
+  self->priv->loop_context_id=gtk_statusbar_get_context_id(GTK_STATUSBAR(self->priv->loop),"default");
   gtk_widget_set_size_request(GTK_WIDGET(self->priv->loop),100,-1);
   gtk_misc_set_alignment(GTK_MISC(GTK_STATUSBAR(self->priv->loop)->label),1.0,0.5);
   gtk_statusbar_push(GTK_STATUSBAR(self->priv->loop),self->priv->loop_context_id,str);
@@ -312,35 +344,24 @@ static gboolean bt_main_statusbar_init_ui(const BtMainStatusbar *self, const BtE
   gtk_box_pack_start(GTK_BOX(self),ev_box,FALSE,FALSE,1);
 
   // register event handlers
-  g_signal_connect(G_OBJECT(app), "notify::song", G_CALLBACK(on_song_changed), (gpointer)self);
-
-  return(TRUE);
+  g_signal_connect(self->priv->app, "notify::song", G_CALLBACK(on_song_changed), (gpointer)self);
 }
 
 //-- constructor methods
 
 /**
  * bt_main_statusbar_new:
- * @app: the application the statusbar belongs to
  *
  * Create a new instance
  *
- * Returns: the new instance or %NULL in case of an error
+ * Returns: the new instance
  */
-BtMainStatusbar *bt_main_statusbar_new(const BtEditApplication *app) {
+BtMainStatusbar *bt_main_statusbar_new(void) {
   BtMainStatusbar *self;
 
-  if(!(self=BT_MAIN_STATUSBAR(g_object_new(BT_TYPE_MAIN_STATUSBAR,"app",app,NULL)))) {
-    goto Error;
-  }
-  // generate UI
-  if(!bt_main_statusbar_init_ui(self,app)) {
-    goto Error;
-  }
+  self=BT_MAIN_STATUSBAR(g_object_new(BT_TYPE_MAIN_STATUSBAR,NULL));
+  bt_main_statusbar_init_ui(self);
   return(self);
-Error:
-  if(self) gtk_object_destroy(GTK_OBJECT(self));
-  return(NULL);
 }
 
 //-- methods
@@ -348,41 +369,12 @@ Error:
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_main_statusbar_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
+static void bt_main_statusbar_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
   BtMainStatusbar *self = BT_MAIN_STATUSBAR(object);
   return_if_disposed();
   switch (property_id) {
-    case MAIN_STATUSBAR_APP: {
-      g_value_set_object(value, self->priv->app);
-    } break;
-    default: {
-       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
-/* sets the given properties for this object */
-static void bt_main_statusbar_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
-  BtMainStatusbar *self = BT_MAIN_STATUSBAR(object);
-  return_if_disposed();
-  switch (property_id) {
-    case MAIN_STATUSBAR_APP: {
-      g_object_try_weak_unref(self->priv->app);
-      self->priv->app = BT_EDIT_APPLICATION(g_value_get_object(value));
-      g_object_try_weak_ref(self->priv->app);
-      //GST_DEBUG("set the app for main_statusbar: %p",self->priv->app);
-    } break;
     case MAIN_STATUSBAR_STATUS: {
-      char *str=g_value_dup_string(value);
+      gchar *str=g_value_dup_string(value);
       gtk_statusbar_pop(GTK_STATUSBAR(self->priv->status),self->priv->status_context_id);
       if(str) {
         gtk_statusbar_push(GTK_STATUSBAR(self->priv->status),self->priv->status_context_id,str);
@@ -407,13 +399,13 @@ static void bt_main_statusbar_dispose(GObject *object) {
 
   GST_DEBUG("!!!! self=%p",self);
 
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   if(song) {
     BtSongInfo *song_info;
     BtSequence *sequence;
 
     GST_DEBUG("disconnect handlers from song=%p",song);
-    g_object_get(G_OBJECT(song),"sequence",&sequence,"song-info",&song_info,NULL);
+    g_object_get(song,"sequence",&sequence,"song-info",&song_info,NULL);
 
     g_signal_handlers_disconnect_matched(song,G_SIGNAL_MATCH_FUNC,0,0,NULL,on_song_is_playing_notify,NULL);
     g_signal_handlers_disconnect_matched(song,G_SIGNAL_MATCH_FUNC,0,0,NULL,on_song_play_pos_notify,NULL);
@@ -424,43 +416,28 @@ static void bt_main_statusbar_dispose(GObject *object) {
     g_object_unref(song);
   }
   g_source_remove(self->priv->cpu_load_handler_id);
+#ifdef USE_MAIN_LOOP_IDLE_TRACKER
+  g_source_remove(self->priv->main_loop_idle_handler_id);
+#endif
 
-  g_object_try_weak_unref(self->priv->app);
+  g_object_unref(self->priv->app);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_main_statusbar_parent_class)->dispose(object);
 }
 
-static void bt_main_statusbar_finalize(GObject *object) {
-  //BtMainStatusbar *self = BT_MAIN_STATUSBAR(object);
-
-  //GST_DEBUG("!!!! self=%p",self);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
-}
-
-static void bt_main_statusbar_init(GTypeInstance *instance, gpointer g_class) {
-  BtMainStatusbar *self = BT_MAIN_STATUSBAR(instance);
-
+static void bt_main_statusbar_init(BtMainStatusbar *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_MAIN_STATUSBAR, BtMainStatusbarPrivate);
+  GST_DEBUG("!!!! self=%p",self);
+  self->priv->app = bt_edit_application_new();
 }
 
 static void bt_main_statusbar_class_init(BtMainStatusbarClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtMainStatusbarPrivate));
 
   gobject_class->set_property = bt_main_statusbar_set_property;
-  gobject_class->get_property = bt_main_statusbar_get_property;
   gobject_class->dispose      = bt_main_statusbar_dispose;
-  gobject_class->finalize     = bt_main_statusbar_finalize;
-
-  g_object_class_install_property(gobject_class,MAIN_STATUSBAR_APP,
-                                  g_param_spec_object("app",
-                                     "app contruct prop",
-                                     "Set application object, the menu belongs to",
-                                     BT_TYPE_EDIT_APPLICATION, /* object type */
-                                     G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property(gobject_class,MAIN_STATUSBAR_STATUS,
                                   g_param_spec_string("status",
@@ -470,22 +447,3 @@ static void bt_main_statusbar_class_init(BtMainStatusbarClass *klass) {
                                      G_PARAM_WRITABLE|G_PARAM_STATIC_STRINGS));
 }
 
-GType bt_main_statusbar_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtMainStatusbarClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_main_statusbar_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtMainStatusbar),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_main_statusbar_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(GTK_TYPE_HBOX,"BtMainStatusbar",&info,0);
-  }
-  return type;
-}
