@@ -1,4 +1,4 @@
-/* $Id: playback-controller-socket.c 2120 2008-12-04 22:29:01Z ensonic $
+/* $Id: playback-controller-socket.c 3364 2011-05-12 14:55:59Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2007 Buzztard team <buzztard-devel@lists.sf.net>
@@ -33,14 +33,11 @@
 
 #include "bt-edit.h"
 
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 
-enum {
-  SETTINGS_DIALOG_APP=1,
-};
-
 struct _BtPlaybackControllerSocketPrivate {
   /* used to validate if dispose has run */
   gboolean dispose_has_run;
@@ -68,10 +65,13 @@ struct _BtPlaybackControllerSocketPrivate {
   GIOChannel *client_channel;
 };
 
-static GObjectClass *parent_class=NULL;
-
 #define DEFAULT_LABEL "start"
 
+//-- the class
+
+G_DEFINE_TYPE (BtPlaybackControllerSocket, bt_playback_controller_socket, G_TYPE_OBJECT);
+
+
 //-- helper methods
 
 static void client_connection_close(BtPlaybackControllerSocket *self) {
@@ -102,7 +102,7 @@ static gchar *client_cmd_parse_and_process(BtPlaybackControllerSocket *self,gcha
   gchar *reply=NULL;
   BtSong *song;
 
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   if(!song) return(NULL);
 
   if(!strcasecmp(cmd,"browse")) {
@@ -112,9 +112,9 @@ static gchar *client_cmd_parse_and_process(BtPlaybackControllerSocket *self,gcha
     gulong i,length;
     gboolean no_labels=TRUE;
 
-    g_object_get(G_OBJECT(song),"sequence",&sequence,"song-info",&song_info,NULL);
-    g_object_get(G_OBJECT(song_info),"name",&str,NULL);
-    g_object_get(G_OBJECT(sequence),"length",&length,NULL);
+    g_object_get(song,"sequence",&sequence,"song-info",&song_info,NULL);
+    g_object_get(song_info,"name",&str,NULL);
+    g_object_get(sequence,"length",&length,NULL);
 
     reply=g_strconcat("playlist|",str,NULL);
     g_free(str);
@@ -172,7 +172,7 @@ static gchar *client_cmd_parse_and_process(BtPlaybackControllerSocket *self,gcha
     //GST_INFO("seeking to pos=%d",self->priv->cur_pos);
     // this causes stack smashing, but only if we play afterwards
     // its also avoided by making the string buffer in main-statusbar.c (notify) +4 bytes
-    //g_object_set(G_OBJECT(song),"play-pos",self->priv->cur_pos,NULL);
+    //g_object_set(song,"play-pos",self->priv->cur_pos,NULL);
     self->priv->seek=TRUE;
 
   }
@@ -189,9 +189,9 @@ static gchar *client_cmd_parse_and_process(BtPlaybackControllerSocket *self,gcha
     gdouble volume;
     gboolean loop;
 
-    g_object_get(G_OBJECT(song),"play-pos",&pos,NULL);
+    g_object_get(song,"play-pos",&pos,NULL);
     bar_time=bt_sequence_get_bar_time(self->priv->sequence);
-    g_object_get(G_OBJECT(self->priv->sequence),"loop",&loop,NULL);
+    g_object_get(self->priv->sequence,"loop",&loop,NULL);
 
     // calculate playtime
     msec=(gulong)((pos*bar_time)/G_USEC_PER_SEC);
@@ -226,10 +226,10 @@ static gchar *client_cmd_parse_and_process(BtPlaybackControllerSocket *self,gcha
     }
     else if(!strncasecmp(subcmd,"repeat|",7)) {
       if(!strcasecmp(&subcmd[7],"on")) {
-        g_object_set(G_OBJECT(self->priv->sequence),"loop",TRUE,NULL);
+        g_object_set(self->priv->sequence,"loop",TRUE,NULL);
       }
       else if(!strcasecmp(&subcmd[7],"off")) {
-        g_object_set(G_OBJECT(self->priv->sequence),"loop",FALSE,NULL);
+        g_object_set(self->priv->sequence,"loop",FALSE,NULL);
       }
     }
     else {
@@ -253,7 +253,7 @@ static gchar *client_cmd_parse_and_process(BtPlaybackControllerSocket *self,gcha
       gboolean loop;
       gchar *mode[]={"on","off"};
 
-      g_object_get(G_OBJECT(self->priv->sequence),"loop",&loop,NULL);
+      g_object_get(self->priv->sequence,"loop",&loop,NULL);
       reply=g_strdup_printf("repeat|%s",mode[loop]);
     }
     else {
@@ -275,7 +275,7 @@ static gchar *client_read(BtPlaybackControllerSocket *self) {
 
   g_io_channel_read_line(self->priv->client_channel,&str,&len,&term,&error);
   if(!error) {
-    if(str && term>=0) str[term]='\0';
+    if(str && len>0) str[term]='\0';
     GST_INFO("command received : %s",str);
   }
   else {
@@ -386,15 +386,13 @@ static gboolean master_socket_io_handler(GIOChannel *channel,GIOCondition condit
 static void on_song_is_playing_notify(const BtSong *song,GParamSpec *arg,gpointer user_data) {
   BtPlaybackControllerSocket *self=BT_PLAYBACK_CONTROLLER_SOCKET(user_data);
 
-  g_assert(user_data);
-
-  g_object_get(G_OBJECT(song),"is-playing",&self->priv->is_playing,NULL);
+  g_object_get((gpointer)song,"is-playing",&self->priv->is_playing,NULL);
 
   if(self->priv->is_playing && self->priv->seek) {
     // do this only if play was invoked via playbackcontroller
     self->priv->seek=FALSE;
     GST_INFO("seeking to pos=%lu",self->priv->cur_pos);
-    g_object_set(G_OBJECT(song),"play-pos",self->priv->cur_pos,NULL);
+    g_object_set((gpointer)song,"play-pos",self->priv->cur_pos,NULL);
   }
 }
 
@@ -404,24 +402,22 @@ static void on_song_changed(const BtEditApplication *app,GParamSpec *arg,gpointe
   BtSinkMachine *master;
   gulong msec,sec,min;
 
-  g_assert(user_data);
-
   GST_INFO("song has changed : app=%p, toolbar=%p",app,user_data);
 
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   if(!song) return;
 
   self->priv->cur_pos=0;
   client_write(self,"flush");
-  g_signal_connect(G_OBJECT(song),"notify::is-playing",G_CALLBACK(on_song_is_playing_notify),(gpointer)self);
+  g_signal_connect(song,"notify::is-playing",G_CALLBACK(on_song_is_playing_notify),(gpointer)self);
 
   g_object_try_weak_unref(self->priv->sequence);
-  g_object_get(G_OBJECT(song),"sequence",&self->priv->sequence,"master",&master,NULL);
+  g_object_get(song,"sequence",&self->priv->sequence,"master",&master,NULL);
   g_object_try_weak_ref(self->priv->sequence);
   g_object_unref(self->priv->sequence);
 
   g_object_try_weak_unref(self->priv->gain);
-  g_object_get(G_OBJECT(master),"input-gain",&self->priv->gain,NULL);
+  g_object_get(master,"input-gain",&self->priv->gain,NULL);
   g_object_try_weak_ref(self->priv->gain);
   g_object_unref(self->priv->gain);
 
@@ -470,8 +466,8 @@ static void master_connection_open(BtPlaybackControllerSocket *self) {
   guint port;
   static struct sockaddr_in serv_addr;
 
-  g_object_get(G_OBJECT(self->priv->app),"settings",&settings,NULL);
-  g_object_get(G_OBJECT(settings),"coherence-upnp-active",&active,"coherence-upnp-port",&port,NULL);
+  g_object_get(self->priv->app,"settings",&settings,NULL);
+  g_object_get(settings,"coherence-upnp-active",&active,"coherence-upnp-port",&port,NULL);
   g_object_unref(settings);
 
   if(!active) return;
@@ -517,7 +513,7 @@ static void on_port_notify(BtSettings * const settings, GParamSpec * const arg,
   BtPlaybackControllerSocket *self=BT_PLAYBACK_CONTROLLER_SOCKET(user_data);
   gboolean active;
 
-  g_object_get(G_OBJECT(settings),"coherence-upnp-active",&active,NULL);
+  g_object_get(settings,"coherence-upnp-active",&active,NULL);
 
   if(active) {
     master_connection_close(self);
@@ -529,7 +525,7 @@ static void on_active_notify(BtSettings * const settings, GParamSpec * const arg
   BtPlaybackControllerSocket *self=BT_PLAYBACK_CONTROLLER_SOCKET(user_data);
   gboolean active;
 
-  g_object_get(G_OBJECT(settings),"coherence-upnp-active",&active,NULL);
+  g_object_get(settings,"coherence-upnp-active",&active,NULL);
 
   if(active) {
     if(!self->priv->master_channel) {
@@ -544,9 +540,9 @@ static void on_active_notify(BtSettings * const settings, GParamSpec * const arg
 static void settings_listen(BtPlaybackControllerSocket *self) {
   BtSettings *settings;
 
-  g_object_get(G_OBJECT(self->priv->app),"settings",&settings,NULL);
-  g_signal_connect(G_OBJECT(settings), "notify::coherence-upnp-active", G_CALLBACK(on_active_notify), (gpointer)self);
-  g_signal_connect(G_OBJECT(settings), "notify::coherence-upnp-port", G_CALLBACK(on_port_notify), (gpointer)self);
+  g_object_get(self->priv->app,"settings",&settings,NULL);
+  g_signal_connect(settings, "notify::coherence-upnp-active", G_CALLBACK(on_active_notify), (gpointer)self);
+  g_signal_connect(settings, "notify::coherence-upnp-port", G_CALLBACK(on_port_notify), (gpointer)self);
   on_active_notify(settings,NULL,(gpointer)self);
   g_object_unref(settings);
 
@@ -556,22 +552,16 @@ static void settings_listen(BtPlaybackControllerSocket *self) {
 
 /**
  * bt_playback_controller_socket_new:
- * @app: the application to create the controller for
  *
  * Create a new instance
  *
- * Returns: the new instance or %NULL in case of an error
+ * Returns: the new instance
  */
-BtPlaybackControllerSocket *bt_playback_controller_socket_new(const BtEditApplication *app) {
+BtPlaybackControllerSocket *bt_playback_controller_socket_new(void) {
   BtPlaybackControllerSocket *self;
 
-  if(!(self=BT_PLAYBACK_CONTROLLER_SOCKET(g_object_new(BT_TYPE_PLAYBACK_CONTROLLER_SOCKET,"app",app,NULL)))) {
-    goto Error;
-  }
+  self=BT_PLAYBACK_CONTROLLER_SOCKET(g_object_new(BT_TYPE_PLAYBACK_CONTROLLER_SOCKET,NULL));
   return(self);
-Error:
-  if(self) gtk_object_destroy(GTK_OBJECT(self));
-  return(NULL);
 }
 
 //-- methods
@@ -580,50 +570,6 @@ Error:
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_playback_controller_socket_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
-  BtPlaybackControllerSocket *self = BT_PLAYBACK_CONTROLLER_SOCKET(object);
-  return_if_disposed();
-  switch (property_id) {
-    case SETTINGS_DIALOG_APP: {
-      g_value_set_object(value, self->priv->app);
-    } break;
-    default: {
-       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
-/* sets the given properties for this object */
-static void bt_playback_controller_socket_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
-  BtPlaybackControllerSocket *self = BT_PLAYBACK_CONTROLLER_SOCKET(object);
-  return_if_disposed();
-  switch (property_id) {
-    case SETTINGS_DIALOG_APP: {
-      g_object_try_weak_unref(self->priv->app);
-      self->priv->app = BT_EDIT_APPLICATION(g_value_get_object(value));
-      g_object_try_weak_ref(self->priv->app);
-      //GST_DEBUG("set the app for settings_dialog: %p",self->priv->app);
-      // check settings
-      //master_connection_open(self);
-      settings_listen(self);
-      // register event handlers
-      g_signal_connect(G_OBJECT(self->priv->app), "notify::song", G_CALLBACK(on_song_changed), (gpointer)self);
-    } break;
-    default: {
-      G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
 static void bt_playback_controller_socket_dispose(GObject *object) {
   BtPlaybackControllerSocket *self = BT_PLAYBACK_CONTROLLER_SOCKET(object);
 
@@ -631,13 +577,13 @@ static void bt_playback_controller_socket_dispose(GObject *object) {
   self->priv->dispose_has_run = TRUE;
 
   GST_DEBUG("!!!! self=%p",self);
-  g_object_try_weak_unref(self->priv->app);
   g_object_try_weak_unref(self->priv->gain);
   g_object_try_weak_unref(self->priv->sequence);
+  g_object_try_weak_unref(self->priv->app);
 
   master_connection_close(self);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_playback_controller_socket_parent_class)->dispose(object);
 }
 
 static void bt_playback_controller_socket_finalize(GObject *object) {
@@ -653,51 +599,30 @@ static void bt_playback_controller_socket_finalize(GObject *object) {
   g_free(self->priv->length_str);
   g_free(self->priv->cur_label);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(bt_playback_controller_socket_parent_class)->finalize(object);
 }
 
-static void bt_playback_controller_socket_init(GTypeInstance *instance, gpointer g_class) {
-  BtPlaybackControllerSocket *self = BT_PLAYBACK_CONTROLLER_SOCKET(instance);
-
+static void bt_playback_controller_socket_init(BtPlaybackControllerSocket *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_PLAYBACK_CONTROLLER_SOCKET, BtPlaybackControllerSocketPrivate);
+  GST_DEBUG("!!!! self=%p",self);
+  /* this is created from the app, we need to avoid a ref-cycle */
+  self->priv->app = bt_edit_application_new();
+  g_object_try_weak_ref(self->priv->app);
+  g_object_unref(self->priv->app);
+
+  // check settings
+  //master_connection_open(self);
+  settings_listen(self);
+  // register event handlers
+  g_signal_connect(self->priv->app, "notify::song", G_CALLBACK(on_song_changed), (gpointer)self);
 }
 
 static void bt_playback_controller_socket_class_init(BtPlaybackControllerSocketClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtPlaybackControllerSocketPrivate));
 
-  gobject_class->set_property = bt_playback_controller_socket_set_property;
-  gobject_class->get_property = bt_playback_controller_socket_get_property;
   gobject_class->dispose      = bt_playback_controller_socket_dispose;
   gobject_class->finalize     = bt_playback_controller_socket_finalize;
-
-  g_object_class_install_property(gobject_class,SETTINGS_DIALOG_APP,
-                                  g_param_spec_object("app",
-                                     "app construct prop",
-                                     "Set application object, the dialog belongs to",
-                                     BT_TYPE_EDIT_APPLICATION, /* object type */
-                                     G_PARAM_CONSTRUCT_ONLY|G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
-
 }
 
-GType bt_playback_controller_socket_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtPlaybackControllerSocketClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_playback_controller_socket_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtPlaybackControllerSocket),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_playback_controller_socket_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(G_TYPE_OBJECT,"BtPlaybackControllerSocket",&info,0);
-  }
-  return type;
-}
