@@ -1,4 +1,4 @@
-/* $Id: ui-resources.c 2543 2009-07-31 20:35:05Z ensonic $
+/* $Id: ui-resources.c 3642 2011-09-16 12:31:35Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -25,7 +25,7 @@
  * This class serves as a central storage for colors and icons.
  * It is implemented as a singleton.
  */
- 
+
 /* @todo: manage GdkColor entries and guint colors (via gdk_color.pixel)
  */
 
@@ -45,7 +45,7 @@ struct _BtUIResourcesPrivate {
 
   /* colors */
   GdkColor colors[BT_UI_RES_COLOR_COUNT];
-  
+
   /* the keyboard shortcut table for the window */
   GtkAccelGroup *accel_group;
 
@@ -56,9 +56,11 @@ struct _BtUIResourcesPrivate {
   GdkPixbuf *sink_machine_pixbufs[BT_MACHINE_STATE_COUNT];
 };
 
-static GObjectClass *parent_class=NULL;
+static BtUIResources *singleton=NULL;
 
-static gpointer singleton=NULL;
+//-- the class
+
+G_DEFINE_TYPE (BtUIResources, bt_ui_resources, G_TYPE_OBJECT);
 
 //-- event handler
 
@@ -79,14 +81,14 @@ static gpointer singleton=NULL;
   self->priv->colors[ix].green=(guint16)((g2+((g1-g2)>>1))*256); \
   self->priv->colors[ix].blue= (guint16)((b2+((b1-b2)>>1))*256)
 
-static gboolean bt_ui_resources_init_colors(BtUIResources *self) {
+static void bt_ui_resources_init_colors(BtUIResources *self) {
   GdkColormap *colormap;
   GtkSettings *settings;
   gboolean color_res[BT_UI_RES_COLOR_COUNT];
   gint res;
   gchar *icon_theme_name;
   gboolean use_tango_colors=FALSE;
-  
+
   settings=gtk_settings_get_default();
   /* get the theme name - we need different machine colors for tango
    * http://tango.freedesktop.org/Tango_Icon_Theme_Guidelines
@@ -94,29 +96,29 @@ static gboolean bt_ui_resources_init_colors(BtUIResources *self) {
    */
   g_object_get(settings, "gtk-icon-theme-name", &icon_theme_name, NULL);
   GST_INFO("Icon Theme: %s",icon_theme_name);
-  
+
   /* @todo: can we get some colors from the theme ?
    * gtk_widget_style_get(widget,
    *   "cursor-color",&self->priv->colors[ix],
    *   "secondary-cursor-color",&self->priv->colors[ix],
    *   NULL);
    */
-   
+
   if(!strcasecmp(icon_theme_name,"tango") || !strcasecmp(icon_theme_name,"gnome"))
     use_tango_colors=TRUE;
-  
+
   // cursor
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_CURSOR,                    0.85,0.85,0.20);
-  
+
   // selection background
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_SELECTION1,                1.00,1.00,0.60);
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_SELECTION2,                0.95,0.95,0.55);
-  
+
   // tree view lines
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_PLAYLINE,                  0.00,0.00,1.00);
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_LOOPLINE,                  1.00,0.75,0.00);
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_ENDLINE,                   1.00,0.30,0.20);
-  
+
   // source machine
   if(use_tango_colors) {
     /* (#ffd699) #fcaf3e / #f57900 / #ce5c00
@@ -139,7 +141,7 @@ static gboolean bt_ui_resources_init_colors(BtUIResources *self) {
     MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_SOURCE_MACHINE_DARK1,      0.60,0.40,0.40);
     MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_SOURCE_MACHINE_DARK2,      0.50,0.20,0.20);
   }
-  
+
   // processor machine
   if(use_tango_colors) {
     /* (#cbff99) #8ae234 / #73d216 / #4e9a06
@@ -185,13 +187,13 @@ static gboolean bt_ui_resources_init_colors(BtUIResources *self) {
     MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_SINK_MACHINE_DARK1,        0.40,0.40,0.60);
     MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_SINK_MACHINE_DARK2,        0.20,0.20,0.50);
   }
-  
+
   // analyzer window
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_ANALYZER_PEAK,             1.00,0.75,0.00);
   MAKE_COLOR_FROM_FLOATS(BT_UI_RES_COLOR_GRID_LINES,                0.5,0.5,0.5);
-  
+
   g_free(icon_theme_name);
-   
+
   // now allocate colors
   colormap=gdk_colormap_get_system();
   if((res=gdk_colormap_alloc_colors(colormap,self->priv->colors,BT_UI_RES_COLOR_COUNT,FALSE,TRUE,color_res))) {
@@ -203,23 +205,22 @@ static gboolean bt_ui_resources_init_colors(BtUIResources *self) {
       }
     }
   }
-
-  return(TRUE);
+  GST_INFO("colors created");
 }
 
 
-static gboolean bt_ui_resources_init_icons(BtUIResources *self) {
+static void bt_ui_resources_init_icons(BtUIResources *self) {
   GtkSettings *settings;
   gchar *icon_theme_name,*fallback_icon_theme_name;
   gint w,h;
-  
+
   settings=gtk_settings_get_default();
-  g_object_get(settings, 
+  g_object_get(settings,
     "gtk-icon-theme-name", &icon_theme_name,
     "gtk-fallback-icon-theme", &fallback_icon_theme_name,
     NULL);
   GST_INFO("Icon Theme: %s, Fallback Icon Theme: %s",icon_theme_name,fallback_icon_theme_name);
-  
+
   if(strcasecmp(icon_theme_name,"gnome")) {
     if(fallback_icon_theme_name && strcasecmp(fallback_icon_theme_name,"gnome")) {
       g_object_set(settings,"gtk-fallback-icon-theme","gnome",NULL);
@@ -240,14 +241,14 @@ static gboolean bt_ui_resources_init_icons(BtUIResources *self) {
   self->priv->processor_machine_pixbuf=gdk_pixbuf_new_from_theme("buzztard_menu_processor_machine",w);
   self->priv->sink_machine_pixbuf     =gdk_pixbuf_new_from_theme("buzztard_menu_sink_machine",w);
 
-  return(TRUE);
+  GST_INFO("images created");
 }
 
 /*
 static GdkPixbuf *bt_ui_resources_load_svg(const gchar *file_name) {
   GError *error = NULL;
   GdkPixbuf *pixbuf;
-  
+
   pixbuf = rsvg_pixbuf_from_file_at_size (file_name, 96, 96, &error);
   if(error) {
     GST_WARNING ("loading svg %s failed : %s", file_name, error->message);
@@ -266,13 +267,16 @@ static void bt_ui_resources_free_graphics(BtUIResources *self) {
     g_object_try_unref(self->priv->sink_machine_pixbufs[state]);
   }
 }
-  
-static gboolean bt_ui_resources_init_graphics(BtUIResources *self) {
+
+static void bt_ui_resources_init_graphics(BtUIResources *self) {
   // 12*6=72, 14*6=84
   const gint size=(gint)(self->priv->zoom*(gdouble)(GTK_ICON_SIZE_DIALOG*14));
-  
+  //const gint size=(gint)(self->priv->zoom*(gdouble)(6*14));
+
+  GST_INFO("regenerating machine graphics at %d pixels",size);
+
   //self->priv->source_machine_pixbufs[BT_MACHINE_STATE_NORMAL] = bt_ui_resources_load_svg ("generator.svg");
-  
+
   self->priv->source_machine_pixbufs   [BT_MACHINE_STATE_NORMAL]=gdk_pixbuf_new_from_theme("buzztard_generator",size);
   self->priv->source_machine_pixbufs   [BT_MACHINE_STATE_MUTE  ]=gdk_pixbuf_new_from_theme("buzztard_generator_mute",size);
   self->priv->source_machine_pixbufs   [BT_MACHINE_STATE_SOLO  ]=gdk_pixbuf_new_from_theme("buzztard_generator_solo",size);
@@ -283,14 +287,12 @@ static gboolean bt_ui_resources_init_graphics(BtUIResources *self) {
 
   self->priv->sink_machine_pixbufs     [BT_MACHINE_STATE_NORMAL]=gdk_pixbuf_new_from_theme("buzztard_master",size);
   self->priv->sink_machine_pixbufs     [BT_MACHINE_STATE_MUTE  ]=gdk_pixbuf_new_from_theme("buzztard_master_mute",size);
-   
+
   /* DEBUG
   gint w,h;
   g_object_get(self->priv->source_machine_pixbufs[BT_MACHINE_STATE_NORMAL],"width",&w,"height",&h,NULL);
-  GST_DEBUG("svg: w,h = %d, %d",w,h);  
+  GST_DEBUG("svg: w,h = %d, %d",w,h);
   */
-
-  return(TRUE);
 }
 
 //-- constructor methods
@@ -298,37 +300,12 @@ static gboolean bt_ui_resources_init_graphics(BtUIResources *self) {
 /**
  * bt_ui_resources_new:
  *
- * Create a new instance
+ * Create a new instance on first call and return a reference later on.
  *
- * Returns: the new instance or %NULL in case of an error
+ * Returns: the new signleton instance
  */
 BtUIResources *bt_ui_resources_new(void) {
-  if(!singleton) {
-    // create singleton
-    if(!(singleton=(gpointer)(g_object_new(BT_TYPE_UI_RESOURCES,NULL)))) {
-      goto Error;
-    }
-    else {
-      BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
-      // initialise ressources
-      if(!bt_ui_resources_init_colors(ui_resources)) {
-        goto Error;
-      }
-      if(!bt_ui_resources_init_icons(ui_resources)) {
-        goto Error;
-      }
-      ui_resources->priv->accel_group=gtk_accel_group_new();
-  
-      g_object_add_weak_pointer(G_OBJECT(singleton),&singleton);
-    }
-  }
-  else {
-    singleton=g_object_ref(G_OBJECT(singleton));
-  } 
-  return(BT_UI_RESOURCES(singleton));
-Error:
-  g_object_try_unref(G_OBJECT(singleton));
-  return(NULL);
+  return (g_object_new(BT_TYPE_UI_RESOURCES,NULL));
 }
 
 //-- methods
@@ -342,16 +319,14 @@ Error:
  * Returns: a #GdkPixbuf image
  */
 GdkPixbuf *bt_ui_resources_get_icon_pixbuf_by_machine(const BtMachine *machine) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
-
   if(BT_IS_SOURCE_MACHINE(machine)) {
-    return(g_object_ref(ui_resources->priv->source_machine_pixbuf));
+    return(g_object_ref(singleton->priv->source_machine_pixbuf));
   }
   else if(BT_IS_PROCESSOR_MACHINE(machine)) {
-    return(g_object_ref(ui_resources->priv->processor_machine_pixbuf));
+    return(g_object_ref(singleton->priv->processor_machine_pixbuf));
   }
   else if(BT_IS_SINK_MACHINE(machine)) {
-    return(g_object_ref(ui_resources->priv->sink_machine_pixbuf));
+    return(g_object_ref(singleton->priv->sink_machine_pixbuf));
   }
   return(NULL);
 }
@@ -367,26 +342,25 @@ GdkPixbuf *bt_ui_resources_get_icon_pixbuf_by_machine(const BtMachine *machine)
  * Returns: a #GdkPixbuf image
  */
 GdkPixbuf *bt_ui_resources_get_machine_graphics_pixbuf_by_machine(const BtMachine *machine, gdouble zoom) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
   BtMachineState state;
-  
-  if(zoom!=ui_resources->priv->zoom) {
-    GST_DEBUG("change zoom %f -> %f",ui_resources->priv->zoom,zoom);
-    bt_ui_resources_free_graphics(ui_resources);
-    ui_resources->priv->zoom=zoom;
-    bt_ui_resources_init_graphics(ui_resources);
+
+  if(zoom!=singleton->priv->zoom) {
+    GST_DEBUG("change zoom %f -> %f",singleton->priv->zoom,zoom);
+    bt_ui_resources_free_graphics(singleton);
+    singleton->priv->zoom=zoom;
+    bt_ui_resources_init_graphics(singleton);
   }
-  
-  g_object_get(G_OBJECT(machine),"state",&state,NULL);
+
+  g_object_get((gpointer)machine,"state",&state,NULL);
 
   if(BT_IS_SOURCE_MACHINE(machine)) {
-    return(g_object_ref(ui_resources->priv->source_machine_pixbufs[state]));
+    return(g_object_ref(singleton->priv->source_machine_pixbufs[state]));
   }
   else if(BT_IS_PROCESSOR_MACHINE(machine)) {
-    return(g_object_ref(ui_resources->priv->processor_machine_pixbufs[state]));
+    return(g_object_ref(singleton->priv->processor_machine_pixbufs[state]));
   }
   else if(BT_IS_SINK_MACHINE(machine)) {
-    return(g_object_ref(ui_resources->priv->sink_machine_pixbufs[state]));
+    return(g_object_ref(singleton->priv->sink_machine_pixbufs[state]));
   }
   return(NULL);
 }
@@ -400,16 +374,14 @@ GdkPixbuf *bt_ui_resources_get_machine_graphics_pixbuf_by_machine(const BtMachin
  * Returns: a #GtkImage widget
  */
 GtkWidget *bt_ui_resources_get_icon_image_by_machine(const BtMachine *machine) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
-
   if(BT_IS_SOURCE_MACHINE(machine)) {
-    return(gtk_image_new_from_pixbuf(ui_resources->priv->source_machine_pixbuf));
+    return(gtk_image_new_from_pixbuf(singleton->priv->source_machine_pixbuf));
   }
   else if(BT_IS_PROCESSOR_MACHINE(machine)) {
-    return(gtk_image_new_from_pixbuf(ui_resources->priv->processor_machine_pixbuf));
+    return(gtk_image_new_from_pixbuf(singleton->priv->processor_machine_pixbuf));
   }
   else if(BT_IS_SINK_MACHINE(machine)) {
-    return(gtk_image_new_from_pixbuf(ui_resources->priv->sink_machine_pixbuf));
+    return(gtk_image_new_from_pixbuf(singleton->priv->sink_machine_pixbuf));
   }
   return(NULL);
 }
@@ -423,16 +395,14 @@ GtkWidget *bt_ui_resources_get_icon_image_by_machine(const BtMachine *machine) {
  * Returns: a #GtkImage widget
  */
 GtkWidget *bt_ui_resources_get_icon_image_by_machine_type(GType machine_type) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
-
   if(machine_type==BT_TYPE_SOURCE_MACHINE) {
-    return(gtk_image_new_from_pixbuf(ui_resources->priv->source_machine_pixbuf));
+    return(gtk_image_new_from_pixbuf(singleton->priv->source_machine_pixbuf));
   }
   else if(machine_type==BT_TYPE_PROCESSOR_MACHINE) {
-    return(gtk_image_new_from_pixbuf(ui_resources->priv->processor_machine_pixbuf));
+    return(gtk_image_new_from_pixbuf(singleton->priv->processor_machine_pixbuf));
   }
   else if(machine_type==BT_TYPE_SINK_MACHINE) {
-    return(gtk_image_new_from_pixbuf(ui_resources->priv->sink_machine_pixbuf));
+    return(gtk_image_new_from_pixbuf(singleton->priv->sink_machine_pixbuf));
   }
   return(NULL);
 }
@@ -446,8 +416,25 @@ GtkWidget *bt_ui_resources_get_icon_image_by_machine_type(GType machine_type) {
  * Returns: the requested #GdkColor.
  */
 GdkColor *bt_ui_resources_get_gdk_color(BtUIResourcesColors color_type) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
-  return(&ui_resources->priv->colors[color_type]);
+  return(&singleton->priv->colors[color_type]);
+}
+
+/**
+ * bt_ui_resources_get_rgb_color:
+ * @color_type: the color id
+ * @r: target for red color part
+ * @g: target for green color part
+ * @b: target for blue color part
+ *
+ * Gets a prealocated color by id. Sets the given parts to values from 0.0 to
+ * 1.0.
+ */
+void bt_ui_resources_get_rgb_color(BtUIResourcesColors color_type, gdouble *r, gdouble *g, gdouble *b) {
+  GdkColor *c=&singleton->priv->colors[color_type];
+
+  *r=c->red/65535.0;
+  *g=c->green/65535.0;
+  *b=c->blue/65535.0;
 }
 
 /**
@@ -460,10 +447,10 @@ GdkColor *bt_ui_resources_get_gdk_color(BtUIResourcesColors color_type) {
  * Returns: a color depending on machine class and color_type
  */
 guint32 bt_ui_resources_get_color_by_machine(const BtMachine *machine,BtUIResourcesMachineColors color_type) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
   gulong ix=0;
   guint32 color=0;
-  
+  GdkColor *c;
+
   if(BT_IS_SOURCE_MACHINE(machine)) {
     ix=BT_UI_RES_COLOR_SOURCE_MACHINE_BASE+color_type;
   }
@@ -473,9 +460,10 @@ guint32 bt_ui_resources_get_color_by_machine(const BtMachine *machine,BtUIResour
   else if(BT_IS_SINK_MACHINE(machine)) {
     ix=BT_UI_RES_COLOR_SINK_MACHINE_BASE+color_type;
   }
-  color=(((guint32)(ui_resources->priv->colors[ix].red&0xFF00))<<16)|
-    (((guint32)(ui_resources->priv->colors[ix].green&0xFF00))<<8)|
-    ((guint32)(ui_resources->priv->colors[ix].blue&0xFF00))|
+  c=&singleton->priv->colors[ix];
+  color=(((guint32)(c->red&0xFF00))<<16)|
+    (((guint32)(c->green&0xFF00))<<8)|
+    ((guint32)(c->blue&0xFF00))|
     0x000000FF;
   //GST_INFO("color[%2d/%1d] : 0x%08lx : %04x,%04x,%04x",ix,color_type,color,
   //  ui_resources->priv->colors[ix].red,ui_resources->priv->colors[ix].green,ui_resources->priv->colors[ix].blue);
@@ -490,110 +478,65 @@ guint32 bt_ui_resources_get_color_by_machine(const BtMachine *machine,BtUIResour
  * Returns: the shared keyboard accelerator map
  */
 GtkAccelGroup *bt_ui_resources_get_accel_group(void) {
-  BtUIResources *ui_resources=BT_UI_RESOURCES(singleton);
-  
-  return(ui_resources->priv->accel_group);
+  return(singleton->priv->accel_group);
 }
 
 //-- wrapper
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_ui_resources_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
-  BtUIResources *self = BT_UI_RESOURCES(object);
-  return_if_disposed();
-  switch (property_id) {
-    default: {
-       G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
-/* sets the given properties for this object */
-static void bt_ui_resources_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
-  BtUIResources *self = BT_UI_RESOURCES(object);
-  return_if_disposed();
-  switch (property_id) {
-    default: {
-      G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
-    } break;
-  }
-}
-
 static void bt_ui_resources_dispose(GObject *object) {
   BtUIResources *self = BT_UI_RESOURCES(object);
-  
+
   return_if_disposed();
   self->priv->dispose_has_run = TRUE;
 
   GST_DEBUG("!!!! self=%p",self);
-  
-  GST_DEBUG("  pb->ref_counts: %d, %d, %d", 
-    self->priv->source_machine_pixbuf?G_OBJECT(self->priv->source_machine_pixbuf)->ref_count:0,
-    self->priv->processor_machine_pixbuf?G_OBJECT(self->priv->processor_machine_pixbuf)->ref_count:0,
-    self->priv->sink_machine_pixbuf?G_OBJECT(self->priv->sink_machine_pixbuf)->ref_count:0);
+
+  GST_DEBUG("  pb->ref_cts: %d, %d, %d",
+    G_OBJECT_REF_COUNT(self->priv->source_machine_pixbuf),
+    G_OBJECT_REF_COUNT(self->priv->processor_machine_pixbuf),
+    G_OBJECT_REF_COUNT(self->priv->sink_machine_pixbuf));
   g_object_try_unref(self->priv->source_machine_pixbuf);
   g_object_try_unref(self->priv->processor_machine_pixbuf);
   g_object_try_unref(self->priv->sink_machine_pixbuf);
-  
+
   bt_ui_resources_free_graphics(self);
-  
+
   g_object_try_unref(self->priv->accel_group);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(bt_ui_resources_parent_class)->dispose(object);
 }
 
-static void bt_ui_resources_finalize(GObject *object) {
-  BtUIResources *self = BT_UI_RESOURCES(object);
-  
-  GST_DEBUG("!!!! self=%p",self);
+static GObject *bt_ui_resources_constructor(GType type,guint n_construct_params,GObjectConstructParam *construct_params) {
+  GObject *object;
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  if(G_UNLIKELY(!singleton)) {
+    object=G_OBJECT_CLASS(bt_ui_resources_parent_class)->constructor(type,n_construct_params,construct_params);
+    singleton=BT_UI_RESOURCES(object);
+    g_object_add_weak_pointer(object,(gpointer*)(gpointer)&singleton);
+
+    // initialise ressources
+    bt_ui_resources_init_colors(singleton);
+    bt_ui_resources_init_icons(singleton);
+    singleton->priv->accel_group=gtk_accel_group_new();
+  }
+  else {
+    object=g_object_ref(singleton);
+  }
+  return object;
 }
 
-static void bt_ui_resources_init(GTypeInstance *instance, gpointer g_class) {
-  BtUIResources *self = BT_UI_RESOURCES(instance);
-  
+static void bt_ui_resources_init(BtUIResources *self) {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_UI_RESOURCES, BtUIResourcesPrivate);
 }
 
 static void bt_ui_resources_class_init(BtUIResourcesClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
   g_type_class_add_private(klass,sizeof(BtUIResourcesPrivate));
 
-  gobject_class->set_property = bt_ui_resources_set_property;
-  gobject_class->get_property = bt_ui_resources_get_property;
+  gobject_class->constructor  = bt_ui_resources_constructor;
   gobject_class->dispose      = bt_ui_resources_dispose;
-  gobject_class->finalize     = bt_ui_resources_finalize;
 }
 
-GType bt_ui_resources_get_type(void) {
-  static GType type = 0;
-  if (type == 0) {
-    const GTypeInfo info = {
-      sizeof(BtUIResourcesClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_ui_resources_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtUIResources),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_ui_resources_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(G_TYPE_OBJECT,"BtUIResources",&info,0);
-  }
-  return type;
-}
