@@ -1,4 +1,4 @@
-/* $Id: wire-canvas-item.c 2333 2009-04-05 20:49:42Z ensonic $
+/* $Id: wire-canvas-item.c 3642 2011-09-16 12:31:35Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -21,17 +21,21 @@
 /**
  * SECTION:btwirecanvasitem
  * @short_description: class for the editor wire views wire canvas item
- * @see_also: #btwireanalysisdialog
+ * @see_also: #BtSignalAnalysisDialog
  *
  * Provides volume control on the wires, as well as a menu to disconnect wires
  * and to launch the analyzer screen.
  */
-/* @todo:
- * - right now a click on the triangle pops up the volume or panorama slider
- *   - it could popup a whole mixer strip
- * - add "insert effect" to context menu, could use the machine menu and work
- *   simillar to connect menu on canvas, would turn existing wire into 2nd wire
- *   to preserve wirepatterns
+/* @todo: mixer strip
+ *   - right now a click on the triangle pops up the volume or panorama slider
+ *   - it could popup a whole mixer strip (with level meters)
+ *
+ * @todo: add "insert effect" to context menu
+ *   - could use the machine menu and work simillar to connect menu on canvas
+ *   - would turn existing wire into 2nd wire to preserve wirepatterns
+ *
+ * @todo: dialog manager
+ *   - see machine-canvas-item.c
  */
 #define BT_EDIT
 #define BT_WIRE_CANVAS_ITEM_C
@@ -41,13 +45,13 @@
 //-- property ids
 
 enum {
-  WIRE_CANVAS_ITEM_APP=1,
-  WIRE_CANVAS_ITEM_MACHINES_PAGE,
+  WIRE_CANVAS_ITEM_MACHINES_PAGE=1,
   WIRE_CANVAS_ITEM_WIRE,
   WIRE_CANVAS_ITEM_W,
   WIRE_CANVAS_ITEM_H,
   WIRE_CANVAS_ITEM_SRC,
-  WIRE_CANVAS_ITEM_DST
+  WIRE_CANVAS_ITEM_DST,
+  WIRE_CANVAS_ITEM_ANALYSIS_DIALOG
 };
 
 // @todo what do we set here? canvas dimensions?
@@ -58,7 +62,8 @@ struct _BtWireCanvasItemPrivate {
   gboolean dispose_has_run;
 
   /* the application */
-  G_POINTER_ALIAS(BtEditApplication *,app);
+  BtEditApplication *app;
+
   /* the machine page we are on */
   G_POINTER_ALIAS(BtMainPageMachines *,main_page_machines);
 
@@ -91,13 +96,21 @@ struct _BtWireCanvasItemPrivate {
   gdouble offx,offy,dragx,dragy;
 };
 
-static GnomeCanvasGroupClass *parent_class=NULL;
+static GQuark wire_canvas_item_quark=0;
+
+//-- the class
+
+G_DEFINE_TYPE (BtWireCanvasItem, bt_wire_canvas_item, GNOME_TYPE_CANVAS_GROUP);
+
+//-- prototypes
+
+static void on_signal_analysis_dialog_destroy(GtkWidget *widget, gpointer user_data);
 
 //-- helper
 
 static void wire_set_line_points(BtWireCanvasItem *self) {
   GnomeCanvasPoints *points;
- 
+
   points=gnome_canvas_points_new(2);
   points->coords[0]=0.0;
   points->coords[1]=0.0;
@@ -144,7 +157,7 @@ static void wire_set_triangle_points(BtWireCanvasItem *self) {
     dx=dy=0.0;
     //df=0.0;
   }
-  
+
   // first triangle
   part_x=mid_x+(s2*dx);
   part_y=mid_y+(s2*dy);
@@ -162,7 +175,7 @@ static void wire_set_triangle_points(BtWireCanvasItem *self) {
   // point over the line
   base12_x=base_x+(sb*dy);
   base12_y=base_y-(sb*dx);
-  
+
   // second triangle
   part_x=mid_x-(s2*dx);
   part_y=mid_y-(s2*dy);
@@ -177,7 +190,7 @@ static void wire_set_triangle_points(BtWireCanvasItem *self) {
   // point over the line
   base22_x=base_x+(sb*dy);
   base22_y=base_y-(sb*dx);
-  
+
   /* // debug
   GST_DEBUG(" delta=%f,%f, s=%f, sa=%f sb=%f",dx,dy,s,sa,sb);
   GST_DEBUG(" w/h=%f,%f",w,h);
@@ -194,14 +207,14 @@ static void wire_set_triangle_points(BtWireCanvasItem *self) {
   points->coords[3]=tip1_y;
   points->coords[4]=base12_x;
   points->coords[5]=base12_y;
-  
+
   points->coords[6]=base22_x;
   points->coords[7]=base22_y;
   points->coords[8]=base21_x;
   points->coords[9]=base21_y;
   gnome_canvas_item_set(GNOME_CANVAS_ITEM(self->priv->triangle),"points",points,NULL);
   gnome_canvas_points_free(points);
-  
+
   gdouble ang=-M_PI_2+atan2(dx,dy);
   gdouble affine[]={cos(ang),-sin(ang),sin(ang),cos(ang),0.0,0.0};
   gnome_canvas_item_affine_absolute(GNOME_CANVAS_ITEM(self->priv->vol_item),affine);
@@ -216,15 +229,26 @@ static void wire_set_triangle_points(BtWireCanvasItem *self) {
     NULL);
 }
 
+static void show_wire_analyzer_dialog(BtWireCanvasItem *self) {
+  if(!self->priv->analysis_dialog) {
+    self->priv->analysis_dialog=GTK_WIDGET(bt_signal_analysis_dialog_new(GST_BIN(self->priv->wire)));
+    bt_edit_application_attach_child_window(self->priv->app,GTK_WINDOW(self->priv->analysis_dialog));
+    GST_INFO("analyzer dialog opened");
+    // remember open/closed state
+    g_hash_table_insert(self->priv->properties,g_strdup("analyzer-shown"),g_strdup("1"));
+    g_signal_connect(self->priv->analysis_dialog,"destroy",G_CALLBACK(on_signal_analysis_dialog_destroy),(gpointer)self);
+    g_object_notify((GObject *)self,"analysis-dialog");
+  }
+  gtk_window_present(GTK_WINDOW(self->priv->analysis_dialog));
+}
 //-- event handler
 
-static void on_wire_analysis_dialog_destroy(GtkWidget *widget, gpointer user_data) {
+static void on_signal_analysis_dialog_destroy(GtkWidget *widget, gpointer user_data) {
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
 
-  g_assert(user_data);
-
-  GST_INFO("wire analysis dialog destroy occurred");
+  GST_INFO("signal analysis dialog destroy occurred");
   self->priv->analysis_dialog=NULL;
+  g_object_notify((GObject *)self,"analysis-dialog");
   // remember open/closed state
   g_hash_table_remove(self->priv->properties,"analyzer-shown");
 }
@@ -233,52 +257,52 @@ static void on_machine_removed(BtSetup *setup,BtMachine *machine,gpointer user_d
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
   BtMachine *src,*dst;
 
-  g_assert(user_data);
   g_return_if_fail(BT_IS_MACHINE(machine));
 
   g_object_get(self->priv->src,"machine",&src,NULL);
   g_object_get(self->priv->dst,"machine",&dst,NULL);
 
-  GST_INFO("machine %p,machine->ref_ct=%d has been removed, checking wire %p->%p", machine,G_OBJECT(machine)->ref_count,src,dst);
+  GST_INFO("machine %p,machine->ref_ct=%d has been removed, checking wire %p->%p", machine,G_OBJECT_REF_COUNT(machine),src,dst);
   if((src==machine) || (dst==machine) || (src==NULL) || (dst==NULL)) {
     GST_INFO("the machine, this wire is connected to, has been removed");
     bt_setup_remove_wire(setup,self->priv->wire);
 
-    GST_INFO("... machine %p,ref_count=%d has been removed, src %p,ref=%d, dst %p,ref=%d",
-      machine,G_OBJECT(machine)->ref_count,
-      src,(src?G_OBJECT(src)->ref_count:0),
-      dst,(dst?G_OBJECT(dst)->ref_count:0)
+    GST_INFO("... machine %p,ref_ct=%d has been removed, src %p,ref=%d, dst %p,ref=%d",
+      machine,G_OBJECT_REF_COUNT(machine),
+      src,G_OBJECT_REF_COUNT(src),
+      dst,G_OBJECT_REF_COUNT(dst)
     );
   }
   g_object_try_unref(src);
   g_object_try_unref(dst);
 }
 
-static void on_wire_position_changed(BtMachineCanvasItem *machine_item, gpointer user_data) {
+static void on_wire_src_position_changed(BtMachineCanvasItem *machine_item, gpointer user_data) {
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
-  BtMachine *machine;
-  GHashTable *properties;
-  gdouble pos_xs,pos_ys,pos_xe,pos_ye;
+  gdouble px,py;
+  gdouble wx,wy,ww,wh;
+  gdouble dx,dy;
 
-  //GST_INFO("wire pos has changed : machine_item=%p, user_data=%p",machine_item,user_data);
+  g_object_get(machine_item,"x",&px,"y",&py,NULL);
+  g_object_get(self,"x",&wx,"y",&wy,"w",&ww,"h",&wh,NULL);
+  dx=wx-px;
+  dy=wy-py;
+  g_object_set(self,"x",px,"y",py,"w",ww+dx,"h",wh+dy,NULL);
 
-  g_object_get(self->priv->src,"machine",&machine,NULL);
-  g_object_get(machine,"properties",&properties,NULL);
-  machine_view_get_machine_position(properties,&pos_xs,&pos_ys);
-  g_object_unref(machine);
-  g_object_get(self->priv->dst,"machine",&machine,NULL);
-  g_object_get(machine,"properties",&properties,NULL);
-  machine_view_get_machine_position(properties,&pos_xe,&pos_ye);
-  g_object_unref(machine);
+  // we need to reset all the coords for our wire items now
+  wire_set_line_points(self);
+  wire_set_triangle_points(self);
+}
 
-  //GST_INFO("  set new coords: %+5.1f,%+5.1f %+5.1f,%+5.1f",pos_xs,pos_ys,pos_xe,pos_ye);
+static void on_wire_dst_position_changed(BtMachineCanvasItem *machine_item, gpointer user_data) {
+  BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
+  gdouble px,py;
+  gdouble wx,wy;
+
+  g_object_get(machine_item,"x",&px,"y",&py,NULL);
+  g_object_get(self,"x",&wx,"y",&wy,NULL);
+  g_object_set(self,"x",wx,"y",wy,"w",px-wx,"h",py-wy,NULL);
 
-  gnome_canvas_item_set(GNOME_CANVAS_ITEM(self),
-                      "x", pos_xs,
-                      "y", pos_ys,
-                      "w", (pos_xe-pos_xs),
-                      "h", (pos_ye-pos_ys),
-                      NULL);
   // we need to reset all the coords for our wire items now
   wire_set_line_points(self);
   wire_set_triangle_points(self);
@@ -286,49 +310,24 @@ static void on_wire_position_changed(BtMachineCanvasItem *machine_item, gpointer
 
 static void on_context_menu_disconnect_activate(GtkMenuItem *menuitem,gpointer user_data) {
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
-  BtSong *song;
-  BtSetup *setup;
 
-  g_assert(user_data);
-  GST_INFO("context_menu disconnect item selected");
-
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"setup",&setup,NULL);
-
-  GST_INFO("now removing wire : %p,ref_count=%d",self->priv->wire,G_OBJECT(self->priv->wire)->ref_count);
-  bt_setup_remove_wire(setup,self->priv->wire);
-  //GST_INFO("... wire : %p,ref_count=%d",self->priv->wire,G_OBJECT(self->priv->wire)->ref_count);
-
-  g_object_unref(setup);
-  g_object_unref(song);
+  bt_main_page_machines_delete_wire(self->priv->main_page_machines,self->priv->wire);
 }
 
 static void on_context_menu_analysis_activate(GtkMenuItem *menuitem,gpointer user_data) {
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
 
-  g_assert(user_data);
-  GST_INFO("context_menu analysis item selected");
-  if(!self->priv->analysis_dialog) {
-    if((self->priv->analysis_dialog=GTK_WIDGET(bt_wire_analysis_dialog_new(self->priv->app,self->priv->wire)))) {
-      GST_INFO("analyzer dialog opened");
-      // remember open/closed state
-      g_hash_table_insert(self->priv->properties,g_strdup("analyzer-shown"),g_strdup("1"));
-      g_signal_connect(G_OBJECT(self->priv->analysis_dialog),"destroy",G_CALLBACK(on_wire_analysis_dialog_destroy),(gpointer)self);
-    }
-  }
-  else {
-    gtk_window_present(GTK_WINDOW(self->priv->analysis_dialog));
-  }
+  show_wire_analyzer_dialog(self);
 }
 
 static void on_gain_changed(GstElement *element, GParamSpec *arg, gpointer user_data) {
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
   gdouble s=MACHINE_VIEW_WIRE_PAD_SIZE,ox=2.5*s,px=-ox+(1.3*s);
   gdouble gain;
-  
-  g_object_get(G_OBJECT(self->priv->wire_gain),"volume",&gain,NULL);
+
+  g_object_get(self->priv->wire_gain,"volume",&gain,NULL);
   // do some sensible clamping
-  if(gain>4.0) gain=4.0;  
+  if(gain>4.0) gain=4.0;
   gnome_canvas_item_set(GNOME_CANVAS_ITEM(self->priv->vol_level_item),"x2", px+(gain*0.55*s),NULL);
 }
 
@@ -336,8 +335,8 @@ static void on_pan_changed(GstElement *element, GParamSpec *arg, gpointer user_d
   BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(user_data);
   gdouble s=MACHINE_VIEW_WIRE_PAD_SIZE,ox=2.5*s,px=-ox+(1.3*s);
   gfloat pan;
-  
-  g_object_get(G_OBJECT(self->priv->wire_pan),"panorama",&pan,NULL);
+
+  g_object_get(self->priv->wire_pan,"panorama",&pan,NULL);
   if(pan<0.0) {
     gnome_canvas_item_set(GNOME_CANVAS_ITEM(self->priv->pan_pos_item),
       "x1", px+((1.0+pan)*1.1*s),
@@ -381,15 +380,12 @@ static void on_wire_pan_changed(GstElement *element, GParamSpec *arg, gpointer u
  */
 BtWireCanvasItem *bt_wire_canvas_item_new(const BtMainPageMachines *main_page_machines,BtWire *wire,gdouble pos_xs,gdouble pos_ys,gdouble pos_xe,gdouble pos_ye,BtMachineCanvasItem *src_machine_item,BtMachineCanvasItem *dst_machine_item) {
   BtWireCanvasItem *self;
-  BtEditApplication *app;
   GnomeCanvas *canvas;
   BtSong *song;
   BtSetup *setup;
   gdouble w,h;
 
-  g_object_get(G_OBJECT(main_page_machines),"app",&app,"canvas",&canvas,NULL);
-  g_object_get(G_OBJECT(app),"song",&song,NULL);
-  g_object_get(G_OBJECT(song),"setup",&setup,NULL);
+  g_object_get((gpointer)main_page_machines,"canvas",&canvas,NULL);
 
   w=(pos_xe-pos_xs);
   h=(pos_ye-pos_ys);
@@ -398,7 +394,6 @@ BtWireCanvasItem *bt_wire_canvas_item_new(const BtMainPageMachines *main_page_ma
   self=BT_WIRE_CANVAS_ITEM(gnome_canvas_item_new(gnome_canvas_root(canvas),
                             BT_TYPE_WIRE_CANVAS_ITEM,
                             "machines-page",main_page_machines,
-                            "app", app,
                             "wire", wire,
                             "x", pos_xs,
                             "y", pos_ys,
@@ -408,35 +403,43 @@ BtWireCanvasItem *bt_wire_canvas_item_new(const BtMainPageMachines *main_page_ma
                             "dst", dst_machine_item,
                             NULL));
   gnome_canvas_item_lower_to_bottom(GNOME_CANVAS_ITEM(self));
-  g_signal_connect(G_OBJECT(setup),"machine-removed",G_CALLBACK(on_machine_removed),(gpointer)self);
-  
+
+  g_object_get(self->priv->app,"song",&song,NULL);
+  g_object_get(song,"setup",&setup,NULL);
+  g_signal_connect(setup,"machine-removed",G_CALLBACK(on_machine_removed),(gpointer)self);
+
   //GST_INFO("wire canvas item added");
 
   g_object_unref(setup);
   g_object_unref(song);
   g_object_unref(canvas);
-  g_object_unref(app);
   return(self);
 }
 
 //-- methods
 
+/**
+ * bt_wire_show_analyzer_dialog:
+ * @wire: wire to show the dialog for
+ *
+ * Shows the wire analyzer dialog.
+ *
+ * Since: 0.6
+ */
+void bt_wire_show_analyzer_dialog(BtWire *wire) {
+  BtWireCanvasItem *self=BT_WIRE_CANVAS_ITEM(g_object_get_qdata((GObject *)wire,wire_canvas_item_quark));
+
+  show_wire_analyzer_dialog(self);
+}
+
 //-- wrapper
 
 //-- class internals
 
-/* returns a property for the given property_id for this object */
-static void bt_wire_canvas_item_get_property(GObject      *object,
-                               guint         property_id,
-                               GValue       *value,
-                               GParamSpec   *pspec)
-{
+static void bt_wire_canvas_item_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
   BtWireCanvasItem *self = BT_WIRE_CANVAS_ITEM(object);
   return_if_disposed();
   switch (property_id) {
-    case WIRE_CANVAS_ITEM_APP: {
-      g_value_set_object(value, self->priv->app);
-    } break;
     case WIRE_CANVAS_ITEM_MACHINES_PAGE: {
       g_value_set_object(value, self->priv->main_page_machines);
     } break;
@@ -455,27 +458,19 @@ static void bt_wire_canvas_item_get_property(GObject      *object,
     case WIRE_CANVAS_ITEM_DST: {
       g_value_set_object(value, self->priv->dst);
     } break;
+    case WIRE_CANVAS_ITEM_ANALYSIS_DIALOG: {
+      g_value_set_object(value, self->priv->analysis_dialog);
+    } break;
     default: {
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object,property_id,pspec);
     } break;
   }
 }
 
-/* sets the given properties for this object */
-static void bt_wire_canvas_item_set_property(GObject      *object,
-                              guint         property_id,
-                              const GValue *value,
-                              GParamSpec   *pspec)
-{
+static void bt_wire_canvas_item_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
   BtWireCanvasItem *self = BT_WIRE_CANVAS_ITEM(object);
   return_if_disposed();
   switch (property_id) {
-    case WIRE_CANVAS_ITEM_APP: {
-      g_object_try_weak_unref(self->priv->app);
-      self->priv->app=BT_EDIT_APPLICATION(g_value_get_object(value));
-      g_object_try_weak_ref(self->priv->app);
-      //GST_DEBUG("set the app for wire_canvas_item: %p",self->priv->app);
-    } break;
     case WIRE_CANVAS_ITEM_MACHINES_PAGE: {
       g_object_try_weak_unref(self->priv->main_page_machines);
       self->priv->main_page_machines = BT_MAIN_PAGE_MACHINES(g_value_get_object(value));
@@ -487,6 +482,7 @@ static void bt_wire_canvas_item_set_property(GObject      *object,
       self->priv->wire=BT_WIRE(g_value_dup_object(value));
       if(self->priv->wire) {
         //GST_DEBUG("set the wire for wire_canvas_item: %p",self->priv->wire);
+        g_object_set_qdata((GObject *)self->priv->wire,wire_canvas_item_quark,(gpointer)self);
         g_object_get(self->priv->wire,"properties",&(self->priv->properties),NULL);
       }
     } break;
@@ -497,18 +493,24 @@ static void bt_wire_canvas_item_set_property(GObject      *object,
       self->priv->h=g_value_get_double(value);
     } break;
     case WIRE_CANVAS_ITEM_SRC: {
-      g_object_try_unref(self->priv->src);
+      if(self->priv->src) {
+        g_signal_handlers_disconnect_matched(self->priv->src,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_src_position_changed,(gpointer)self);
+        g_object_unref(self->priv->src);
+      }
       self->priv->src=BT_MACHINE_CANVAS_ITEM(g_value_dup_object(value));
       if(self->priv->src) {
-        g_signal_connect(G_OBJECT(self->priv->src),"position-changed",G_CALLBACK(on_wire_position_changed),(gpointer)self);
+        g_signal_connect(self->priv->src,"position-changed",G_CALLBACK(on_wire_src_position_changed),(gpointer)self);
         GST_DEBUG("set the src for wire_canvas_item: %p",self->priv->src);
       }
     } break;
     case WIRE_CANVAS_ITEM_DST: {
-      g_object_try_unref(self->priv->dst);
+      if(self->priv->dst) {
+        g_signal_handlers_disconnect_matched(self->priv->dst,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_dst_position_changed,(gpointer)self);
+        g_object_unref(self->priv->dst);
+      }
       self->priv->dst=BT_MACHINE_CANVAS_ITEM(g_value_dup_object(value));
       if(self->priv->dst) {
-        g_signal_connect(G_OBJECT(self->priv->dst),"position-changed",G_CALLBACK(on_wire_position_changed),(gpointer)self);
+        g_signal_connect(self->priv->dst,"position-changed",G_CALLBACK(on_wire_dst_position_changed),(gpointer)self);
         GST_DEBUG("set the dst for wire_canvas_item: %p",self->priv->dst);
       }
     } break;
@@ -527,36 +529,36 @@ static void bt_wire_canvas_item_dispose(GObject *object) {
 
   GST_DEBUG("!!!! self=%p",self);
   if(self->priv->src) {
-    g_signal_handlers_disconnect_matched(G_OBJECT(self->priv->src),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_position_changed,(gpointer)self);
+    g_signal_handlers_disconnect_matched(self->priv->src,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_src_position_changed,(gpointer)self);
   }
   if(self->priv->dst) {
-    g_signal_handlers_disconnect_matched(G_OBJECT(self->priv->dst),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_position_changed,(gpointer)self);
+    g_signal_handlers_disconnect_matched(self->priv->dst,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_dst_position_changed,(gpointer)self);
   }
-  g_object_get(G_OBJECT(self->priv->app),"song",&song,NULL);
+  g_object_get(self->priv->app,"song",&song,NULL);
   if(song) {
     BtSetup *setup;
 
-    g_object_get(G_OBJECT(song),"setup",&setup,NULL);
-    g_signal_handlers_disconnect_matched(G_OBJECT(setup),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_machine_removed,(gpointer)self);
+    g_object_get(song,"setup",&setup,NULL);
+    g_signal_handlers_disconnect_matched(setup,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_machine_removed,(gpointer)self);
     g_object_unref(setup);
     g_object_unref(song);
   }
   if(self->priv->wire_gain) {
-    g_signal_handlers_disconnect_matched(G_OBJECT(self->priv->wire_gain),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_gain_changed,(gpointer)self);
+    g_signal_handlers_disconnect_matched(self->priv->wire_gain,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_gain_changed,(gpointer)self);
     g_object_unref(self->priv->wire_gain);
   }
   if(self->priv->wire_pan) {
-    g_signal_handlers_disconnect_matched(G_OBJECT(self->priv->wire_pan),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_pan_changed,(gpointer)self);
+    g_signal_handlers_disconnect_matched(self->priv->wire_pan,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_pan_changed,(gpointer)self);
     g_object_unref(self->priv->wire_pan);
   }
-  g_signal_handlers_disconnect_matched(G_OBJECT(self->priv->wire),G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_pan_changed,(gpointer)self);
+  g_signal_handlers_disconnect_matched(self->priv->wire,G_SIGNAL_MATCH_FUNC|G_SIGNAL_MATCH_DATA,0,0,NULL,on_wire_pan_changed,(gpointer)self);
   GST_DEBUG("  signal disconected");
-  
-  g_object_try_weak_unref(self->priv->app);
+
   g_object_try_unref(self->priv->wire);
   g_object_try_unref(self->priv->src);
   g_object_try_unref(self->priv->dst);
   g_object_try_weak_unref(self->priv->main_page_machines);
+  g_object_unref(self->priv->app);
 
   GST_DEBUG("  unrefing done");
 
@@ -565,19 +567,10 @@ static void bt_wire_canvas_item_dispose(GObject *object) {
   }
 
   gtk_widget_destroy(GTK_WIDGET(self->priv->context_menu));
-  g_object_unref(G_OBJECT(self->priv->context_menu));
+  g_object_unref(self->priv->context_menu);
 
   GST_DEBUG("  chaining up");
-  G_OBJECT_CLASS(parent_class)->dispose(object);
-  GST_DEBUG("  done");
-}
-
-static void bt_wire_canvas_item_finalize(GObject *object) {
-  //BtWireCanvasItem *self = BT_WIRE_CANVAS_ITEM(object);
-
-  //GST_DEBUG("!!!! self=%p",self);
-
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(bt_wire_canvas_item_parent_class)->dispose(object);
   GST_DEBUG("  done");
 }
 
@@ -592,8 +585,7 @@ static void bt_wire_canvas_item_realize(GnomeCanvasItem *citem) {
   GnomeCanvasPoints *points;
   gdouble s=MACHINE_VIEW_WIRE_PAD_SIZE,oy=0.25*s,ox=2.5*s,px;
 
-  if(GNOME_CANVAS_ITEM_CLASS(parent_class)->realize)
-    (GNOME_CANVAS_ITEM_CLASS(parent_class)->realize)(citem);
+  GNOME_CANVAS_ITEM_CLASS(bt_wire_canvas_item_parent_class)->realize(citem);
 
   GST_DEBUG("realize for wire occurred, wire=%p : w=%f,h=%f",self->priv->wire,self->priv->w,self->priv->h);
 
@@ -606,7 +598,7 @@ static void bt_wire_canvas_item_realize(GnomeCanvasItem *citem) {
     g_signal_connect(self->priv->wire,"notify::pan",G_CALLBACK(on_wire_pan_changed),(gpointer)self);
   }
 
-  
+
   self->priv->line=gnome_canvas_item_new(GNOME_CANVAS_GROUP(citem),
                            GNOME_TYPE_CANVAS_LINE,
                            "fill-color", "black",
@@ -661,7 +653,7 @@ static void bt_wire_canvas_item_realize(GnomeCanvasItem *citem) {
                            NULL);
   on_gain_changed(self->priv->wire_gain,NULL,(gpointer)self);
 
-  // @todo: check if wire has PAN and set color accordingly
+  // check if wire has PAN and set color accordingly
   if(self->priv->wire_pan) {
     color1="gray16";color2="gray32";
   }
@@ -720,8 +712,9 @@ static void bt_wire_canvas_item_realize(GnomeCanvasItem *citem) {
 
   prop=(gchar *)g_hash_table_lookup(self->priv->properties,"analyzer-shown");
   if(prop && prop[0]=='1' && prop[1]=='\0') {
-    if((self->priv->analysis_dialog=GTK_WIDGET(bt_wire_analysis_dialog_new(self->priv->app,self->priv->wire)))) {
-      g_signal_connect(G_OBJECT(self->priv->analysis_dialog),"destroy",G_CALLBACK(on_wire_analysis_dialog_destroy),(gpointer)self);
+    if((self->priv->analysis_dialog=GTK_WIDGET(bt_signal_analysis_dialog_new(GST_BIN(self->priv->wire))))) {
+      bt_edit_application_attach_child_window(self->priv->app,GTK_WINDOW(self->priv->analysis_dialog));
+      g_signal_connect(self->priv->analysis_dialog,"destroy",G_CALLBACK(on_signal_analysis_dialog_destroy),(gpointer)self);
     }
   }
 
@@ -778,36 +771,36 @@ static gboolean bt_wire_canvas_item_event(GnomeCanvasItem *citem, GdkEvent *even
   }
   /* we don't want the click falling through to the parent canvas item, if we have handled it */
   if(!res) {
-    if(GNOME_CANVAS_ITEM_CLASS(parent_class)->event) {
-      res=(GNOME_CANVAS_ITEM_CLASS(parent_class)->event)(citem,event);
+    if(GNOME_CANVAS_ITEM_CLASS(bt_wire_canvas_item_parent_class)->event) {
+      res=(GNOME_CANVAS_ITEM_CLASS(bt_wire_canvas_item_parent_class)->event)(citem,event);
     }
   }
   return res;
 }
 
-static void bt_wire_canvas_item_init(GTypeInstance *instance, gpointer g_class) {
-  BtWireCanvasItem *self = BT_WIRE_CANVAS_ITEM(instance);
+static void bt_wire_canvas_item_init(BtWireCanvasItem *self) {
   GtkWidget *menu_item;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE(self, BT_TYPE_WIRE_CANVAS_ITEM, BtWireCanvasItemPrivate);
+  GST_DEBUG("!!!! self=%p",self);
+  self->priv->app = bt_edit_application_new();
 
   // generate the context menu
-  self->priv->context_menu=GTK_MENU(g_object_ref_sink(G_OBJECT(gtk_menu_new())));
+  self->priv->context_menu=GTK_MENU(g_object_ref_sink(gtk_menu_new()));
 
   menu_item=gtk_menu_item_new_with_label(_("Disconnect"));
   gtk_menu_shell_append(GTK_MENU_SHELL(self->priv->context_menu),menu_item);
   gtk_widget_show(menu_item);
-  g_signal_connect(G_OBJECT(menu_item),"activate",G_CALLBACK(on_context_menu_disconnect_activate),(gpointer)self);
+  g_signal_connect(menu_item,"activate",G_CALLBACK(on_context_menu_disconnect_activate),(gpointer)self);
 
   menu_item=gtk_separator_menu_item_new();
   gtk_menu_shell_append(GTK_MENU_SHELL(self->priv->context_menu),menu_item);
-  gtk_widget_set_sensitive(menu_item,FALSE);
   gtk_widget_show(menu_item);
 
   menu_item=gtk_menu_item_new_with_label(_("Signal Analysis..."));
   gtk_menu_shell_append(GTK_MENU_SHELL(self->priv->context_menu),menu_item);
   gtk_widget_show(menu_item);
-  g_signal_connect(G_OBJECT(menu_item),"activate",G_CALLBACK(on_context_menu_analysis_activate),(gpointer)self);
+  g_signal_connect(menu_item,"activate",G_CALLBACK(on_context_menu_analysis_activate),(gpointer)self);
 
 }
 
@@ -815,27 +808,17 @@ static void bt_wire_canvas_item_class_init(BtWireCanvasItemClass *klass) {
   GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
   GnomeCanvasItemClass *citem_class=GNOME_CANVAS_ITEM_CLASS(klass);
 
-  parent_class=g_type_class_peek_parent(klass);
+  wire_canvas_item_quark=g_quark_from_static_string("wire-canvas-item");
+
   g_type_class_add_private(klass,sizeof(BtWireCanvasItemPrivate));
 
   gobject_class->set_property = bt_wire_canvas_item_set_property;
   gobject_class->get_property = bt_wire_canvas_item_get_property;
   gobject_class->dispose      = bt_wire_canvas_item_dispose;
-  gobject_class->finalize     = bt_wire_canvas_item_finalize;
 
   citem_class->realize        = bt_wire_canvas_item_realize;
   citem_class->event          = bt_wire_canvas_item_event;
 
-  g_object_class_install_property(gobject_class,WIRE_CANVAS_ITEM_APP,
-                                  g_param_spec_object("app",
-                                     "app contruct prop",
-                                     "Set application object, the window belongs to",
-                                     BT_TYPE_EDIT_APPLICATION, /* object type */
-#ifndef GNOME_CANVAS_BROKEN_PROPERTIES
-                                     G_PARAM_CONSTRUCT_ONLY |
-#endif
-                                     G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
-
   g_object_class_install_property(gobject_class,WIRE_CANVAS_ITEM_MACHINES_PAGE,
                                   g_param_spec_object("machines-page",
                                      "machines-page contruct prop",
@@ -893,24 +876,12 @@ static void bt_wire_canvas_item_class_init(BtWireCanvasItemClass *klass) {
                                      G_PARAM_CONSTRUCT_ONLY |
 #endif
                                      G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_property(gobject_class,WIRE_CANVAS_ITEM_ANALYSIS_DIALOG,
+                                  g_param_spec_object("analysis-dialog",
+                                     "analysis dialog prop",
+                                     "Get the the analysis dialog if shown",
+                                     BT_TYPE_SIGNAL_ANALYSIS_DIALOG,
+                                     G_PARAM_READABLE|G_PARAM_STATIC_STRINGS));
 }
 
-GType bt_wire_canvas_item_get_type(void) {
-  static GType type = 0;
-  if (G_UNLIKELY(type == 0)) {
-    const GTypeInfo info = {
-      sizeof(BtWireCanvasItemClass),
-      NULL, // base_init
-      NULL, // base_finalize
-      (GClassInitFunc)bt_wire_canvas_item_class_init, // class_init
-      NULL, // class_finalize
-      NULL, // class_data
-      sizeof(BtWireCanvasItem),
-      0,   // n_preallocs
-      (GInstanceInitFunc)bt_wire_canvas_item_init, // instance_init
-      NULL // value_table
-    };
-    type = g_type_register_static(GNOME_TYPE_CANVAS_GROUP,"BtWireCanvasItem",&info,0);
-  }
-  return type;
-}
