@@ -1,4 +1,4 @@
-/* $Id: bt-check.c 2304 2009-03-23 16:12:14Z ensonic $
+/* $Id: bt-check.c 3632 2011-09-15 08:00:14Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -28,6 +28,7 @@
 #include "bt-check.h"
 #include <sys/types.h>
 #include <signal.h>
+#include <math.h> 
 
 #ifdef HAVE_SETRLIMIT
   #include <sys/time.h>
@@ -45,24 +46,29 @@ void bt_check_init(void) {
     bt_test_plugin_init,
     VERSION, "LGPL", PACKAGE, PACKAGE_NAME, "http://www.buzztard.org");
 #endif
-  
+
   GST_DEBUG_CATEGORY_INIT(GST_CAT_DEFAULT, "bt-check", 0, "music production environment / unit tests");
   // no ansi color codes in logfiles please
   gst_debug_set_colored(FALSE);
   // use our dummy settings
   bt_settings_set_factory((BtSettingsFactory)bt_test_settings_new);
-  
+
 #ifdef HAVE_SETRLIMIT
-  struct rlimit rl;
-  
-  rl.rlim_max = RLIM_INFINITY;
-  
-  // limit cpu in seconds
-  rl.rlim_cur = 20;
-  if(setrlimit(RLIMIT_CPU,&rl)<0) perror("setrlimit(RLIMIT_CPU) failed");
-  // limit process’s virtual memory in bytes
-  rl.rlim_cur = 1024 * 1024 * 256; // 256 Mb
-  if(setrlimit(RLIMIT_AS,&rl)<0) perror("setrlimit(RLIMIT_AS) failed");
+  // only fork mode limit cpu/mem usage
+  const gchar *mode = g_getenv ("CK_FORK");
+  if (!mode || strcmp(mode, "no")) {
+    struct rlimit rl;
+
+    rl.rlim_max = RLIM_INFINITY;
+    // limit cpu in seconds
+    rl.rlim_cur = 20;
+    if(setrlimit(RLIMIT_CPU,&rl)<0) perror("setrlimit(RLIMIT_CPU) failed");
+    // limit process’s virtual memory in bytes
+    // if we get failing tests and "mmap() failed: Cannot allocate memory"
+    // this limmit needs to be increased
+    rl.rlim_cur = 1024 * 1024 * 1024; // 1024 Mb = 1GB
+    if(setrlimit(RLIMIT_AS,&rl)<0) perror("setrlimit(RLIMIT_AS) failed");
+  }
 #endif
 }
 
@@ -115,7 +121,6 @@ static void check_print_handler(const gchar * const message) {
     FILE *logfile;
     gboolean add_nl=FALSE;
     guint sl=strlen(message);
-    size_t written;
 
     //-- check if messages has no newline
     if((sl>1) && (message[sl-1]!='\n')) add_nl=TRUE;
@@ -126,8 +131,8 @@ static void check_print_handler(const gchar * const message) {
     else if(__check_test && (strstr(message,__check_test)!=NULL) && !__check_method) __check_error_trapped=TRUE;
 
     if((logfile=fopen(__log_file_name, "a")) || (logfile=fopen(__log_file_name, "w"))) {
-      written=fwrite(message,strlen(message),1,logfile);
-      if(add_nl) written=fwrite("\n",1,1,logfile);
+      (void)fwrite(message,sl,1,logfile);
+      if(add_nl) (void)fwrite("\n",1,1,logfile);
       fclose(logfile);
     }
     else { /* Fall back to console output if unable to open file */
@@ -165,6 +170,18 @@ static void check_log_handler(const gchar * const log_domain, const GLogLevelFla
     check_print_handler(msg);
 }
 
+#ifndef GST_DISABLE_GST_DEBUG
+static void check_gst_log_handler(GstDebugCategory *category, GstDebugLevel level, const gchar *file,const gchar *function,gint line,GObject *object,GstDebugMessage *_message,gpointer data) G_GNUC_NO_INSTRUMENT;
+static void check_gst_log_handler(GstDebugCategory *category, GstDebugLevel level, const gchar *file,const gchar *function,gint line,GObject *object,GstDebugMessage *_message,gpointer data) {
+  const gchar *message=gst_debug_message_get(_message);
+
+  //-- check message contents
+  if(__check_method  && (strstr(function,__check_method)!=NULL) && __check_test && (strstr(message,__check_test)!=NULL)) __check_error_trapped=TRUE;
+  else if(__check_method && (strstr(function,__check_method)!=NULL) && !__check_test) __check_error_trapped=TRUE;
+  else if(__check_test && (strstr(message,__check_test)!=NULL) && !__check_method) __check_error_trapped=TRUE;
+}
+#endif
+
 /*
  * setup_log:
  * @argc: command line argument count received in main()
@@ -199,6 +216,7 @@ void setup_log(int argc, char **argv) {
   }
   // reset logfile
   g_unlink(__log_file_name);
+  g_setenv("GST_DEBUG_FILE", __log_file_name, TRUE);
 }
 
 /*
@@ -216,6 +234,10 @@ void setup_log_capture(void) {
   (void)g_log_set_handler("GLib-GObject",G_LOG_LEVEL_MASK|G_LOG_FLAG_FATAL|G_LOG_FLAG_RECURSION, check_log_handler, NULL);
   (void)g_log_set_handler(NULL          ,G_LOG_LEVEL_MASK|G_LOG_FLAG_FATAL|G_LOG_FLAG_RECURSION, check_log_handler, NULL);
   (void)g_set_printerr_handler(check_print_handler);
+
+#ifndef GST_DISABLE_GST_DEBUG
+  gst_debug_add_log_function(check_gst_log_handler, NULL);
+#endif
 }
 
 
@@ -251,9 +273,8 @@ gboolean file_contains_str(gchar *tmp_file_name, gchar *str) {
 }
 
 
-// ttest selection
-gboolean
-_bt_check_run_test_func (const gchar * func_name)
+// runtime test selection via env-var BT_CHECKS
+gboolean _bt_check_run_test_func(const gchar * func_name)
 {
   const gchar *checks;
   gboolean res = FALSE;
@@ -265,10 +286,10 @@ _bt_check_run_test_func (const gchar * func_name)
   if (checks == NULL || *checks == '\0')
     return TRUE;
 
-  /* only run specified functions */
+  /* only run specified functions, regexps would be nice */
   funcs = g_strsplit (checks, ",", -1);
   for (f = funcs; f != NULL && *f != NULL; ++f) {
-    if (strcmp (*f, func_name) == 0) {
+    if (g_pattern_match_simple (*f, func_name)) {
       res = TRUE;
       break;
     }
@@ -282,7 +303,7 @@ _bt_check_run_test_func (const gchar * func_name)
 const gchar *check_get_test_song_path(const gchar *name) {
   static gchar path[2048];
 
-  // TESTSONGDIR gets defined in Makefile.am
+  // TESTSONGDIR gets defined in Makefile.am and is an absolute path
   g_snprintf(path,2047,TESTSONGDIR""G_DIR_SEPARATOR_S"%s",name);
   GST_INFO("build path: '%s'",path);
   return(path);
@@ -760,6 +781,8 @@ void check_setup_test_server(void) {
     /* 32 bit does not work */
     "-screen","0","1024x786x24",
     "-render",/*"color",*/
+    // seems whatever we do, Xvfb won't get xrandr support
+    "-extension", "RANDR",
     NULL
   };
   gboolean found=FALSE,launched=FALSE,trying=TRUE;
@@ -781,7 +804,7 @@ void check_setup_test_server(void) {
       guint pid;
       gchar proc_file[15];
 
-      // read pid
+      // read pid, normal X11 is owned by root
       if((pid_file=fopen(lock_file,"rt"))) {
         gchar *pid_str_res=fgets(pid_str,20,pid_file);
         fclose(pid_file);
@@ -804,7 +827,7 @@ void check_setup_test_server(void) {
     }
 
     // this display is not yet in use
-    if(!g_file_test(display_file, G_FILE_TEST_EXISTS)) {
+    if(found && !g_file_test(display_file, G_FILE_TEST_EXISTS)) {
       // create the testing server
       server_argv[1]=display_name;
       if(!(g_spawn_async(NULL,server_argv,NULL,flags,NULL,NULL,&server_pid,&error))) {
@@ -824,7 +847,6 @@ void check_setup_test_server(void) {
             sleep(1);
           }
         }
-        //sleep(2);
       }
     }
     if(!launched) {
@@ -838,6 +860,12 @@ void check_setup_test_server(void) {
     GST_WARNING("no free display number found");
   }
   else {
+    // we still get a dozen of
+    //   Xlib:  extension "RANDR" missing on display ...
+    // this is a gdk bug:
+    //   https://bugzilla.gnome.org/show_bug.cgi?id=645856
+    //   and it seems to be fixed in Jan.2011
+
     //printf("####### Server started  \"%s\" is up (pid=%d)\n",display_name,server_pid);
     g_setenv("DISPLAY",display_name,TRUE);
     GST_INFO("test server \"%s\" is up (pid=%d)",display_name,server_pid);
@@ -886,17 +914,18 @@ void check_setup_test_display(void) {
       else GST_WARNING("can't get default_screen");
 
       if((test_display = gdk_display_open(display_name))) {
+#if 0
         GdkScreen *test_screen;
 
         if((test_screen = gdk_display_get_default_screen(test_display))) {
           GtkSettings *test_settings;
-          //gdk_threads_enter();
+
           if((test_settings = gtk_settings_get_for_screen(test_screen))) {
             // this just switches to the default theme
             //g_object_set(test_settings,"gtk-theme-name",NULL,NULL);
             /* Is there a bug in gtk+? None of this reliable creates a working
              * theme setup
-              */
+             */
             //g_object_set(test_settings,"gtk-theme-name",theme_name,NULL);
             /* Again this shows no effect */
             //g_object_set(test_settings,"gtk-toolbar-style",GTK_TOOLBAR_ICONS,NULL);
@@ -906,11 +935,10 @@ void check_setup_test_display(void) {
             //g_object_unref(test_settings);
           }
           else GST_WARNING("can't get test_settings on display: \"%s\"",display_name);
-          //gdk_threads_leave();
           //g_object_unref(test_screen);
         }
         else GST_WARNING("can't get test_screen on display: \"%s\"",display_name);
-
+#endif
         gdk_display_manager_set_default_display(display_manager,test_display);
         GST_INFO("display %p,\"%s\" is active",test_display,gdk_display_get_name(test_display));
       }
@@ -971,7 +999,7 @@ void check_shutdown_test_server(void) {
     // wait for the server to finish (use waitpid() here ?)
     while(wait_for_server && wait_count) {
       sleep(1);
-			wait_count--;
+      wait_count--;
     }
     GST_INFO("test server has been shut down");
   }
@@ -983,25 +1011,16 @@ void check_shutdown_test_server(void) {
 
 // gtk+ gui screenshooter
 
-/*
- * check_make_widget_screenshot:
- * @widget: a #GtkWidget to screenshoot
- * @name: filename or NULL.
- *
- * Captures the given widget as png files. The filename is built from tmpdir,
- * application-name, widget-name and give @name.
- */
-void check_make_widget_screenshot(GtkWidget *widget, const gchar *name) {
-  GdkColormap *colormap=gdk_colormap_get_system();
-  GdkWindow *window=widget->window;
-  GdkPixbuf *pixbuf, *scaled_pixbuf;
-  gint wx,wy,ww,wh;
-  gchar *filename;
+#if !GTK_CHECK_VERSION(2,18,0)
+#define gtk_widget_get_visible(widget) GTK_WIDGET_VISIBLE(widget)
+#endif
 
-  g_return_if_fail(GTK_IS_WIDGET(widget));
+static GdkPixbuf *make_screenshot(GtkWidget *widget) {
+  GdkWindow *window=widget->window;
+  gint ww,wh;
 
   // make sure the window gets drawn
-  if(!GTK_WIDGET_VISIBLE(widget)) {
+  if(!gtk_widget_get_visible(widget)) {
     gtk_widget_show_all(widget);
   }
   if(GTK_IS_WINDOW(widget)) {
@@ -1011,6 +1030,13 @@ void check_make_widget_screenshot(GtkWidget *widget, const gchar *name) {
   gtk_widget_queue_draw(widget);
   while(gtk_events_pending()) gtk_main_iteration();
 
+  gdk_window_get_geometry(window,NULL,NULL,&ww,&wh,NULL);
+  return(gdk_pixbuf_get_from_drawable(NULL,window,NULL,0,0,0,0,ww,wh));
+}
+
+static gchar *make_filename(GtkWidget *widget, const gchar *name) {
+  gchar *filename;
+
   if(!name) {
     filename=g_strdup_printf("%s"G_DIR_SEPARATOR_S"%s_%s.png",
       g_get_tmp_dir(),g_get_prgname(),gtk_widget_get_name(widget));
@@ -1019,38 +1045,348 @@ void check_make_widget_screenshot(GtkWidget *widget, const gchar *name) {
     filename=g_strdup_printf("%s"G_DIR_SEPARATOR_S"%s_%s_%s.png",
       g_get_tmp_dir(),g_get_prgname(),gtk_widget_get_name(widget),name);
   }
-  /* @todo: if file exists, make backup */
-  /* @todo: remember all names to build an index.html with a table and all images */
+  return(filename);
+}
 
-  gdk_window_get_geometry(window,&wx,&wy,&ww,&wh,NULL);
-  //pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB,FALSE,8,ww,wh);
-  //gdk_pixbuf_get_from_drawable(pixbuf,window,colormap,0,0,0,0,ww,wh);
-  pixbuf = gdk_pixbuf_get_from_drawable(NULL,window,colormap,0,0,0,0,ww,wh);
-  scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf,ww*0.75, wh*0.75, GDK_INTERP_HYPER);
-  gdk_pixbuf_save(scaled_pixbuf,filename,"png",NULL,NULL);
+static void add_shadow_and_save(cairo_surface_t *image, gchar *filename, gint iw, gint ih) {
+  cairo_surface_t *shadow;
+  cairo_t *cr;
+  gint x,y;
+  gint ss=12;  // shadow size
+  gint so=0;  // shadow offset
+  gint sd=1+(ss-so); // number of draws
+
+  // stupid simple drop shadow code
+  shadow=cairo_image_surface_create(CAIRO_FORMAT_ARGB32,iw+ss,ih+ss);
+  cr=cairo_create(shadow);
+  cairo_rectangle(cr,0.0,0.0,iw+ss,ih+ss);
+  cairo_set_source_rgba(cr,1.0,1.0,1.0,1.0);
+  cairo_fill(cr);
+  // we draw sd*sd-24 and we don't want the gray-level to become >0.7
+  cairo_set_source_rgba(cr,0.0,0.0,0.0,0.7/(sd*sd-24.0));
+  for(x=ss;x>=so;x--) {
+    for(y=ss;y>=so;y--) {
+      // skip corners (this would need to be dependend on shadow size)
+      if((x==ss || x==so) && (y>(ss-2) || y<(so+2)))
+        continue;
+      if((y==ss || y==so) && (x>(ss-2) || x<(so+2)))
+        continue;
+      if((x==(ss-1) || x==(so+1)) && (y>(ss-1) || y<(so+1)))
+        continue;
+      if((y==(ss-1) || y==(so+1)) && (x>(ss-1) || x<(so+1)))
+        continue;
+      cairo_mask_surface(cr,image,x,y);
+    }
+  }
+  cairo_set_source_surface(cr,image,3,3);
+  cairo_paint(cr);
   
-  /* @todo: create diff images
-   * load old image and subtract pixels
-   * - should we use the backup or a reference picture?
-   * - should we return a boolean for detected pixel changes?
-   */
+  cairo_surface_write_to_png(shadow,filename);
+  
+  // cleanup
+  cairo_destroy(cr);
+  cairo_surface_destroy(shadow);
+}
 
-  /* @todo: add shadow to screenshots
-  // see: http://developer.gnome.org/doc/books/WGA/graphics-gdk-pixbuf.html
-  // gdk_drawable_get_image / gdk_pixbuf_get_from_image
+/* @todo: remember all names to build an index.html with a table and all images */
+/* @todo: create diff images
+ * - check if we have a ref image, skip otherwise
+ * - should we have ref images in repo?
+ * - own diff
+ *   - load old image and subtract pixels
+ *   - should we return a boolean for detected pixel changes?
+ * - imagemagic
+ *   - http://www.imagemagick.org/script/compare.php
+ * - need to make a html with table containing
+ *   [ name, ref image, cur image, diff image ]
+ */
+/* @todo: write a html image map for selected regions
+ * - allows xrefs to the related part in the docs
+ * - having tooltips on images wuld be cool
+ */
+/*
+ * check_make_widget_screenshot:
+ * @widget: a #GtkWidget to screenshoot
+ * @name: filename or NULL.
+ *
+ * Captures the given widget as a png file. The filename is built from tmpdir,
+ * application-name, widget-name and give @name.
+ */
+void check_make_widget_screenshot(GtkWidget *widget, const gchar *name) {
+  GdkPixbuf *pixbuf, *scaled_pixbuf;
+  gint iw,ih;
+  gchar *filename;
+  cairo_surface_t *surface;
+  cairo_t *cr;
 
-  // add alpha channel for shadow
-  shadow_pixbuf = gdk_pixbuf_add_alpha(scaled_pixbuf, FALSE, 0,0,0);
-  // enlarge pixbuf
-  gdk_pixbuf_copy_area(const GdkPixbuf *src_pixbuf,
-    int src_x, int src_y, int width, int height,
-    GdkPixbuf *dest_pixbuf, int dest_x, int dest_y);
-  // draw shadow
-  // how to draw into a pixbuf? map to offscreen drawable?
-  */
+  g_return_if_fail(GTK_IS_WIDGET(widget));
 
+  filename=make_filename(widget,name);
+
+  pixbuf = make_screenshot(widget);
+  iw=gdk_pixbuf_get_width(pixbuf)*0.75;
+  ih=gdk_pixbuf_get_height(pixbuf)*0.75;
+  scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf,iw,ih,GDK_INTERP_HYPER);
+  //gdk_pixbuf_save(scaled_pixbuf,filename,"png",NULL,NULL);
+
+  // create a image surface with screenshot
+  surface=cairo_image_surface_create(CAIRO_FORMAT_ARGB32,iw,ih);
+  cr=cairo_create(surface);
+  gdk_cairo_set_source_pixbuf(cr,scaled_pixbuf,0,0);
+  cairo_paint(cr);
+  
+  add_shadow_and_save(surface,filename,iw,ih);
+  
+  // cleanup
+  cairo_destroy(cr);
+  cairo_surface_destroy(surface);
   g_object_unref(pixbuf);
   g_object_unref(scaled_pixbuf);
-  g_object_unref(colormap);
   g_free(filename);
 }
+
+static GtkWidget *find_child(GtkWidget *w, BtCheckWidgetScreenshotRegions *r) {
+  GtkWidget *f=NULL;
+  gboolean match=TRUE;
+  
+  GST_INFO("  trying widget: '%s', '%s', '%s',%d",
+    gtk_widget_get_name(w),(GTK_IS_LABEL(w)?gtk_label_get_text((GtkLabel *)w):NULL),
+    G_OBJECT_TYPE_NAME(w),G_OBJECT_TYPE(w));
+
+  if(r->match&BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_TYPE) {
+    if(!g_type_is_a(G_OBJECT_TYPE(w),r->type)) {
+      GST_DEBUG("    wrong type: %d,%s",G_OBJECT_TYPE(w),G_OBJECT_TYPE_NAME(w));
+      match=FALSE;
+    }
+  }
+  if(r->match&BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_NAME) {
+    if(g_strcmp0(gtk_widget_get_name(w),r->name)) {
+      GST_DEBUG("    wrong name: %s",gtk_widget_get_name(w));
+      match=FALSE;
+    }
+  }
+  if(r->match&BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_LABEL) {
+    if((!GTK_IS_LABEL(w)) || g_strcmp0(gtk_label_get_text((GtkLabel *)w),r->label)) {
+      GST_DEBUG("    not a label or wrong label text");
+      match=FALSE;
+    }
+  }
+  if(match) {
+    f=w;
+  } else if(GTK_IS_CONTAINER(w)) {
+    GList *node,*list=gtk_container_get_children(GTK_CONTAINER(w));
+    GtkWidget *c=NULL;
+
+    GST_INFO("  searching container: '%s'",G_OBJECT_TYPE_NAME(w));
+    for(node=list;node;node=g_list_next(node)) {
+      c=node->data;
+      if((f=find_child(c,r))) {
+        break;
+      }
+    }
+    g_list_free(list);
+  }
+  return(f);
+}
+
+/*
+ * check_make_widget_screenshot_with_highlight:
+ * @widget: a #GtkWidget to screenshoot
+ * @name: filename or NULL.
+ * @regions: array of regions to highlight
+ *
+ * Captures the given widget as png files. The filename is built from tmpdir,
+ * application-name, widget-name and give @name.
+ *
+ * Locates the widgets listed in @regions, highlights their position and draws
+ * callouts to the given position. The callouts are numberd with the array
+ * index. Widgets can be specified by name and/or type.
+ *
+ * The array needs to be terminated with an entry using 
+ * match=%BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_NONE.
+ */
+void check_make_widget_screenshot_with_highlight(GtkWidget *widget, const gchar *name, BtCheckWidgetScreenshotRegions *regions) {
+  GdkPixbuf *pixbuf, *scaled_pixbuf;
+  GtkWidget *child,*parent;
+  GtkAllocation a;
+  gint iw,ih;
+  gchar *filename;
+  cairo_surface_t *surface;
+  cairo_t *cr;
+  cairo_pattern_t *grad;
+  cairo_text_extents_t extent;
+  BtCheckWidgetScreenshotRegions *r;
+  gint c=12,b=(c/4)+(c*3),bl=c,br=c,bt=c,bb=c;
+  gint wx,wy,ww,wh;
+  gfloat lx1=0.0,ly1=0.0,lx2=0.0,ly2=0.0;
+  gfloat cx=0.0,cy=0.0;
+  gfloat f=0.75;
+  gint num=1;
+  gchar num_str[5];
+  gboolean have_space;
+
+  g_return_if_fail(GTK_IS_WIDGET(widget));
+
+  filename=make_filename(widget,name);
+
+  pixbuf=make_screenshot(widget);
+  iw=gdk_pixbuf_get_width(pixbuf)*f;
+  ih=gdk_pixbuf_get_height(pixbuf)*f;
+  scaled_pixbuf=gdk_pixbuf_scale_simple(pixbuf,iw,ih,GDK_INTERP_HYPER);
+  
+  // check borders
+  r=regions;
+  while(r->match!=BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_NONE) {
+    switch(r->pos) {
+      case GTK_POS_LEFT: 
+        bl=b;
+        break;
+      case GTK_POS_RIGHT:
+        br=b;
+        break;
+      case GTK_POS_TOP:
+        bt=b;
+        break;
+      case GTK_POS_BOTTOM:
+        bb=b;
+        break;
+    }
+    r++;
+  }
+
+  // create a image surface with screenshot
+  surface=cairo_image_surface_create(CAIRO_FORMAT_ARGB32,iw+bl+br,ih+bt+bb);
+  cr=cairo_create(surface);
+  gdk_cairo_set_source_pixbuf(cr,scaled_pixbuf,bl,bt);
+  cairo_paint(cr);
+
+  // locate widgets and highlight the areas
+  r=regions;
+  while(r->match!=BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_NONE) {
+    GST_INFO("searching widget: '%s', '%s', '%s',%d",r->name,r->label,g_type_name(r->type),r->type);
+    if((child=find_child(widget,r))) {
+      // for label matches we look for a sensible parent
+      if(r->match&BT_CHECK_WIDGET_SCREENSHOT_REGION_MATCH_LABEL) {
+        // TODO: should the region include gtk_label_get_mnemonic_widget()?
+        if((parent=gtk_widget_get_ancestor(child,GTK_TYPE_BUTTON))) {
+          child=parent;
+        }
+        if((parent=gtk_widget_get_ancestor(child,GTK_TYPE_TOOL_ITEM))) {
+          child=parent;
+        }
+      }
+      // highlight area
+      parent=gtk_widget_get_parent(child);
+      have_space=FALSE;
+      if(parent) {
+        if(GTK_IS_CONTAINER(parent) && gtk_container_get_border_width((GtkContainer *)parent)>0) {
+          have_space=TRUE;
+        }
+        if(GTK_IS_BOX(parent) && gtk_box_get_spacing((GtkBox *)parent)>0) {
+          have_space=TRUE;
+        }
+      }
+      /* it seems that the API docs are lying and the allocations are relative to
+       * the toplevel and not their parent
+       */
+      // @todo: if we don't snapshot a top-level, we need to subtract the offset
+      gtk_widget_get_allocation(child,&a);
+      wx=a.x*f;wy=a.y*f;ww=a.width*f;wh=a.height*f;
+      GST_INFO("found widget at: %d,%d with %dx%d pixel size",a.x,a.y,a.width,a.height);
+
+      if(have_space) {
+        cairo_rectangle(cr,bl+wx,bt+wy,ww,wh);
+      } else {
+        // we make this a bit smaller to separate adjacent widgets
+        cairo_rectangle(cr,bl+wx+1,bt+wy+1,ww-2,wh-2);
+      }
+      cairo_set_source_rgba(cr,1.0,0.0,0.2,0.3);
+      cairo_fill(cr);
+      cairo_set_line_width(cr, 4);
+      // callouts positions
+      switch(r->pos) {
+        case GTK_POS_LEFT:
+          lx1=bl+wx;
+          lx2=b-c;
+          cx=lx2-c;
+          cy=ly1=ly2=bt+wy+wh/2.0;
+          break;
+        case GTK_POS_RIGHT:
+          lx1=bl+wx+ww;
+          lx2=bl+iw+c;
+          cx=lx2+c;
+          cy=ly1=ly2=bt+wy+wh/2.0;
+          break;
+        case GTK_POS_TOP:
+          break;
+        case GTK_POS_BOTTOM:
+          break;
+      }
+      // callout line
+      grad=cairo_pattern_create_linear(lx1,ly1,lx2,ly2);
+      cairo_pattern_add_color_stop_rgba(grad,0.00,1.0,0.0,0.2,0.3);
+      cairo_pattern_add_color_stop_rgba(grad,1.00,1.0,0.0,0.2,1.0);
+      cairo_set_source(cr,grad);
+      cairo_move_to(cr,lx1,ly1);
+      cairo_line_to(cr,lx2,ly2);
+      cairo_stroke(cr);
+      cairo_pattern_destroy(grad);
+      // callout circle
+      cairo_arc(cr,cx,cy,c,0.0,2*M_PI);
+      cairo_set_source_rgba(cr,1.0,0.0,0.2,1.0);
+      cairo_stroke_preserve(cr);
+      cairo_set_source_rgba(cr,1.0,1.0,1.0,1.0);
+      cairo_fill(cr);
+      // callout label
+      sprintf(num_str,"%d",num);
+      cairo_set_source_rgba(cr,0.0,0.0,0.0,1.0); 
+      cairo_set_font_size(cr,10);
+      cairo_text_extents(cr,num_str,&extent);
+       // this is the bottom left pos for the text
+      cairo_move_to(cr,cx-(extent.width/2.0),cy+(extent.height/2.0));
+      cairo_show_text(cr,num_str);
+    } 
+    else {
+      GST_WARNING("widget not found: '%s',%d",r->name,r->type);
+    }
+    r++;
+    num++;
+  }  
+  add_shadow_and_save(surface,filename,iw+bl+br,ih+bt+bb);
+  
+  // cleanup
+  cairo_destroy(cr);
+  cairo_surface_destroy(surface);
+  g_object_unref(pixbuf);
+  g_object_unref(scaled_pixbuf);
+  g_free(filename);
+}
+
+/*
+ * check_send_key:
+ * @widget: a #GtkWidget to send the key even to
+ * @keyval: the key code
+ *
+ * Send a key-press and a key-release of the given key to the @widget.
+ */
+void check_send_key(GtkWidget *widget, guint keyval) {
+  GdkEventKey *e;
+  GdkWindow *w;
+
+  w=gtk_widget_get_window(widget);
+
+  e=(GdkEventKey *)gdk_event_new(GDK_KEY_PRESS);
+  e->window=g_object_ref(w);
+  e->keyval=keyval;
+  gtk_main_do_event((GdkEvent *)e);
+  while(gtk_events_pending()) gtk_main_iteration();
+  gdk_event_free((GdkEvent *)e);
+
+  e=(GdkEventKey *)gdk_event_new(GDK_KEY_RELEASE);
+  e->window=g_object_ref(w);
+  e->keyval=keyval;
+  e->state|=GDK_RELEASE_MASK;
+  gtk_main_do_event((GdkEvent *)e);
+  while(gtk_events_pending()) gtk_main_iteration();
+  gdk_event_free((GdkEvent *)e);
+}
