@@ -1,4 +1,4 @@
-/* $Id: e-network.c 2308 2009-03-25 19:45:26Z ensonic $
+/* $Id: e-network.c 3713 2011-12-01 11:22:55Z ensonic $
  *
  * Buzztard
  * Copyright (C) 2006 Buzztard team <buzztard-devel@lists.sf.net>
@@ -53,10 +53,8 @@ BT_START_TEST(test_btcore_net_static1) {
   // wires
   BtWire *wire, *wire1=NULL;
 
-  /* create a dummy app */
-  app=g_object_new(BT_TYPE_APPLICATION,NULL);
-
-  /* create a new song */
+  /* create app and song */
+  app=bt_test_application_new();
   song=bt_song_new(app);
 
   /* get the song setup and sequence */
@@ -99,7 +97,7 @@ BT_START_TEST(test_btcore_net_static1) {
 
   /* enlarge the sequence */
   g_object_set(sequence,"length",1L,"tracks",1L,NULL);
-  bt_sequence_add_track(sequence,gen1);
+  bt_sequence_add_track(sequence,gen1,-1);
 
   /* try to start playing the song */
   if(bt_song_play(song)) {
@@ -138,10 +136,8 @@ BT_START_TEST(test_btcore_net_static2) {
   // wires
   BtWire *wire1=NULL, *wire2=NULL;
 
-  /* create a dummy app */
-  app=g_object_new(BT_TYPE_APPLICATION,NULL);
-
-  /* create a new song */
+  /* create app and song */
+  app=bt_test_application_new();
   song=bt_song_new(app);
 
   /* get the song setup and sequence */
@@ -176,8 +172,8 @@ BT_START_TEST(test_btcore_net_static2) {
 
   /* enlarge the sequence */
   g_object_set(sequence,"length",1L,"tracks",2L,NULL);
-  bt_sequence_add_track(sequence,gen1);
-  bt_sequence_add_track(sequence,gen2);
+  bt_sequence_add_track(sequence,gen1,-1);
+  bt_sequence_add_track(sequence,gen2,-1);
   mark_point();
 
   /* try to start playing the song */
@@ -221,10 +217,8 @@ BT_START_TEST(test_btcore_net_static3) {
   // wires
   BtWire *wire1=NULL, *wire2=NULL, *wire3=NULL;
 
-  /* create a dummy app */
-  app=g_object_new(BT_TYPE_APPLICATION,NULL);
-
-  /* create a new song */
+  /* create app and song */
+  app=bt_test_application_new();
   song=bt_song_new(app);
 
   /* get the song setup and sequence */
@@ -271,8 +265,8 @@ BT_START_TEST(test_btcore_net_static3) {
 
   /* enlarge the sequence */
   g_object_set(sequence,"length",1L,"tracks",2L,NULL);
-  bt_sequence_add_track(sequence,gen1);
-  bt_sequence_add_track(sequence,gen2);
+  bt_sequence_add_track(sequence,gen1,-1);
+  bt_sequence_add_track(sequence,gen2,-1);
   mark_point();
 
   /* try to start playing the song */
@@ -319,10 +313,8 @@ BT_START_TEST(test_btcore_net_static4) {
   BtWire *wire1=NULL, *wire2=NULL;
   gboolean res;
 
-  /* create a dummy app */
-  app=g_object_new(BT_TYPE_APPLICATION,NULL);
-
-  /* create a new song */
+  /* create app and song */
+  app=bt_test_application_new();
   song=bt_song_new(app);
 
   /* get the song setup and sequence */
@@ -340,26 +332,32 @@ BT_START_TEST(test_btcore_net_static4) {
   gen1=BT_MACHINE(bt_source_machine_new(song,"generator1","audiotestsrc",0,&err));
   fail_unless(gen1!=NULL, NULL);
   fail_unless(err==NULL, NULL);
+  GST_DEBUG_OBJECT(gen1,"added machine: %p,ref_count=%d",gen1,G_OBJECT_REF_COUNT(gen1));
 
   /* try to create generator2 */
   gen2=BT_MACHINE(bt_source_machine_new(song,"generator2","audiotestsrc",0,&err));
   fail_unless(gen2!=NULL, NULL);
   fail_unless(err==NULL, NULL);
+  GST_DEBUG_OBJECT(gen2,"added machine: %p,ref_count=%d",gen2,G_OBJECT_REF_COUNT(gen2));
 
   /* try to create a wire from gen1 to proc */
   wire1=bt_wire_new(song,gen1,sink,&err);
   fail_unless(wire1!=NULL, NULL);
   fail_unless(err==NULL, NULL);
+  GST_DEBUG_OBJECT(gen1,"linked machine: %p,ref_count=%d",gen1,G_OBJECT_REF_COUNT(gen1));
 
   /* try to create a wire from gen2 to proc */
   wire2=bt_wire_new(song,gen2,sink,&err);
   fail_unless(wire2!=NULL, NULL);
   fail_unless(err==NULL, NULL);
+  GST_DEBUG_OBJECT(gen2,"linked machine: %p,ref_count=%d",gen2,G_OBJECT_REF_COUNT(gen2));
 
   /* enlarge the sequence */
   g_object_set(sequence,"length",1L,"tracks",2L,NULL);
-  bt_sequence_add_track(sequence,gen1);
-  bt_sequence_add_track(sequence,gen2);
+  bt_sequence_add_track(sequence,gen1,-1);
+  GST_DEBUG_OBJECT(gen1,"added machine to sequence: %p,ref_count=%d",gen1,G_OBJECT_REF_COUNT(gen1));
+  bt_sequence_add_track(sequence,gen2,-1);
+  GST_DEBUG_OBJECT(gen2,"added machine to sequence: %p,ref_count=%d",gen2,G_OBJECT_REF_COUNT(gen2));
   mark_point();
 
   /* try to start playing the song */
@@ -375,9 +373,11 @@ BT_START_TEST(test_btcore_net_static4) {
   /* remove one machine */
   bt_setup_remove_wire(setup,wire2);
   g_object_unref(wire2);
+  GST_DEBUG_OBJECT(gen2,"unlinked machine: %p,ref_count=%d",gen2,G_OBJECT_REF_COUNT(gen2));
   res=bt_sequence_remove_track_by_machine(sequence,gen2);
   fail_unless(res, NULL);
   bt_setup_remove_machine(setup,gen2);
+  GST_DEBUG_OBJECT(gen2,"removed machine from sequence: %p,ref_count=%d",gen2,G_OBJECT_REF_COUNT(gen2));
   g_object_unref(gen2);
   mark_point();
 
@@ -404,10 +404,9 @@ BT_END_TEST
 
 
 /*
-* check if we can connect a sine machine to a sink, then play and while playing
-* add and connect another sine and a bit later disconnect the first.
+* check if we can connect a src machine to a sink while playing
 */
-BT_START_TEST(test_btcore_net_dynamic1) {
+BT_START_TEST(test_btcore_net_dynamic_add_src) {
   BtApplication *app=NULL;
   GError *err=NULL;
   // the song
@@ -420,10 +419,8 @@ BT_START_TEST(test_btcore_net_dynamic1) {
   // wires
   BtWire *wire1=NULL, *wire2=NULL;
 
-  /* create a dummy app */
-  app=g_object_new(BT_TYPE_APPLICATION,NULL);
-
-  /* create a new song */
+  /* create app and song */
+  app=bt_test_application_new();
   song=bt_song_new(app);
 
   /* get the song setup and sequence */
@@ -447,27 +444,111 @@ BT_START_TEST(test_btcore_net_dynamic1) {
 
   /* enlarge the sequence */
   g_object_set(sequence,"length",10L,"tracks",1L,NULL);
-  bt_sequence_add_track(sequence,gen1);
+  bt_sequence_add_track(sequence,gen1,-1);
 
   /* try to start playing the song */
   if(bt_song_play(song)) {
     mark_point();
     sleep(1);
-    
+    GST_DEBUG("song plays");
+
     /* try to create generator1 */
     gen2=BT_MACHINE(bt_source_machine_new(song,"generator2","audiotestsrc",0,&err));
     fail_unless(gen2!=NULL, NULL);
     fail_unless(err==NULL, NULL);
-  
+    GST_DEBUG("generator added");
+
     /* try to link machines */
     wire2=bt_wire_new(song, gen2, sink,&err);
     fail_unless(wire2!=NULL, NULL);
     fail_unless(err==NULL, NULL);
+    GST_DEBUG("wire added");
 
     sleep(1);
-    
+
+    /* stop the song */
+    bt_song_stop(song);
+  } else {
+    fail("playing of network song failed");
+  }
+
+  g_object_unref(gen1);
+  g_object_unref(gen2);
+  g_object_unref(sink);
+  g_object_unref(wire1);
+  g_object_unref(wire2);
+  g_object_unref(setup);
+  g_object_unref(sequence);
+  g_object_checked_unref(song);
+  g_object_checked_unref(app);
+}
+BT_END_TEST
+
+
+/*
+* check if we can disconnect a src machine from a sink while playing.
+*/
+BT_START_TEST(test_btcore_net_dynamic_rem_src) {
+  BtApplication *app=NULL;
+  GError *err=NULL;
+  // the song
+  BtSong *song=NULL;
+  BtSetup *setup=NULL;
+  BtSequence *sequence=NULL;
+  // machines
+  BtMachine *gen1=NULL,*gen2=NULL;
+  BtMachine *sink=NULL;
+  // wires
+  BtWire *wire1=NULL, *wire2=NULL;
+
+  /* create app and song */
+  app=bt_test_application_new();
+  song=bt_song_new(app);
+
+  /* get the song setup and sequence */
+  g_object_get(song,"setup",&setup,"sequence",&sequence,NULL);
+  fail_unless(setup!=NULL, NULL);
+
+  /* try to create the sink */
+  sink=BT_MACHINE(bt_sink_machine_new(song,"master",&err));
+  fail_unless(sink!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to create generator1 */
+  gen1=BT_MACHINE(bt_source_machine_new(song,"generator1","audiotestsrc",0,&err));
+  fail_unless(gen1!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to link machines */
+  wire1=bt_wire_new(song, gen1, sink,&err);
+  fail_unless(wire1!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to create generator1 */
+  gen2=BT_MACHINE(bt_source_machine_new(song,"generator2","audiotestsrc",0,&err));
+  fail_unless(gen2!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+  GST_DEBUG("generator added");
+
+  /* try to link machines */
+  wire2=bt_wire_new(song, gen2, sink,&err);
+  fail_unless(wire2!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+  GST_DEBUG("wire added");
+
+  /* enlarge the sequence */
+  g_object_set(sequence,"length",10L,"tracks",1L,NULL);
+  bt_sequence_add_track(sequence,gen1,-1);
+
+  /* try to start playing the song */
+  if(bt_song_play(song)) {
+    mark_point();
+    sleep(1);
+    GST_DEBUG("song plays");
+
     /* try to unlink machines */
-    bt_setup_remove_wire(setup,wire1);
+    bt_setup_remove_wire(setup,wire2);
+    GST_DEBUG("wire removed");
 
     sleep(1);
 
@@ -490,11 +571,9 @@ BT_START_TEST(test_btcore_net_dynamic1) {
 BT_END_TEST
 
 /*
- * check if we can connect a sine machine to an effect and this to a sink, then
- * play and while playing add and connect another sine and a bit later
- * disconnect the first.
+ * check if we can connect a processor machine to a src and sink while playing
  */
-BT_START_TEST(test_btcore_net_dynamic2) {
+BT_START_TEST(test_btcore_net_dynamic_add_proc) {
   BtApplication *app=NULL;
   GError *err=NULL;
   // the song
@@ -502,16 +581,14 @@ BT_START_TEST(test_btcore_net_dynamic2) {
   BtSetup *setup=NULL;
   BtSequence *sequence=NULL;
   // machines
-  BtMachine *gen1=NULL,*gen2=NULL;
+  BtMachine *gen1=NULL;
   BtMachine *proc=NULL;
   BtMachine *sink=NULL;
   // wires
   BtWire *wire1=NULL, *wire2=NULL, *wire3=NULL;
 
-  /* create a dummy app */
-  app=g_object_new(BT_TYPE_APPLICATION,NULL);
-
-  /* create a new song */
+  /* create app and song */
+  app=bt_test_application_new();
   song=bt_song_new(app);
 
   /* get the song setup and sequence */
@@ -529,48 +606,40 @@ BT_START_TEST(test_btcore_net_dynamic2) {
   fail_unless(gen1!=NULL, NULL);
   fail_unless(err==NULL, NULL);
 
-  /* try to create a processor with volume */
-  proc=BT_MACHINE(bt_processor_machine_new(song,"processor","volume",0,&err));
-  fail_unless(proc!=NULL, NULL);
-  fail_unless(err==NULL, NULL);
-
-  /* try to create a wire from gen1 to proc */
-  wire1=bt_wire_new(song,gen1,proc,&err);
+  /* try to create a wire from gen1 to sink */
+  wire1=bt_wire_new(song,gen1,sink,&err);
   fail_unless(wire1!=NULL, NULL);
   fail_unless(err==NULL, NULL);
-
-  /* try to create a wire from proc to sink */
-  wire2=bt_wire_new(song,proc,sink,&err);
-  fail_unless(wire2!=NULL, NULL);
-  fail_unless(err==NULL, NULL);
+  GST_DEBUG("wire added");
 
   /* enlarge the sequence */
   g_object_set(sequence,"length",10L,"tracks",1L,NULL);
-  bt_sequence_add_track(sequence,gen1);
+  bt_sequence_add_track(sequence,gen1,-1);
   mark_point();
 
   /* try to start playing the song */
   if(bt_song_play(song)) {
     mark_point();
     sleep(1);
+    GST_DEBUG("song plays");
 
-    /* try to create generator2 */
-    gen2=BT_MACHINE(bt_source_machine_new(song,"generator2","audiotestsrc",0,&err));
-    fail_unless(gen2!=NULL, NULL);
+    /* try to create a processor with volume */
+    proc=BT_MACHINE(bt_processor_machine_new(song,"processor","volume",0,&err));
+    fail_unless(proc!=NULL, NULL);
     fail_unless(err==NULL, NULL);
 
-    /* try to create a wire from gen2 to proc */
-    wire3=bt_wire_new(song,gen2,sink,&err);
+    /* try to create a wire from gen1 to proc */
+    wire2=bt_wire_new(song,gen1,proc,&err);
+    fail_unless(wire2!=NULL, NULL);
+    fail_unless(err==NULL, NULL);
+  
+    /* try to create a wire from proc to sink */
+    wire3=bt_wire_new(song,proc,sink,&err);
     fail_unless(wire3!=NULL, NULL);
     fail_unless(err==NULL, NULL);
 
     sleep(1);
 
-    /* try to unlink machines */
-    bt_setup_remove_wire(setup,wire1);
-
-    sleep(1);
-
     /* stop the song */
     bt_song_stop(song);
   } else {
@@ -578,7 +647,6 @@ BT_START_TEST(test_btcore_net_dynamic2) {
   }
 
   g_object_unref(gen1);
-  g_object_unref(gen2);
   g_object_unref(proc);
   g_object_unref(sink);
   g_object_unref(wire1);
@@ -591,6 +659,104 @@ BT_START_TEST(test_btcore_net_dynamic2) {
 }
 BT_END_TEST
 
+/*
+ * check if we can disconnect a processor machine to a src and sink while playing
+ */
+BT_START_TEST(test_btcore_net_dynamic_rem_proc) {
+  BtApplication *app=NULL;
+  GError *err=NULL;
+  // the song
+  BtSong *song=NULL;
+  BtSetup *setup=NULL;
+  BtSequence *sequence=NULL;
+  // machines
+  BtMachine *gen1=NULL;
+  BtMachine *proc=NULL;
+  BtMachine *sink=NULL;
+  // wires
+  BtWire *wire1=NULL, *wire2=NULL, *wire3=NULL;
+
+  /* create app and song */
+  app=bt_test_application_new();
+  song=bt_song_new(app);
+
+  /* get the song setup and sequence */
+  g_object_get(song,"setup",&setup,"sequence",&sequence,NULL);
+  fail_unless(setup!=NULL, NULL);
+  fail_unless(sequence!=NULL, NULL);
+
+  /* try to create the sink */
+  sink=BT_MACHINE(bt_sink_machine_new(song,"master",&err));
+  fail_unless(sink!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to create generator1 */
+  gen1=BT_MACHINE(bt_source_machine_new(song,"generator1","audiotestsrc",0,&err));
+  fail_unless(gen1!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to create a wire from gen1 to sink */
+  wire1=bt_wire_new(song,gen1,sink,&err);
+  fail_unless(wire1!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+  GST_DEBUG("wire added");
+
+  /* try to create a processor with volume */
+  proc=BT_MACHINE(bt_processor_machine_new(song,"processor","volume",0,&err));
+  fail_unless(proc!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to create a wire from gen1 to proc */
+  wire2=bt_wire_new(song,gen1,proc,&err);
+  fail_unless(wire2!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* try to create a wire from proc to sink */
+  wire3=bt_wire_new(song,proc,sink,&err);
+  fail_unless(wire3!=NULL, NULL);
+  fail_unless(err==NULL, NULL);
+
+  /* enlarge the sequence */
+  g_object_set(sequence,"length",10L,"tracks",1L,NULL);
+  bt_sequence_add_track(sequence,gen1,-1);
+  mark_point();
+
+  /* try to start playing the song */
+  if(bt_song_play(song)) {
+    mark_point();
+    sleep(1);
+    GST_DEBUG("song plays");
+
+    /* try to unlink machines */
+    bt_setup_remove_wire(setup,wire2);
+    GST_DEBUG("wire 2 removed");
+
+    bt_setup_remove_wire(setup,wire3);
+    GST_DEBUG("wire 2 removed");
+
+    sleep(1);
+
+    /* stop the song */
+    bt_song_stop(song);
+  } else {
+    fail("playing song failed");
+  }
+
+  g_object_unref(gen1);
+  g_object_unref(proc);
+  g_object_unref(sink);
+  g_object_unref(wire1);
+  g_object_unref(wire2);
+  g_object_unref(wire3);
+  g_object_unref(setup);
+  g_object_unref(sequence);
+  g_object_checked_unref(song);
+  g_object_checked_unref(app);
+}
+BT_END_TEST
+
+/* should we have variants, where we remove the machines instead of the wires? */
+
 
 TCase *bt_network_example_case(void) {
   TCase *tc = tcase_create("BtNetworkExamples");
@@ -599,8 +765,10 @@ TCase *bt_network_example_case(void) {
   tcase_add_test(tc,test_btcore_net_static2);
   tcase_add_test(tc,test_btcore_net_static3);
   tcase_add_test(tc,test_btcore_net_static4);
-  tcase_add_test(tc,test_btcore_net_dynamic1);
-  tcase_add_test(tc,test_btcore_net_dynamic2);
+  tcase_add_test(tc,test_btcore_net_dynamic_add_src);
+  tcase_add_test(tc,test_btcore_net_dynamic_rem_src);
+  tcase_add_test(tc,test_btcore_net_dynamic_add_proc);
+  tcase_add_test(tc,test_btcore_net_dynamic_rem_proc);
   tcase_add_unchecked_fixture(tc, test_setup, test_teardown);
   return(tc);
 }
