@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: xmlvalid.sh 2032 2008-10-15 13:22:57Z ensonic $
+# $Id: xmlvalid.sh 3716 2011-12-01 15:45:09Z ensonic $
 # test validity of xml files
 
 if [ -z $srcdir ]; then
@@ -19,8 +19,8 @@ xmllint $XML_OPTS $srcdir/../docs/buzztard.xsd
 if [ $? -ne 0 ]; then exit 1; fi
 
 # do schema validation
-#xmllint $XML_OPTS --schema ../docs/buzztard.xsd ./songs/*.xml
-#if [ $? -ne 0 ]; then exit 1; fi
+xmllint $XML_OPTS --schema $srcdir/../docs/buzztard.xsd $E_SONGS
+if [ $? -ne 0 ]; then exit 1; fi
 
 # test the docs
 xmllint $XML_OPTS --xinclude --postvalid $srcdir/../docs/help/bt-edit/C/buzztard-edit.xml
